package zamani.metafy.core;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.URL;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.ArrayList;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JApplet;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyledDocument;

import zamani.metafy.core.data.CameraCalibration;
import zamani.metafy.core.data.DataDealer;
import zamani.metafy.core.gui.AboutDialog;
import zamani.metafy.core.gui.BooleanTableModel;
import zamani.metafy.core.gui.GUIBuilder;
import zamani.metafy.core.gui.GUIModifier;
import zamani.metafy.core.gui.ImageScalingPanel;
import zamani.metafy.core.gui.PopupWindow;
import zamani.metafy.core.utility.ImageLoader;
import zamani.metafy.core.utility.Utility;


/** 
 * Meta-fy JApplet - Deals with Applet initialisation, continuation, 
 *  User actions and stores frequently used variables statically
 * 
 * @author Michael Ferguson
 * 
 */
public class Main extends JApplet implements ActionListener {
	private static final long serialVersionUID = 6046505263609923822L;
		
	public static boolean stateChanged = false;
	
	
	// GUIBuilder
	GUIBuilder guiBuilder = new GUIBuilder();
	
	// Total Files count
	public static int totalFilesCount = 0;
	
	// Popup Window variables
	public static double progressCount = 0.0;
	public static JProgressBar progressImport,validatemetadataProgressBar,transformMetadataProgressBar,  associateFilesProgressBar, loadMetadataProgressBar, exportMetadataProgressBar;
	public static JPanel JProgressBarPanelAssociateFiles, JProgressBarPanel, JProgressBarPanelExportMetadata;

	public static JFrame batchModifyWindow, exportMetadataWindow, associateFilesWindow, popupValidateMetadataProgressWindow, popupImportProgressWindow, popupLoadMetadataProgressWindow, transformAlukaMetadataWindow;

	public static JLabel associateFilesValidationLabel,exportMetadataValidationLabel, associateFilesFromTextFileValidationLabel, transformMetadataValidationLabel;
	
	// Save info Label
	public static JLabel saveInformationLabel = new JLabel();
	
	// Menu variables
	public static JMenuItem batchModifyMI, loadMetadataMI,openFilesMI, addDirectoriesMI, addFilesMI, openDirectoriesMI, importSubMenu, importCoordinates, addCameraCalibrationSettings, exportMI, transformAlukaMetadata, calculateTitles, associateFilesMI, validateMetadata, saveMetadata, saveMetadataAs, aboutMI, removeCameraCalibrationSettings;
	public static JMenu exportSubMenu;
	
	// Camera Calibration lists
	public static ArrayList<String> cameraCalibrationSettings = new ArrayList<String>();
	public static ArrayList<String> cameraCalibrationValues = new ArrayList<String>();

	public static Dimension screenDimensions;

	// Console variables
	public static JTextPane console;
	public static StyledDocument consoleDocument;
	public static SimpleAttributeSet infoAttributes;

	public static SimpleAttributeSet errorAttributes;

	public static SimpleAttributeSet boldAttributes;
	
	// Directory of the selected data set
	public static String datasetDirectory = "";
	
	// Metadata arrays
	public static ArrayList<File> filesInDataset = new ArrayList<File>();
	public static ArrayList<String> listOfFilePathsInDataset = new ArrayList<String>();
	public static ArrayList<String> listOfFileCorrespondingPIDSInDataset = new ArrayList<String>();
	public static ArrayList<String[]> idAndFileTypeList = new ArrayList<String[]>();
	public static ArrayList<ArrayList<String>> metadataFields = new ArrayList<ArrayList<String>>();
	public static ArrayList<ArrayList<String>> metadata = new ArrayList<ArrayList<String>>();
	
	// Table variables
	public static JTable table;
	public static Object[][] data;
	public static BooleanTableModel booleanTableModel;
	
	// Image viewer variables
	public static JScrollPane imageScrollPane;
	public static ImageScalingPanel imagePanel;
	public static ImageLoader imageLoader = new ImageLoader();
	
	// Combobox variables
	public static DefaultComboBoxModel<String> defaultComboBoxModel;
	public static JComboBox<String> fieldComboBox, itemComboBox;
	public static ArrayList<String> fieldLines;
	public static ArrayList<String[]> items;
	public static String[] fieldLabels;
	public static Object previousItemComboBoxValue;
	public static int itemComboBoxSelectedIndex;
	public static int fieldComboBoxSelectedIndex;
	public static JTextField addItemTextField;
	public static int indexOfCreatorInFieldComboBox;
	public static URL fieldNamesURL = Main.class.getResource("settings/fieldNames.txt");

	// Id Prefex variables
	public static JTextField identifierPrefix;
	public static String idPrefix;
	
	// General Automation text fields
	public static JTextField sourceOfData, coverageSpatialCountry, coverageSpatialSitename, coverageSpatialStructure, sharedDrivePathOnServer;
	
	// Basic metadata schema list
	public static ArrayList<String> basicMetadataSchema;

	// Metadata panel variables
	public static JScrollPane metadataScrollPane;	
	public static JPanel metadataPanel;
	JPanel tempPanel;
	
	// File index variables
	public static int fileIndex = 1;
	public static int index = 0;
	
	// Sort variables
	public static String[] tagSortDataset;
	public static JTextField tagSortText;
	public static JComboBox<String> tagComboBox;
	public static String currentSort = "Identity";
	public static JRadioButton identityButton;
	
	public static int length_constant = 0;

	// Save variables
    boolean isSaved = false;
    String lastSaved = "";
    String lastSavedFilePath = "";
    
    // Tabbed Pane
    public static JTabbedPane tabbedPane;
    
	/**
	 * Init method of Applet
	 * Called when applet loaded into browser
	 */
    public void init() {
    	
    	
    	
    	// Configure GUI Look and Feel	
    	guiBuilder.buildGUILookAndFeel();   	
    	
    	// Initialise menu items
    	guiBuilder.buildMenuItems();
    	
    	// Add action listeners to menu items
    	addMenuItemActionListeners();
          	
        try {
            SwingUtilities.invokeAndWait(new Runnable() {
                public void run() {

                	// Find the frame of the Applet
            		Object appletFrame = getParent();
            		
            		// Configure the main frame
            		guiBuilder.configureMainFrame(appletFrame);
					  
					progressImport = validatemetadataProgressBar = transformMetadataProgressBar = associateFilesProgressBar = loadMetadataProgressBar = exportMetadataProgressBar  = new JProgressBar(0, 100);
					JProgressBarPanel = JProgressBarPanelAssociateFiles = JProgressBarPanelExportMetadata = new JPanel();
					
					// Set progress bar colour
					guiBuilder.setProgressBarColour();
					
					// Set progress bar size
					guiBuilder.setProgressBarSize();
					
					// Build Camera Calibration Setttings Array
					DataDealer.buildCameraCalibrationSettings();

					// Store basic metadata schema in list
					basicMetadataSchema = Utility.readTextFile(this.getClass().getResource("settings/basicMetadataSchema.txt"));
					
					// Populate tag sort dataset
					ArrayList<String> tempTagNameslist = Utility.readTextFile(this.getClass().getResource("settings/tagNames.txt")); 
					tagSortDataset = new String[tempTagNameslist.size()];
					tagSortDataset = tempTagNameslist.toArray(tagSortDataset);
					  	
					// Configure the console
					guiBuilder.configureConsole();
					
					// Build application GUI content                
					add(guiBuilder.buildApplicationContent());
					
					// Initialise indexes
					fileIndex = 1;
					index = 0;
					
					// Disable sort combo boxes initially
					tagComboBox.setEnabled(false);  
					tagSortText.setEnabled(false); 
                }
            });
        } catch (Exception e) {
        	e.printStackTrace();
            System.err.println("GUI creation was unsuccessful");
        }
    }
  
	/**
	 * Add Action Listeners to all the menu items
	 */
    public void addMenuItemActionListeners(){
    	saveMetadata.addActionListener(this);
    	saveMetadataAs.addActionListener(this);    	
    	loadMetadataMI.addActionListener(this);
    	openFilesMI.addActionListener(this); 
    	openDirectoriesMI.addActionListener(this);
    	addDirectoriesMI.addActionListener(this); 
    	addFilesMI.addActionListener(this);
        exportMI.addActionListener(this);
        importCoordinates.addActionListener(this);
        addCameraCalibrationSettings.addActionListener(this);
        removeCameraCalibrationSettings.addActionListener(this);
        transformAlukaMetadata.addActionListener(this);
        associateFilesMI.addActionListener(this);
        calculateTitles.addActionListener(this);
        validateMetadata.addActionListener(this);
        aboutMI.addActionListener(this);
        batchModifyMI.addActionListener(this);       
    }
    	
	/**
	 * Action events for button presses
	 * @param  e  ActionEvent used to determine button and radio button presses
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		// Source of action
		JMenuItem source = (JMenuItem)(e.getSource());
		
		if (source.getText()=="Save Metadata"){
			saveMetadata();
		}
		else if (source.getText()=="Save Metadata As..."){
			saveMetadataAs();
		}
		else if (source.getText()=="Export Metadata..."){
			exportMetadata();
		}
		else if (source.getText()=="Load Metadata..."){
			loadMetadata();
		}
		else if (source.getText()=="Open Files..."){
			openFiles();
		}
		else if (source.getText()=="Open Directories..."){
			openDirectories();
	    } 
		else if (source.getText()=="Add Directories..."){
			addDirectories();
		}
		else if (source.getText()=="Add Files..."){
			addFiles();
		}
		else if (source.getText()=="Import Co-ords..."){
			importCoOrds();
		}
		else if (source.getText()=="Import Camera Calibration Settings..."){
			importCameraCalibrationSettings();
		}	
		else if (source.getText()=="Remove Camera Calibration Settings"){
			removeCameraCalibrationSettings();
		}
		else if (source.getText()=="Transform Aluka Metadata..."){
			transformAlukaMetadata();
		}
		else if (source.getText()=="Associate Files..."){
			associateFiles();
		}
		else if (source.getText()=="Calculate Titles"){
			calculateTitles();
		}
		else if (source.getText()=="Validate Metadata"){
			validateMetadata();
		}
		else if (source.getText()=="Batch Modify"){
			batchModify();
		}
		else if(source.getText()=="About"){
			about();
		}
	}
	
	// Action performed Events
    //-------------	
	/**
     * Menu Item on Action Performed: Save metadata 
    */ 
    public void saveMetadata(){
		// Save over previous file
		if (isSaved){
			DataDealer.saveMetadataFile(lastSavedFilePath);
			String timeFormat = new java.text.SimpleDateFormat("dd/MM/yyyy HH:mm:ss").format((new Timestamp(System.currentTimeMillis())));
			String[] timeFormatSplit = timeFormat.split(" ");
			timeFormat = "on "+timeFormatSplit[0]+" at "+timeFormatSplit[1];
			saveInformationLabel.setText("<html><div align=right width="+(int)(screenDimensions.width*0.67)+"px>"+(new File(lastSavedFilePath)).getName()+" (last saved: "+timeFormat+")</div></html>");
	    }
		// Create new .metafy file (prompt for file path)
		else{
			String tempSaveFilePath = Utility.getFileSavePath("Save as .metafy file",this);
			
			// validate filepath
			if ( tempSaveFilePath.endsWith( ".metafy" )){
				lastSavedFilePath = tempSaveFilePath;

				// Create and save new .metadata file
				isSaved = true;
				DataDealer.saveMetadataFile(lastSavedFilePath);
				String timeFormat = new java.text.SimpleDateFormat("dd/MM/yyyy HH:mm:ss").format((new Timestamp(System.currentTimeMillis())));
				String[] timeFormatSplit = timeFormat.split(" ");
				timeFormat = "on "+timeFormatSplit[0]+" at "+timeFormatSplit[1];
				saveInformationLabel.setText("<html><div align=right width="+(int)(screenDimensions.width*0.67)+"px>"+(new File(lastSavedFilePath)).getName()+" (last saved: "+timeFormat+")</div></html>"); 
			}
			else{
			    //Console Feedback ---------------------------------------- 
				try
				{
					consoleDocument.insertString(consoleDocument.getLength(), "Info: ", infoAttributes);
					consoleDocument.insertString(consoleDocument.getLength(), "Error:   ", boldAttributes);
				    consoleDocument.insertString(consoleDocument.getLength(), "Invalid file extension: file extension must be .metafy \n", null);
				}
				catch(Exception ex) { System.out.println(ex); }
			    //---------------------------------------------------------- 
			}
		}
    }
    
	/**
     * Menu Item on Action Performed: Save metadata As.
    */ 
    public void saveMetadataAs(){
		String tempSaveFilePath = Utility.getFileSavePath("Save as .metafy file",this);
		
		// validate filepath
		if ( tempSaveFilePath.endsWith( ".metafy" )){
			lastSavedFilePath = tempSaveFilePath;
			
			// Create and save new .metadata file
			isSaved = true;
			DataDealer.saveMetadataFile(lastSavedFilePath);
			String timeFormat = new java.text.SimpleDateFormat("dd/MM/yyyy HH:mm:ss").format((new Timestamp(System.currentTimeMillis())));
			String[] timeFormatSplit = timeFormat.split(" ");
			timeFormat = "on "+timeFormatSplit[0]+" at "+timeFormatSplit[1];
			saveInformationLabel.setText("<html><div align=right width="+(int)(screenDimensions.width*0.67)+"px>"+(new File(lastSavedFilePath)).getName()+" (last saved: "+timeFormat+")</div></html>");
		}
		else{
		    //Console Feedback ---------------------------------------- 
			try
			{
				consoleDocument.insertString(consoleDocument.getLength(), "Info: ", infoAttributes);
				consoleDocument.insertString(consoleDocument.getLength(), "Error:   ", boldAttributes);
			    consoleDocument.insertString(consoleDocument.getLength(), "Invalid file extension: file extension must be .metafy \n", null);
			}
			catch(Exception ex) { System.out.println(ex); }
		    //---------------------------------------------------------- 
		}
    }
   
	/**
     * Menu Item on Action Performed: Export metadata that has been imported or created. 
    */ 
    public void exportMetadata(){
		identityButton.setSelected(true);
		identityButton.doClick();
				
		PopupWindow.popupExportMetadataWindow(screenDimensions.width, screenDimensions.height);
    }
    
	/**
     * Menu Item on Action Performed: Load metadata that has been serialized in a textfile. 
    */ 
    public void loadMetadata(){
		String pathToMetadataFile = Utility.getLoadPath("Load Metadata (.metafy)", this);
		if (!pathToMetadataFile.equals("")){
			PopupWindow.popupLoadMetadataProgressWindow(screenDimensions.width, screenDimensions.height, pathToMetadataFile);
		}
    }
    
	/**
     * Menu Item on Action Performed: Open Files, previous data is replaced 
    */ 
    public void openFiles(){
    	fileIndex = 1;
    	index = 0;
    	totalFilesCount = 0;
    	
    	DataDealer.importFilesAndDirectories(screenDimensions, false, true, this);
    }
    
	/**
     * Menu Item on Action Performed: Open Directories, previous data is replaced 
    */ 
    public void openDirectories(){
    	fileIndex = 1;
    	index = 0;
    	totalFilesCount = 0;
    	
    	DataDealer.importFilesAndDirectories(screenDimensions, true, true, this);  	
    }
    
	/**
     * Menu Item on Action Performed: Add Directories, previous data is added to 
    */ 
    public void addDirectories(){
		DataDealer.resetMetadataSchemas();
		totalFilesCount = 0;
		DataDealer.importFilesAndDirectories(screenDimensions, true, false, this);
    }
    
	/**
     * Menu Item on Action Performed: Add Files, previous data is added to 
    */ 
    public void addFiles(){
		DataDealer.resetMetadataSchemas();
		totalFilesCount = 0;
		DataDealer.importFilesAndDirectories(screenDimensions, false, false, this);
    }
    
	/**
     * Menu Item on Action Performed: Import Co-ordinate data
    */ 
    public void importCoOrds(){
		String path = Utility.getFilePath("Select a text file containing co-ordinate data (filename latitude longitude)", this);	
		if (!path.equals("")){
			DataDealer.updateCoordinatePositionsFromFile(path);
		}
		
	    //Console Feedback ---------------------------------------- 
		try
		{
			consoleDocument.insertString(consoleDocument.getLength(), "Info: ", infoAttributes);
			consoleDocument.insertString(consoleDocument.getLength(), "Action:   ", boldAttributes);
		    consoleDocument.insertString(consoleDocument.getLength(), "Import Co-ords Complete. \n", null);
		}
		catch(Exception ex) { System.out.println(ex); }
	    //---------------------------------------------------------- 
    }
      
	/**
     * Menu Item on Action Performed: Import Camera Calibration Settings
    */ 
    public void importCameraCalibrationSettings(){
		String path = Utility.getFilePath("Select a text file containing Camera Calibration Settings", this);	
		if (!path.equals("")){
			CameraCalibration cc;
			try {
				cc = new CameraCalibration(path);
							
				int[] selectedTableRows = GUIModifier.getSelectedTableItemIndexes();
				
				for (int index : selectedTableRows){
					System.out.println(index);
					ArrayList<String> itemMetadata = metadata.get(index);
					DataDealer.appendCameraCallibrations(itemMetadata, cc, index);
				}	
				
				// Update selected Metadata Panel
				if (selectedTableRows.length==1){
					GUIModifier.setImageInScrollPane(table, 0,true);
				}
			} catch (ParseException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (Exception e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			
		    //Console Feedback ---------------------------------------- 
			try
			{
				consoleDocument.insertString(consoleDocument.getLength(), "Info: ", infoAttributes);
				consoleDocument.insertString(consoleDocument.getLength(), "Action:   ", boldAttributes);
			    consoleDocument.insertString(consoleDocument.getLength(), "Add Camera Calibration Settings Complete. \n", null);
			}
			catch(Exception ex) { System.out.println(ex); }
		    //---------------------------------------------------------- 
		}
    }
	
    /**
     * Menu Item on Action Performed: Remove Camera Calibration Settings
    */ 
    public void removeCameraCalibrationSettings(){
		int[] selectedTableRows = GUIModifier.getSelectedTableItemIndexes();
		
		for (int index : selectedTableRows){
			ArrayList<String> itemMetadata = metadata.get(index);
			ArrayList<String> itemMetadataFields = metadataFields.get(index);
			DataDealer.removeCameraCalibrationSettings(itemMetadataFields, itemMetadata, index);
			// Update the Status (percentage complete)
			GUIModifier.updateMetadataItemStatus(index);
		}
		
		// Update selected Metadata Panel
		if (selectedTableRows.length==1){
			try {
				GUIModifier.setImageInScrollPane(table, 0,true);
			} catch (Exception e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
		
		if (selectedTableRows.length==0){
			
		}
		else{
		
		    //Console Feedback ---------------------------------------- 
			try
			{
				consoleDocument.insertString(consoleDocument.getLength(), "Info: ", infoAttributes);
				consoleDocument.insertString(consoleDocument.getLength(), "Action:   ", boldAttributes);
			    consoleDocument.insertString(consoleDocument.getLength(), "Remove Camera Calibration Settings Complete. \n", null);
			}
			catch(Exception ex) { System.out.println(ex); }
		    //---------------------------------------------------------- 
		}
    }
    
	/**
     * Menu Item on Action Performed: Transform Aluka Metadata
    */ 
    public void transformAlukaMetadata(){
    	PopupWindow.popupTransformAlukaMetadataWindow(screenDimensions.width, screenDimensions.height);
    }
    
	/**
     * Menu Item on Action Performed: Associate Files
    */ 
    public void associateFiles(){
    	PopupWindow.popupAssociateFilesWindow(screenDimensions.width, screenDimensions.height);
    }
    
	/**
     * Menu Item on Action Performed: Calculate Titles
    */ 
    public void calculateTitles(){
    	DataDealer.setMetadataTitles();
    }
    
	/**
     * Menu Item on Action Performed: Validate Metadata
    */ 
    public void validateMetadata(){
		PopupWindow.popupValidateMetadataProgress(screenDimensions.width, screenDimensions.height);
    }
	/**
     * Menu Item on Action Performed: Batch Modify
    */ 
    public void batchModify(){
    	PopupWindow.popupBatchModify(screenDimensions.width, screenDimensions.height);
    }
    
	/**
     * Menu Item on Action Performed: About Meta-fy
    */ 
    public void about(){
	    JDialog f = new AboutDialog(new JFrame());
	    f.setVisible(true);	
    }
    
    
    //-------------
}