package zamani.metafy.core.data;
import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.FileVisitResult;
import java.nio.file.FileVisitor;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.ArrayList;

import zamani.metafy.core.gui.BackgroundWorker;
import zamani.metafy.core.utility.FileTypes;
import zamani.metafy.core.utility.Utility;

/** 
 * AssociateFilesVisitor class
 * Implements FileVisitor
 * Used to recursively iterate through files in a directory and populate an Associate Filenames list and Associated Files path list
 * 
 * @author Michael Ferguson
 * 
 */
public class AssociateFilesVisitor implements FileVisitor<Path> {

    private Path directoryPath;
    private BackgroundWorker.AssociateFilesWorker associateFilesWorker;
    
	ArrayList<String> associatedFileNameList = new ArrayList<String>();
	ArrayList<String> associatedFilePathList = new ArrayList<String>();
	int associatedFilesCount;
	String fileName, filePath;
	int associatedFilesIndex;
    
    /**
     * AssociateFilesVisitor Constructor
     * @param directoryPath String
     * @param associateFilesWorker BackgroundWorker.AssociateFilesWorker
     * @param associatedFilesCount int
    */
    public AssociateFilesVisitor(String directoryPath, BackgroundWorker.AssociateFilesWorker associateFilesWorker, int associatedFilesCount) {
         this.directoryPath = FileSystems.getDefault().getPath(directoryPath);         
		 this.associateFilesWorker = associateFilesWorker;
		 this.associatedFilesCount = associatedFilesCount;
		 associatedFilesIndex = 0;
    }
    
    /**
     * Process Files in directory
    */
    public void processFiles() throws IOException{
    	Files.walkFileTree(directoryPath, this);
    }
    
    /**
     * Process a single file
     * Populate Associate Filenames list and Associated Files path list
    */
    public void processFile(){
		associatedFileNameList.add(fileName);
		associatedFilePathList.add(filePath);
		
		associatedFilesIndex++;

    	associateFilesWorker.setProgressAmount((int)(100*((double)associatedFilesIndex/(double)associatedFilesCount)));
    }
    
	/**
	 * Get Associated Filenames List
	 * @return associated filename list
	 */
    public ArrayList<String> getAssociatedFileNameList(){
    	return associatedFileNameList;
    }
    
	/**
	 * Get Associated File Path List
	 * @return associated file path list
	 */
    public ArrayList<String> getAssociatedFilePathList(){
    	return associatedFilePathList;
    }

    @Override
    public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs) throws IOException {
        return FileVisitResult.CONTINUE;
    }

    @Override
    public FileVisitResult postVisitDirectory(Path dir, IOException exc) throws IOException {
        return FileVisitResult.CONTINUE;
    }

    @Override
    public FileVisitResult visitFile(Path path, BasicFileAttributes attrs) throws IOException {
		if ((Utility.arrayContains(Utility.getExtension(path.getFileName().toString()),FileTypes.supportedFileTypes))){
			fileName = path.getFileName().toString();
			filePath = path.toFile().getPath();
			processFile();
		}
        return FileVisitResult.CONTINUE;
    }

    @Override
    public FileVisitResult visitFileFailed(Path file, IOException exc) throws IOException {
        return FileVisitResult.CONTINUE;
    }
    
}	