package zamani.metafy.core.data;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Hashtable;

import javax.swing.JFileChooser;
import javax.swing.JTable;
import javax.swing.border.LineBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.text.BadLocationException;

import org.apache.tika.exception.TikaException;
import org.apache.tika.metadata.Metadata;
import org.apache.tika.parser.ParseContext;
import org.apache.tika.parser.Parser;
import org.apache.tika.sax.BodyContentHandler;
import org.xml.sax.SAXException;

import zamani.metafy.core.Main;
import zamani.metafy.core.gui.GUIModifier;
import zamani.metafy.core.gui.PopupWindow;
import zamani.metafy.core.persistence.Serializer;
import zamani.metafy.core.utility.FileTypes;
import zamani.metafy.core.utility.HttpGet;
import zamani.metafy.core.utility.Inflector;
import zamani.metafy.core.utility.ServerPath;
import zamani.metafy.core.utility.Utility;

/** 
 * DataDealer helper class
 * Deals with the majority of data calculations
 * Contains functions relating to file i/o, metadata calculations, reset methods and the core sort function
 * 
 * @author Michael Ferguson
 * 
 */
public class DataDealer {

    /**
     * Non instantiable Constructor
    */
	private DataDealer(){}
	
	// BackgroundWorder instance
	//static BackgroundWorker backgroundWorker = new BackgroundWorker();
	
	/**
	 * Transformed Metadata To Object Array
	 * @param filesInDatasetList ArrayList<File>
	 * @param metadataList ArrayList<ArrayList<String>>
	 * @param idAndFileTypeList ArrayList<String[]>
	 * @return Object[][] containing table data
	 */
	public static Object[][] transformedMetadataToObjectArray(ArrayList<File> filesInDatasetList, ArrayList<ArrayList<String>> metadataList, ArrayList<String[]> idAndFileTypeList){
		Object[][] temp = new Object[filesInDatasetList.size()][filesInDatasetList.size()];
		int index = 0;
		
		for (File file : filesInDatasetList){
			
			int numberFields = 0;
			int fieldsComplete = 0;
			
			temp[index][0] = Boolean.TRUE;
			try {
				temp[index][1] = "<html>&nbsp;&nbsp;"+metadataList.get(index).get(Main.basicMetadataSchema.indexOf("Identifier"))+"<br>"+Utility.indent(10)+"for ["+Utility.getFormattedPath(file.getCanonicalPath())+"]";
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			for (String string : metadataList.get(index)){
				if (string!=null && !string.replaceAll("\\s+","").equals("")){
					fieldsComplete++;
				}		
				numberFields++;		
			}
			int percentageComplete = (int)Math.round(((double)fieldsComplete/(double)numberFields)*100.0);		
			int lengthOfPadding = 4-(((int)percentageComplete)+"").length();
			temp[index][2] = "<html>"+Utility.indent(lengthOfPadding)+percentageComplete+"%";
			idAndFileTypeList.get(index)[3] = percentageComplete+"";
			index++;
		}
		return temp;
	}
	
	/**
	 * Files to Object Array
	 * Perform after files have been imported and there is data
	 * Calculates a Object[][] containing the metadata and sets the table models data to it
	 */
	public static void filesToObjectArray(){
		Object[][] temp = new Object[Main.filesInDataset.size()][3];
		int index = 0;
		
		for (File file : Main.filesInDataset){
			
			int numberFields = 0;
			int fieldsComplete = 0;
			
			temp[index][0] = Boolean.TRUE;
			try {
				temp[index][1] = "<html>&nbsp;&nbsp;"+Main.metadata.get(index).get(Main.basicMetadataSchema.indexOf("Identifier"))+"<br>"+Utility.indent(10)+"for ["+Utility.getFormattedPath(file.getCanonicalPath())+"]";
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			for (String string : Main.metadata.get(index)){
				if (string!=null && !string.replaceAll("\\s+","").equals("")){
					fieldsComplete++;
				}		
				numberFields++;		
			}
			int percentageComplete = (int)Math.round(((double)fieldsComplete/(double)numberFields)*100.0);		
			int lengthOfPadding = 4-(((int)percentageComplete)+"").length();
			temp[index][2] = "<html>"+Utility.indent(lengthOfPadding)+percentageComplete+"%";
			
			Main.idAndFileTypeList.get(index)[3] = percentageComplete+"";
			index++;
		}
		Main.data = temp;
		Main.booleanTableModel.data = temp;
	}

	
	/**
	 * Save Metadata File
	 * Create a metadata file containing the sessions data lists in serialized form
	 * @param filePath String
	 */
	public static void saveMetadataFile(String filePath){
		String[] automationFields = new String[]{Main.identifierPrefix.getText(), Main.sourceOfData.getText(), Main.coverageSpatialCountry.getText(), Main.coverageSpatialSitename.getText(), Main.coverageSpatialStructure.getText(), "", "", "", ""};
		
		Serializer serializer = new Serializer(filePath);
		serializer.serializeMetadata(Main.idAndFileTypeList, Main.metadataFields, Main.metadata, Main.filesInDataset, Main.data, Main.listOfFilePathsInDataset, Main.listOfFileCorrespondingPIDSInDataset, Main.basicMetadataSchema, automationFields, Main.fileIndex);
	}
	
	// Title calculations
	//-----------
	
	/**
	 * Set the calculated metadata titles
	 */
	public static void setMetadataTitles(){
	
		// Title build formula:
		// Title = Type 'of' CoverageSpatialStructure 'in' CoverageSpatialName ',' CoverageSpatialCountry
		int index = 0;
		for (ArrayList<String> recordMetadataFields : Main.metadataFields){
	
			String recordType = Main.metadata.get(index).get(recordMetadataFields.indexOf("Type"));
			
			String recordCoverageSpatialName;
			int indexOfCoverageSpatialName = recordMetadataFields.indexOf("Coverage Spatial Name");
			if (indexOfCoverageSpatialName>-1){
				recordCoverageSpatialName = Main.metadata.get(index).get(indexOfCoverageSpatialName);
			}
			else{
				recordCoverageSpatialName = "";
			}
			
			String recordCoverageSpatialCountry;
			int indexOfCoverageSpatialCountry = recordMetadataFields.indexOf("Coverage Spatial Country");
			if (indexOfCoverageSpatialCountry>-1){
				recordCoverageSpatialCountry = Main.metadata.get(index).get(indexOfCoverageSpatialCountry);
			}
			else{
				recordCoverageSpatialCountry = "";
			}
			
			String recordCoverageSpatialStructure;
			int indexOfCoverageSpatialStructure = recordMetadataFields.indexOf("Coverage Spatial Structure");
			if (indexOfCoverageSpatialStructure>-1){
				recordCoverageSpatialStructure = Main.metadata.get(index).get(indexOfCoverageSpatialStructure);
			}
			else{
				recordCoverageSpatialStructure = "";
			}
			
			String recordCoverageSpatialPart;
			int indexOfSpatialPart = recordMetadataFields.indexOf("Coverage Spatial Part");
			if (indexOfSpatialPart>-1){
				recordCoverageSpatialPart = Main.metadata.get(index).get(indexOfSpatialPart);
			}
			else{
				recordCoverageSpatialPart = "";
			}
					
			String recordCalculatedTitle = calculateMetadataRecordTitle(recordType, recordCoverageSpatialStructure, recordCoverageSpatialName, recordCoverageSpatialCountry, recordCoverageSpatialPart);
	
			if (!recordCalculatedTitle.equals("")){
				Main.metadata.get(index).set(recordMetadataFields.indexOf("Title"), recordCalculatedTitle);
			}
			
			index++;
		}
	}
	
	/**
	 * Calculate a specified metadata records title
	 * @param recordType String
	 * @param recordCoverageSpatialStructure String
	 * @param recordCoverageSpatialName String
	 * @param recordCoverageSpatialCountry String
	 * @param recordCoverageSpatialPart String
	 * @return Calculated metadata record title
	 */
	public static String calculateMetadataRecordTitle(String recordType, String recordCoverageSpatialStructure, String recordCoverageSpatialName, String recordCoverageSpatialCountry, String recordCoverageSpatialPart){
		boolean useCalculatedTitle = true;
		
		String calculatedTitle = "";
		if (recordCoverageSpatialName!=null && recordCoverageSpatialCountry!=null){
			if (recordCoverageSpatialName.equals("") || recordCoverageSpatialCountry.equals("")){
				useCalculatedTitle = false;
			}
		}
		
		if (recordType!=null){
			if (!recordType.equals("")){
				String recordTypeFormattedAndSingularized = Inflector.getInstance().singularize(Utility.capitalize(recordType.toLowerCase()));
				calculatedTitle += recordTypeFormattedAndSingularized + " of ";
			}
		}
		
		boolean partOrStruct = false;
		
		if (recordCoverageSpatialPart!=null){
			if (!recordCoverageSpatialPart.equals("")){
				calculatedTitle += recordCoverageSpatialPart;
				partOrStruct = true;
			}
		}
		
		if (recordCoverageSpatialStructure!=null){
			if (!recordCoverageSpatialStructure.equals("")){
				calculatedTitle += " within the " + recordCoverageSpatialStructure;
				partOrStruct = true;
			}
		}
		
		if (recordCoverageSpatialName!=null){
			if (!recordCoverageSpatialName.equals("")){
				if (partOrStruct){
					calculatedTitle += " in " + recordCoverageSpatialName + ", ";
				}
				else{
					calculatedTitle += recordCoverageSpatialName + ", ";
				}
			}
		}
	
		if (recordCoverageSpatialCountry!=null){
			if (!recordCoverageSpatialCountry.equals("")){
				calculatedTitle += recordCoverageSpatialCountry;
			}
		}
		
		if (!useCalculatedTitle){
			calculatedTitle = "";
		}
		
		return calculatedTitle;
	}
	
	// Camera calibration settings
	//-----------
	
	/**
	 * Append Camera Callibration Settings
	 * @param itemMetadata ArrayList<String>
	 * @param cc CameraCalibration
	 * @param index int
	 */
	public static void appendCameraCallibrations(ArrayList<String> itemMetadata, CameraCalibration cc, int index){
		String fileExtension = Main.idAndFileTypeList.get(index)[1];
		String cameraModel = Main.idAndFileTypeList.get(index)[4];
		
		// If the item is has been accepted as a Calibration Image Type
		if (Utility.arrayContains(fileExtension, FileTypes.cameraCalibrationImageType)){
			// Append Camera Calibration Settings
			appendCameraCalibrationSettingsToItem(index);
			// Set the camera calibration field values
			setItemsCameraCalibrationSettings(itemMetadata, cc, index, cameraModel);
			// Update the Status (percentage complete)
			GUIModifier.updateMetadataItemStatus(index);
		}		
	}
	
	/**
	 * Remove Camera Callibration Settings
	 * @param itemMetadataFields ArrayList<String>
	 * @param itemMetadata ArrayList<String>
	 * @param indexToRemove int
	 */
	public static void removeCameraCalibrationSettings(ArrayList<String> itemMetadataFields, ArrayList<String> itemMetadata, int indexToRemove){
		ArrayList<String> tempItemMetadataFields = itemMetadataFields;
		
		// Only remove if the list contains Camera Calibration Settings
		if (listContainsCameraCalibrationSettings(itemMetadataFields)){
			int cameraCalibrationSettingsStartIndex = itemMetadataFields.indexOf("Calibration Date");
			
			if (cameraCalibrationSettingsStartIndex>-1){
				for (int index=0; index<16;index++){
					tempItemMetadataFields.remove(cameraCalibrationSettingsStartIndex);
				}
				
				// Set the new metadata fields (with camera calibration settings removed)
				Main.metadataFields.set(indexToRemove, tempItemMetadataFields);
				
				// Set the new metadata (Remove the Camera calibration values)
				Main.metadata.set(indexToRemove, removeCameraCalibrationValues(itemMetadata, cameraCalibrationSettingsStartIndex));
			}	
			
		    //Console Feedback ---------------------------------------- 
			try
			{
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Info: ", Main.infoAttributes);
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Action:   ", Main.boldAttributes);
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Camera Calibration Settings removed: \n", null);
			}
			catch(Exception ex) { System.out.println(ex); }
		    //---------------------------------------------------------- 
		}
		else{
		    //Console Feedback ---------------------------------------- 
			try
			{
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Info: ", Main.infoAttributes);
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Action:   ", Main.boldAttributes);
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "No Camera Calibration Settings to remove: \n", null);
			}
			catch(Exception ex) { System.out.println(ex); }
		    //---------------------------------------------------------- 
		}
	}
	
	/**
	 * Remove Camera Callibration Values
	 * @param itemMetadata ArrayList<String>
	 * @param startIndex int start index of camera calibration settings in metadata record
	 */
	public static ArrayList<String> removeCameraCalibrationValues(ArrayList<String> itemMetadata, int startIndex){
		ArrayList<String> tempItemMetadata = itemMetadata;
		for (int index=0; index<16;index++){
			tempItemMetadata.remove(startIndex);
		}
		return tempItemMetadata;
	}
	
	/**
	 * Check to see if list contains camera calibration settings
	 * @param itemMetadataFields ArrayList<String>
	 * @return true if list contains camera calibration settings, else false
	 */
	public static boolean listContainsCameraCalibrationSettings(ArrayList<String> itemMetadataFields){
		for (String metadataField : itemMetadataFields){
			if (metadataField.equals("Calibration Date")){
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Append Camera Calibration Settings to Item
	 * @param index int index of item
	 */
	public static void appendCameraCalibrationSettingsToItem(int index){
		// Add Camera Calibration Settings to the Items metadata fields
		//------------------------------------------------------------
		ArrayList<String> itemMetadataFields = Main.metadataFields.get(index);
			
		// Ensure Item does not already have camera calibration settings
		if (!listContainsCameraCalibrationSettings(itemMetadataFields)){
			for (String d : Main.cameraCalibrationSettings){
				itemMetadataFields.add(d);
			}
			
			// Modify the existing items fields
			Main.metadataFields.set(index, itemMetadataFields);
			//------------------------------------------------------------
			
			// Add 'null' Camera Calibration values to the items metadata 
			//------------------------------------------------------------
			ArrayList<String> itemMetadataValues = Main.metadata.get(index);
			
			for (String d : Main.cameraCalibrationValues){
				itemMetadataValues.add(d);
			}
		
			// Modify the existing items fields
			Main.metadata.set(index, itemMetadataValues);
			//------------------------------------------------------------
			
		    //Console Feedback ---------------------------------------- 
			try
			{
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Info: ", Main.infoAttributes);
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Camera Calibration Settings added to object. \n", null);
			}
			catch(Exception ex) { System.out.println(ex); }
		    //---------------------------------------------------------- 
		}	
		else{
		    //Console Feedback ---------------------------------------- 
			try
			{
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Info: ", Main.infoAttributes);
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Object already contains Camera Calibration Settings. \n", null);
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Object Camera Calibration Settings replaced. \n", null);
			}
			catch(Exception ex) { System.out.println(ex); }
		    //---------------------------------------------------------- 
		}
	}
	
	/**
	 * Append Camera Calibration Settings to Item
	 * @param itemMetadata ArrayList<String> items metadata
	 * @param cc CameraCalibration camera calibration object containing camera calibration settings
	 * @param index int index of item
	 * @param cameraModel String camera model
	 */
	public static void setItemsCameraCalibrationSettings(ArrayList<String> itemMetadata, CameraCalibration cc, int index, String cameraModel){
		// Add the calibration settings
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Date"), cc.date);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Camera"), cameraModel);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Decentering P1"), cc.decenteringP1);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Decentering P2"), cc.decenteringP2);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Distortion Affine B1"), cc.distortionAffineB1);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Distortion Affine B2"), cc.distortionAffineB2);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Distortion Radial K1"), cc.distortionRadialK1);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Distortion Radial K2"), cc.distortionRadialK2);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Distortion Radial K3"), cc.distortionRadialK3);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Length Focal"), cc.lengthFocal);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Pixel Size X"), cc.pixelSizeX);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Pixel Size Y"), cc.pixelSizeY);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Principal Point X"), cc.PrincipalPointX);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Principal Point Y"), cc.PrincipalPointY);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Size X"), cc.sizeX);
		Main.metadata.get(index).set(Main.metadataFields.get(index).indexOf("Calibration Size Y"), cc.sizeY);
	}
	
	/**
	 * Build Camera Calibration Settings
	 */
	public static void buildCameraCalibrationSettings(){
		Main.cameraCalibrationSettings.add("Calibration Date");
		Main.cameraCalibrationSettings.add("Calibration Camera");
		Main.cameraCalibrationSettings.add("Calibration Decentering P1");
		Main.cameraCalibrationSettings.add("Calibration Decentering P2");
		Main.cameraCalibrationSettings.add("Calibration Distortion Affine B1");
		Main.cameraCalibrationSettings.add("Calibration Distortion Affine B2");
		Main.cameraCalibrationSettings.add("Calibration Distortion Radial K1");
		Main.cameraCalibrationSettings.add("Calibration Distortion Radial K2");
		Main.cameraCalibrationSettings.add("Calibration Distortion Radial K3");
		Main.cameraCalibrationSettings.add("Calibration Length Focal");
		Main.cameraCalibrationSettings.add("Calibration Pixel Size X");
		Main.cameraCalibrationSettings.add("Calibration Pixel Size Y");
		Main.cameraCalibrationSettings.add("Calibration Principal Point X");
		Main.cameraCalibrationSettings.add("Calibration Principal Point Y");
		Main.cameraCalibrationSettings.add("Calibration Size X");
		Main.cameraCalibrationSettings.add("Calibration Size Y");
		
		for (int i=0; i<16;i++){
			Main.cameraCalibrationValues.add("null");
		}
	}
	//-----------
	
	
	/**
	 * Update Co-ordinate Positions from textfile
	 * @param filePath String filepath to textfile containing position triplet (filename latitude longitude), separated by tabs, first line of file ignored
	 */
	@SuppressWarnings("static-access")

	public static void updateCoordinatePositionsFromFile(String filePath){
		
		FileCoordinatesList fcl = new FileCoordinatesList(filePath);
		ArrayList<FileCoordinates> listOfFileCoordinates = fcl.listOfFileCoordinates;
		
		// Loop through Metadata Items
		for (int i=0; i<Main.metadataFields.size(); i++){
			// Get Filename
			String fileName = Main.filesInDataset.get(i).getName();
			
			// Search to find index of filename
			int index = fcl.fileNameSearch(fileName);
			
			
			// Filename found
			if (index > -1){

				//only do this if filename matches
				int posLatIndex = Main.metadataFields.get(i).indexOf("Position Latitude");
				int posLongIndex = Main.metadataFields.get(i).indexOf("Position Longitude");
				int posEllipseIndex = Main.metadataFields.get(i).indexOf("Position Ellipse");
				
				// Set Latitude and Longitude values
				Main.metadata.get(i).set(posLatIndex, listOfFileCoordinates.get(index).positionLatitude);
				Main.metadata.get(i).set(posLongIndex, listOfFileCoordinates.get(index).positionLongitude);
				Main.metadata.get(i).set(posEllipseIndex, "WGS84");
				
				// Update the Status (percentage complete)
				GUIModifier.updateMetadataItemStatus(i);
				
			    //Console Feedback ---------------------------------------- 
				try
				{
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Info: ", Main.infoAttributes);
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Co-ordinate Data Added:   ", Main.boldAttributes);
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Metadata Updatated for: "+listOfFileCoordinates.get(index).fileName+"\n", null);
				}
				catch(Exception e) { System.out.println(e); }
			    //---------------------------------------------------------- 
			}
		}	
	}
	
	/**
	 * Import Files and Directories
	 * Handle importing of files (Open files, directories and Add files, directories)
	 * @param screenDimensions Dimension
	 * @param isDirectory boolean
	 * @param reset boolean set to true to reset previously added metadata
	 * @param component Component
	 */
	public static void importFilesAndDirectories(Dimension screenDimensions, boolean isDirectory, boolean reset, Component component){
		
		// Ensure that identifier prefix is valid
		String identifierPrefixString = Main.identifierPrefix.getText();
		if (identifierPrefixString.equals("")){
		    //Console Feedback ---------------------------------------- 
			try
			{
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Error: ", Main.errorAttributes);
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Action:   ", Main.boldAttributes);
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Load/Add Files/Directory. \n", null);
				Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "No Identifier Prefix set \n", null);
			}
			catch(Exception ex) { System.out.println(ex); }
		    //---------------------------------------------------------- 
		}
		else{
	
			if (identifierPrefixString.length()-8>50){
			    //Console Feedback ---------------------------------------- 
				try
				{
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Error: ", Main.errorAttributes);
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Action:   ", Main.boldAttributes);
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Load/Add Files/Directory. \n", null);
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Identifier Prefix is too long (Cannot be longer than 50 characters) \n", null);
				}
				catch(Exception ex) { System.out.println(ex); }
			    //---------------------------------------------------------- 
			}
			else if ((identifierPrefixString+":1").matches("^([A-Za-z0-9]|-|\\.)+:(([A-Za-z0-9])|-|\\.|~|_|(%[0-9A-F]{2}))+$")) {
				Main.datasetDirectory = "";
				
			    JFileChooser chooser = new JFileChooser(); 
			    chooser.setCurrentDirectory(new java.io.File("."));
			    chooser.setDialogTitle("Select dataset");
			    if (isDirectory){
			    	chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			    }
			    else{
			    	chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
			    }
			    chooser.setAcceptAllFileFilterUsed(false);   									// disable the "All files" option.
			    chooser.setMultiSelectionEnabled(true);
			    
			    File[] files = null;
			    
			    String consoleMessage = "";
			    if (chooser.showOpenDialog(component) == JFileChooser.APPROVE_OPTION) { 
			    	files = chooser.getSelectedFiles();
			    	
			    	for (File dir : files){
			    		Main.datasetDirectory += "("+dir.toString()+") ";
			    	}
			    	consoleMessage = "Dataset Directory: "+ Main.datasetDirectory+"\n";  
			    }
			    else {
			    	consoleMessage = "No dataset Directory selected \n";
			    }
			    
			    // Feedback
			    try {
			    	Main.consoleDocument.insertString(Main.consoleDocument.getLength(), consoleMessage, Main.boldAttributes);
				} catch (BadLocationException e2) {
					// TODO Auto-generated catch block
					e2.printStackTrace();
				}
			    
			    if (!Main.datasetDirectory.equals("")){
			    	// Reset metadata (if there is any previous metadata & 'Opened' not Added)
			    	if (reset){
			    		resetMetadata();
			    	}
			    	
					int indexOfCreatorBasicSchema = Main.basicMetadataSchema.indexOf("Creator");  
					Main.indexOfCreatorInFieldComboBox = -1;
					int tempIndex = 0;
					for (String s : Main.fieldLabels){
						if (s.equals("Creator")){
							Main.indexOfCreatorInFieldComboBox=tempIndex;
						}
						tempIndex++;
					}
				
					for (int i=0; i<Main.items.get(Main.indexOfCreatorInFieldComboBox).length-1;i++){
						Main.basicMetadataSchema.add(indexOfCreatorBasicSchema,"Creator");
					}	
					//------------------------------
					
					PopupWindow.popupImportProgress(screenDimensions.width, screenDimensions.height, files);
			    }
			}
			else{
			    //Console Feedback ---------------------------------------- 
				try
				{
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Error: ", Main.errorAttributes);
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Action:   ", Main.boldAttributes);
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Load/Add Files/Directory. \n", null);
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Identifier Prefix syntax error (Allowed character: a-z, A-Z, '.' and '-') \n", null);
				}
				catch(Exception ex) { System.out.println(ex); }
			    //---------------------------------------------------------- 
			}
		}
	}
	
	/**
	 * Get Files in Directory Count
	 * Does a HTTP GET request to quickly determine the number of files in the directory
	 * Reverts to standard directory counting if Response is not 200
	 * Increments Main.totalFilesCount
	 * @param dir File directory to count files in
	 */
	public static void getFilesInDirectoryCount(File dir) throws IOException, SAXException, TikaException {	
		// Directory
		if (dir.isDirectory()){
			// Calculate the Associated Directory path on the server
			String associatedDirServer = ServerPath.calculateServerDirectoryPath(dir.getPath(), Main.sharedDrivePathOnServer.getText());
			
			int temporaryDirectoryCount = -1;
			
			// HTTP GET directory count
			try {
				temporaryDirectoryCount = HttpGet.getDirectoryCount("http://nala.cs.uct.ac.za/zamaniProject_archive/inc/getDirectoryCount.php?dir="+associatedDirServer);
			} catch (Exception e) {
				temporaryDirectoryCount = -1;
			}
			
			
			// Response of 200 
			if (temporaryDirectoryCount>0){
				Main.totalFilesCount += temporaryDirectoryCount;
			}
			// Calculate the directory count ourself
			else{
				
				DirectoryCounter directoryCounter = new DirectoryCounter(dir.getPath(), FileTypes.supportedFileTypes);
				directoryCounter.countRecursively();
				Main.totalFilesCount += directoryCounter.getFilesInDirectoryCount();
			}
		}
		// File
		else{
			Main.totalFilesCount++;
		}
	}
	
	/**
	 * Extract Camera Model from File
	 * Can be extended to extract a large variety of metadata from the file
	 * @param parser Parser
	 * @param file File
	 * @throws IOException
	 * @throws SAXException
	 * @throws TikaException
	 */
	public static Dictionary<String, String> extractFromFile(final Parser parser, File file) throws IOException, SAXException, TikaException {
	
		  Dictionary<String, String> metadataExtractedDictionary = new Hashtable<String, String>(); 
			
		  BodyContentHandler handler = new BodyContentHandler(999);
		  Metadata metadata = new Metadata();
		  InputStream content = new FileInputStream(file);  
		  parser.parse(content, handler, metadata, new ParseContext());
	
		  // camera
		  String cameraModel = metadata.get("tiff:Model");
		  metadataExtractedDictionary.put("Camera Model", cameraModel);
	
		  return metadataExtractedDictionary;
		}

    // Field methods
    //---------
	
	/**
	 * Load Field labels and items from fieldNames.txt settings file used in Automation ComboBox's
	 */
    public static void LoadFields(){
          if (Main.fieldLines==null){
        	  Main.fieldLines = Utility.readTextFile(Main.fieldNamesURL);
          
        	  Main.fieldLabels = new String[Main.fieldLines.size()];
        	  Main.items = new ArrayList<String[]>();
	          
	          int index = 0;
	          for (String s : Main.fieldLines){
	        	  if (s.contains("_SPLIT_")){
	        		  String[] split = s.split("_SPLIT_");
	        		  Main.fieldLabels[index] = split[0];
	        		  
	        		  String[] split2 = split[1].split("_BREAK_");
	        		  Main.items.add(split2);
	        	  }
	        	  else{
	        		  Main.fieldLabels[index] = s;
	        		  String[] temp = null;
	        		  Main.items.add(temp);
	        	  }
	        	  index++;
	          }
          }
    }
    
	/**
	 * Modify Field Name
	 * Modify an existing field by writing the update to the fieldNames.txt settings file
	 */
    public static void modifyFieldName(){

        try {
            BufferedWriter bw = new BufferedWriter(new FileWriter(new File("settings/fieldNames.txt"), false));

	    	String tempString = "";
	    	int index = 0;
	    	for (String s : Main.fieldLabels){
	    		tempString+=s;
	    		int i = 0;
	    		if (Main.items.get(index)!=null){
		    		for (String ss : Main.items.get(index)){
		    			if (i==0){
		    				tempString+="_SPLIT_"+ss;
		    			}
		    			else{
		    				tempString+="_BREAK_"+ss;
		    			}
		    			i++;
		    		}
	    		}
	    		index++;

	            bw.write(tempString);
	            bw.newLine();
	    		tempString = "";
	    	}
	    	
	    	bw.close();    	
	    } catch (Exception e) {
	    }  	
    }
    //---------
    
	/**
	 * Sort Data
	 * This function is used to sort the metadata after a filter has been selected
	 * Filters have individual sort functions, filtering on the idAndFileTypeList, this function is used to sort all the other metadata data structures after the data has been filtered
	 * It is important to note that data structures, specifically arrays and lists relating to the metadata, use corresponding index's, thus all of them need to be normalised
	 */
    public static void sortData(){
		
    	//If there is actually data
    	if (Main.data!=null){
    		if (Main.data.length>0){
    			
		    	int index = 0;
			    ArrayList<File> tempFilesInDataset = new ArrayList<File>();
			    ArrayList<ArrayList<String>> tempMetadataFields = new ArrayList<ArrayList<String>>();
			    ArrayList<ArrayList<String>> tempMetadata = new ArrayList<ArrayList<String>>();
			    Object[][] tempData = new Object[Main.data.length][3];	
			    
				ArrayList<String> tempListOfFilePathsInDataset = new ArrayList<String>();
				ArrayList<String> tempListOfFileCorrespondingPIDSInDataset = new ArrayList<String>();
				
		    	for (String[] s : Main.idAndFileTypeList){ 
		    		int i = Integer.parseInt(s[2]);
		
		    		tempFilesInDataset.add(Main.filesInDataset.get(i));	
		    		tempMetadataFields.add(Main.metadataFields.get(i));
		    		tempMetadata.add(Main.metadata.get(i));
		    		tempData[index] = Main.data[i]; 
		    		
		    		tempListOfFilePathsInDataset.add(Main.listOfFilePathsInDataset.get(i));
		    		tempListOfFileCorrespondingPIDSInDataset.add(Main.listOfFileCorrespondingPIDSInDataset.get(i));
		    		
		    		index++;
		    	}
		    	
		    	Main.filesInDataset = tempFilesInDataset;
		    	Main.metadataFields = tempMetadataFields;
		    	Main.metadata = tempMetadata;
		    	Main.data = tempData;
		    	Main.listOfFilePathsInDataset = tempListOfFilePathsInDataset;
		    	Main.listOfFileCorrespondingPIDSInDataset = tempListOfFileCorrespondingPIDSInDataset;
		
		    	Main.table.setDefaultRenderer(Object.class, new DefaultTableCellRenderer()
				{
		
					private static final long serialVersionUID = 8259955961546083218L;
		
					@Override
				    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
				    {
				        final Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				        
				        if (isSelected && column!=2){
				        	setBorder(new LineBorder(Color.RED));
				        }
				        
				    	return GUIModifier.setCellColours(table, value, isSelected, hasFocus, row, column, c);
				    }
				});
				
			    Main.booleanTableModel.data = Main.data;
			    Main.table.setModel(Main.booleanTableModel);	
			    Main.booleanTableModel.fireTableDataChanged();
    		}
    	}
    }
    
    // Reset methods
    // --------------
	/**
	 * Reset Index's of Sorted Data
	 * After the data has been sorted, the idAndFileTypeList index's are reset, allowing for additional sorts
	 */
    public static void resetIndexesOfSortedData(){
    	int index = 0;
    	for (String[] s : Main.idAndFileTypeList){
    		s[2] = index+"";
    		index++;
    	}
    }
    
	/**
	 * Reset all Metadata data structures
	 */
    public static void resetMetadata(){
    	Main.index = 0;
    	Main.fileIndex = 1;
    	Main.progressCount = 0.0;

    	Main.metadata = new ArrayList<ArrayList<String>>();
    	Main.metadataFields = new ArrayList<ArrayList<String>>();
    	
    	Main.listOfFileCorrespondingPIDSInDataset = new ArrayList<String>();
    	Main.listOfFilePathsInDataset = new ArrayList<String>();
    	
    	Main.idAndFileTypeList = new ArrayList<String[]>();
    	
    	Main.filesInDataset = new ArrayList<File>();
    	
    	Main.data = new Object[Main.filesInDataset.size()][3];
    	Main.booleanTableModel.data = new Object[Main.filesInDataset.size()][3];
    	
    	resetMetadataSchemas();
    }
    
	/**
	 * Reset the Basic Metadata Schema
	 */
    public static void resetMetadataSchemas(){
    	Main.basicMetadataSchema = Utility.readTextFile(Main.class.getResource("settings/basicMetadataSchema.txt"));
    }
    

    
    // --------------
}
