package zamani.metafy.core.data;
import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.FileVisitResult;
import java.nio.file.FileVisitor;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.attribute.BasicFileAttributes;

import zamani.metafy.core.utility.Utility;

/** 
 * DirectoryCounter class
 * Implements FileVisitor, counts the number of supported file types in a directory, recursively
 * 
 * @author Michael Ferguson
 * 
 */
public class DirectoryCounter implements FileVisitor<Path> {

    private int fileCount;
    private Path directoryPath;
    private String[] supportedFileTypes; 
    
    /**
     * DirectoryCounterConstructor
	 * @param directoryPath String
	 * @param supportedFileTypes String[]
    */
    public DirectoryCounter(String directoryPath, String[] supportedFileTypes) {
         fileCount = 0;
         this.directoryPath = FileSystems.getDefault().getPath(directoryPath);
         this.supportedFileTypes = supportedFileTypes;
    }
    
    public void countRecursively() throws IOException{
   	 	Files.walkFileTree(directoryPath, this);
    }

    @Override
    public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs) throws IOException {
        return FileVisitResult.CONTINUE;
    }

    @Override
    public FileVisitResult postVisitDirectory(Path dir, IOException exc) throws IOException {
        return FileVisitResult.CONTINUE;
    }

    @Override
    public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
		if ((Utility.arrayContains(Utility.getExtension(file.getFileName().toString()),supportedFileTypes))){
	    	fileCount++;
		}
        return FileVisitResult.CONTINUE;
    }

    @Override
    public FileVisitResult visitFileFailed(Path file, IOException exc) throws IOException {
        return FileVisitResult.CONTINUE;
    }
    
    public int getFilesInDirectoryCount(){
    	return fileCount;
    }
}
