package zamani.metafy.core.data;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

/** 
 * ErrorFormatter class
 * Formats Error file produced by Metty-D into a filePath List and PID List
 * 
 * @author Michael Ferguson
 * 
 */
public class ErrorFormatter {

	static ArrayList<String> fileLines = new ArrayList<String>();
	public static ArrayList<String> filePathList = new ArrayList<String>();
	public static ArrayList<String> PIDList = new ArrayList<String>();
	
    /**
     * ErrorFormatter Constructor
     * @param errorFilePath String
    */
	public ErrorFormatter(String errorFilePath){
		fileLines = readLinesFromTextFile(errorFilePath);
		generateFilePathAndPIDList();
	}
	
    /**
     * Generate File path and PID list from file line list
    */
	public static void generateFilePathAndPIDList(){
		for (String line : fileLines){

			String filePath = "";
			String PID = "";
						
			int filePathStartIndex = -1;
			int filePathEndIndex = -1;
			
			filePathStartIndex = line.indexOf( '\'' );
			filePathEndIndex = line.indexOf( '\'', filePathStartIndex+1 );
			
			int filePathPIDStartIndex = -1;
			int filePathPIDEndIndex = -1;
			
			filePathPIDStartIndex = line.indexOf( '(' );
			filePathPIDEndIndex = line.indexOf( ')' );

			
			filePath = line.substring(filePathStartIndex+1, filePathEndIndex);
			String filePathIncludingPID = line.substring(filePathPIDStartIndex+1, filePathPIDEndIndex);
			
			if (filePath.contains(".")){
						
				int PIDStartIndex = filePathIncludingPID.lastIndexOf('/');
				int PIDPossibleEndIndex = filePathIncludingPID.indexOf('_', PIDStartIndex+1);
				int PIDEndIndex = filePathIncludingPID.indexOf('.');
				
				
				if (PIDPossibleEndIndex != -1){
					PID = filePathIncludingPID.substring(PIDStartIndex+1,PIDPossibleEndIndex);
				}
				else{
					PID = filePathIncludingPID.substring(PIDStartIndex+1,PIDEndIndex);
				}

				filePathList.add(filePath);
				PIDList.add(PID.replaceAll("(?<=\\p{L})(?=\\d)", ":"));
			}
		}
	}
	
    /**
     * Read Lines From Text File into List
     * @param filePath String
    */
    public static ArrayList<String> readLinesFromTextFile(String filePath){
    	ArrayList<String> listLines = new ArrayList<String>();
    	
    	BufferedReader br = null;
    	 
		try {
 
			String sCurrentLine;
 
			br = new BufferedReader(new FileReader(filePath));
			
			while ((sCurrentLine = br.readLine()) != null) {
				listLines.add(sCurrentLine);
			}
 
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if (br != null)br.close();
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
    	return listLines;
    }	
}
