package zamani.metafy.core.gui;

import java.awt.Toolkit;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Scanner;

import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingWorker;

import org.apache.tika.exception.TikaException;
import org.xml.sax.SAXException;

import zamani.metafy.core.Main;
import zamani.metafy.core.data.AssociateFilesVisitor;
import zamani.metafy.core.data.DataDealer;
import zamani.metafy.core.data.DirectoryCounter;
import zamani.metafy.core.data.DirectoryVisitor;
import zamani.metafy.core.data.ErrorFormatter;
import zamani.metafy.core.persistence.Deserializer;
import zamani.metafy.core.persistence.MetadataStore;
import zamani.metafy.core.persistence.Serializer;
import zamani.metafy.core.utility.FileSize;
import zamani.metafy.core.utility.FileTypes;
import zamani.metafy.core.utility.HttpGet;
import zamani.metafy.core.utility.ServerPath;
import zamani.metafy.core.utility.Utility;
import zamani.metafy.core.xml.XMLReader;
import zamani.metafy.core.xml.XMLWriter;

/** 
 * BackgroundWorker class
 * Contains nested BackgroundWorker classes
 * 
 * @author Michael Ferguson
 * 
 */
public final class BackgroundWorker {

	/** 
	 * Associate Files Worker Class
	 * Extends SwingWorker
	 * 
	 * @author Michael Ferguson
	 * 
	 */
	public class AssociateFilesWorker extends SwingWorker<Void, Void> {   

		private String dataDirectoryPathText;
		private String sharedDrivePathServer;
		
		int associatedFilesCount;
		int associatedFilesIndex;
		
		
		ArrayList<String> associatedFileNameList;
		ArrayList<String> associatedFilePathList;

		public  AssociateFilesWorker(JProgressBar aProgressBar, String dataDirectoryPathText, String sharedDrivePathServer) {  
			this.dataDirectoryPathText = dataDirectoryPathText;
			this.sharedDrivePathServer = sharedDrivePathServer;
			
			Main.associateFilesProgressBar = aProgressBar;
			
			associatedFilesCount = 0;
			associatedFilesIndex = 0;
			
			associatedFileNameList = new ArrayList<String>();
			associatedFilePathList = new ArrayList<String>();
			
			addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if ("progress".equals(evt.getPropertyName())) {
						Main.associateFilesProgressBar.setValue((Integer) evt.getNewValue());
					}
				}
			});

			Main.associateFilesProgressBar.setVisible(true);        
			Main.associateFilesProgressBar.setStringPainted(true);
			Main.associateFilesProgressBar.setValue(0);
			setProgress(0);
		}
		
		public void setProgressAmount(int progressAmount){
			setProgress(progressAmount);
		}
				
		public void associateFiles(String dataDirectoryPathText) throws IOException, SAXException, TikaException{
			// Calculate the Associated Directory path on the server
			String associatedDirServer = ServerPath.calculateServerDirectoryPath(dataDirectoryPathText, sharedDrivePathServer);
			
			getFilesCount(associatedDirServer, dataDirectoryPathText);
			

			// First try and get the associated files from the server, if success is false, get associated files from clients computer (slower)
			if (!getAssociatedFilesFromServer(dataDirectoryPathText, associatedDirServer, associatedFilesCount)){
				// Get Associated files from clients computer
				getAssociatedFiles(dataDirectoryPathText, associatedFilesCount);	
			}
			
			ArrayList<String> datasetFilenames = filePathListToFileNameList();

			int index = 0;
			
			int associatedFileNameIndex;
			String associatedFilePath, oldFildPath;

			for (String fileName : datasetFilenames){
				
				associatedFileNameIndex = associatedFileNameList.indexOf(fileName);

				if (associatedFileNameIndex>-1){
					associatedFilePath = associatedFilePathList.get(associatedFileNameIndex);

					oldFildPath = Main.listOfFilePathsInDataset.get(index);
					Main.filesInDataset.set(index, new File(associatedFilePath));
					Main.listOfFilePathsInDataset.set(index, associatedFilePath);
					
				    //Console Feedback ---------------------------------------- 
					try
					{
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Info: ", Main.infoAttributes);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Action:   ", Main.boldAttributes);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Associate File. \n PID: "+Main.listOfFileCorrespondingPIDSInDataset.get(index)+" \n Old file path: "+oldFildPath+" \n New Associated file path:"+associatedFilePath+" \n", null);
					}
					catch(Exception ex) { System.out.println(ex); }
				    //---------------------------------------------------------- 
				}
				else{
					String tempNewFileName = "";
					//If - replaced with _ exists (use it)
					if (fileName.contains("-")){
						tempNewFileName = fileName.replace("-","_");
					}
					
					associatedFileNameIndex = associatedFileNameList.indexOf(tempNewFileName);

					if (associatedFileNameIndex>-1){
						associatedFilePath = associatedFilePathList.get(associatedFileNameIndex);
						oldFildPath = Main.listOfFilePathsInDataset.get(index);
						Main.filesInDataset.set(index, new File(associatedFilePath));
						Main.listOfFilePathsInDataset.set(index, associatedFilePath);
						
					    //Console Feedback ---------------------------------------- 
						try
						{
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Info: ", Main.infoAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Action:   ", Main.boldAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Associate File. \n PID: "+Main.listOfFileCorrespondingPIDSInDataset.get(index)+" \n Old file path: "+oldFildPath+" \n New Associated file path:"+associatedFilePath+" \n", null);
						}
						catch(Exception ex) { System.out.println(ex); }
					    //---------------------------------------------------------- 
					}
					else{
						
		    			//Console Feedback ---------------------------------------- 
						try
						{
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Error: ", Main.errorAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Associate File failed:   ", Main.boldAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "PID: "+Main.listOfFileCorrespondingPIDSInDataset.get(index)+" \n File path: "+Main.listOfFilePathsInDataset.get(index)+" \n", null);
						}
						catch(Exception ex) { System.out.println(ex); }
					    //---------------------------------------------------------- 
					}
				}
				index++;
			}
			
			Object[][] tempData = DataDealer.transformedMetadataToObjectArray(Main.filesInDataset, Main.metadata, Main.idAndFileTypeList);
			Main.data = tempData;
			Main.booleanTableModel.data = tempData;
			Main.table.setModel(Main.booleanTableModel);	
			Main.booleanTableModel.fireTableDataChanged();
		}
		
		public boolean getAssociatedFilesFromServer(String associatedDirClient, String associatedDirServer, int associatedFilesCount){
	    	boolean success = true;
	    	int currentFileCount = 0;
	    	String fullFilePath;
	    	
	    	// Assumption: use of a shared drive, mapped to a drive letter (format Y:\)
	    	String driveLetter = (new File(associatedDirClient)).toPath().getRoot().toString().replace("\\", ""); // formatted to Y:
	    	
	    	// Stream from the associated data from the server
	    	try {
	    	URL url = new URL("http://nala.cs.uct.ac.za/zamaniProject_archive/inc/associate_files/getAssociatedFilesTextFile.php?dir="+associatedDirServer);
	    	Scanner s = new Scanner(url.openStream());
			    while (s.hasNextLine()) {
			       fullFilePath = s.nextLine().replace("", "");
			       associatedFilePathList.add(driveLetter+fullFilePath.replace(sharedDrivePathServer,""));
			       associatedFileNameList.add((fullFilePath).substring((fullFilePath).lastIndexOf("/")+1, (fullFilePath).length()));

			       currentFileCount++;
			       setProgressAmount((int)(100*((double)currentFileCount/(double)associatedFilesCount)));
			    }
			    s.close();
	    	}
	    	// Connection problem, or file did not exist on the server, or the URL was not in the right format.
	    	catch(IOException ex) {
	    		return false; 
	    	}
	    	
	    	if (currentFileCount==0){
	    		return false;
	    	}
	    	
	    	return success;
		}
		
		public ArrayList<String> filePathListToFileNameList(){
			ArrayList<String> tempListOfFilenames = new ArrayList<String>();
			for (String filePath : Main.listOfFilePathsInDataset){	
				File f = new File(filePath);
				tempListOfFilenames.add(f.getName());
			}
			
			return tempListOfFilenames;
		}


		public void getAssociatedFiles(String dir, int associatedFilesCount) throws IOException, SAXException, TikaException {	
			AssociateFilesVisitor associateFilesVisitor = new AssociateFilesVisitor(dir,this,associatedFilesCount);
			associateFilesVisitor.processFiles();
			associatedFileNameList = associateFilesVisitor.getAssociatedFileNameList();
			associatedFilePathList = associateFilesVisitor.getAssociatedFilePathList();
		}
		

		void getFilesCount(String associatedDirServer, String path) throws IOException {
			
			int temporaryDirectoryCount = -1;
			
			// HTTP GET directory count
			try {
				temporaryDirectoryCount = HttpGet.getDirectoryCount("http://nala.cs.uct.ac.za/zamaniProject_archive/inc/getDirectoryCount.php?dir="+associatedDirServer);
			} catch (Exception e) {
				temporaryDirectoryCount = -1;
			}
			// Response of 200 
			if (temporaryDirectoryCount>0){
				associatedFilesCount = temporaryDirectoryCount;
			}
			// Calculate the directory count ourself
			else{
				
				DirectoryCounter directoryCounter = new DirectoryCounter(path, FileTypes.supportedFileTypes);
				directoryCounter.countRecursively();
				associatedFilesCount = directoryCounter.getFilesInDirectoryCount();
			}
		}

		@Override
		/**
		 * Do in background thread. 
		 * @throws  Exception  Throws exceptions 
		 */
		protected Void doInBackground() throws Exception {
			associateFiles(dataDirectoryPathText);
			return null;
		}    

		@Override
		/** Executed automatically once the doInBackground() method is complete. 
		 */
		public void done() { 
			// Close the transform window
            WindowEvent wev = new WindowEvent(Main.associateFilesWindow, WindowEvent.WINDOW_CLOSING);
            Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(wev);
            
            Main.JProgressBarPanelAssociateFiles = new JPanel();
		}
	}
	
	/** 
	 * Export Metadata Worker Class
	 * Extends SwingWorker
	 * 
	 * @author Michael Ferguson
	 * 
	 */
	public class ExportMetadataWorker extends SwingWorker<Void, Void> {     
		private String siteLatitudeText;
		private String siteLongitudeText;
		private String sharedDrivePathServerText;
		private String sharedDrivePathClientText;
		private String outputMetadataPathText;
		
		public  ExportMetadataWorker(JProgressBar aProgressBar, String siteLatitudeText, String siteLongitudeText, String sharedDrivePathServerText, String sharedDrivePathClientText, String outputMetadataPathText) {  
			Main.exportMetadataProgressBar = aProgressBar; 
			this.siteLatitudeText = siteLatitudeText;
			this.siteLongitudeText = siteLongitudeText;
			this.sharedDrivePathServerText = sharedDrivePathServerText;
			this.sharedDrivePathClientText = sharedDrivePathClientText;
			this.outputMetadataPathText = outputMetadataPathText;
			
			addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if ("progress".equals(evt.getPropertyName())) {
						Main.exportMetadataProgressBar.setValue((Integer) evt.getNewValue());
					}
				}
			});

			Main.exportMetadataProgressBar.setVisible(true);        
			Main.exportMetadataProgressBar.setStringPainted(true);
			Main.exportMetadataProgressBar.setValue(0);
			setProgress(0);
		}

		public void exportMetadata(String siteLatitudeText, String siteLongitudeText, String sharedDrivePathServerText, String sharedDrivePathClientText, String outputMetadataPathText){
			String country = Main.coverageSpatialCountry.getText();
			String countryCamelCase = Utility.toCamelCase(country);
			String countryPID = countryCamelCase+"Collection:1";
			
			// Site
			String site = Main.coverageSpatialSitename.getText();
			String siteCamelCase = Utility.toCamelCase(site);
			String sitePID = siteCamelCase+"Collection:1";
			
			
			String writeDirectory = outputMetadataPathText+"\\"+countryCamelCase+"_"+siteCamelCase+"_output";
			File dir = new File(writeDirectory);
			dir.mkdir();
			
			String writeDirectoryInner = writeDirectory+"\\"+siteCamelCase+"_metadata";
			File dirInner = new File(writeDirectoryInner);
			dirInner.mkdir();
	
			XMLWriter.writeXMLCountryCollectionFile(writeDirectoryInner+"\\"+countryCamelCase+"Collection1.xml", countryPID, country);
			XMLWriter.writeXMLSiteCollectionFile(writeDirectoryInner+"\\"+siteCamelCase+"Collection2.xml", siteCamelCase+"Collection:1", countryPID, site, siteLatitudeText, siteLongitudeText, country);
			
			// Loop through metadata Records
			int metadataRecordIndex = 0;
			int metadataFieldsListSize = Main.metadataFields.size();
			String filename[];
			File file;
			String filesPathOnClient, filesPathOnServer;
			
			for (ArrayList<String> metadataRecord : Main.metadataFields){
				// Only Write metadata files for 'Published' true items
				if ((boolean)Main.data[metadataRecordIndex][0]){

					file = Main.filesInDataset.get(metadataRecordIndex);
					
					// Only write metadata files for 'valid' items (Items where the file actually exists) 
					if (file.exists()){
						filename = Main.metadata.get(metadataRecordIndex).get(1).split(":");
					
						// Get files path on server
						filesPathOnClient = file.getPath();
						filesPathOnServer = ServerPath.getFilepathOnServer(file.getPath(), sharedDrivePathServerText, sharedDrivePathClientText);		
	
					XMLWriter.writeXMLFile(metadataRecord, Main.metadata.get(metadataRecordIndex),  writeDirectoryInner+"\\"+filename[0]+filename[1]+".xml", sitePID, filesPathOnServer, filesPathOnClient, FileSize.humanReadable(file));
					}
				}
				metadataRecordIndex++;	
				setProgress((int)(100*((double)metadataRecordIndex/(double)metadataFieldsListSize)));
			}	
		}

		@Override
		/**
		 * Do in background thread. 
		 * @throws  Exception  Throws exceptions 
		 */
		protected Void doInBackground() throws Exception {
			exportMetadata(siteLatitudeText, siteLongitudeText, sharedDrivePathServerText, sharedDrivePathClientText, outputMetadataPathText);
			return null;
		}    

		@Override
		/** Executed automatically once the doInBackground() method is complete. 
		 */
		public void done() { 
			// Close the transform window
            WindowEvent wev = new WindowEvent(Main.exportMetadataWindow, WindowEvent.WINDOW_CLOSING);
            Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(wev);
            
            Main.JProgressBarPanelExportMetadata = new JPanel();
		}
	}

	/** 
	 * Import Metadata Worker Class
	 * Extends SwingWorker
	 * 
	 * @author Michael Ferguson
	 * 
	 */
	public class ImportWorker extends SwingWorker<Void, Void> {   
		private File[] dir; 
		int filesProcessedCount, filesInDirectory = 0;

		public  ImportWorker(JProgressBar aProgressBar, File[] dir) {  
			Main.progressImport = aProgressBar; 

			this.filesProcessedCount = 0;
			this.dir = dir;
			this.filesInDirectory = 0;
			
			addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if ("progress".equals(evt.getPropertyName())) {
						Main.progressImport.setValue((Integer) evt.getNewValue());
					}
				}
			});
			
			Main.progressImport.setVisible(true);        
			Main.progressImport.setStringPainted(true);
			Main.progressImport.setValue(0);
			setProgress(0);
		}
		
		public void setProgressAmount(int progressAmount){
			setProgress(progressAmount);
		}
		
		private void calculateNumFilesInDirectory() throws IOException, SAXException, TikaException{
		    	for (File file : dir){
		    		DataDealer.getFilesInDirectoryCount(file);
		    	}
		    	filesInDirectory = Main.totalFilesCount;
		}
		
		@Override
		/**
		 * Do in background thread. 
		 * @throws  Exception  Throws exceptions 
		 */
		protected Void doInBackground() throws Exception {
			calculateNumFilesInDirectory();
			for (File directory : dir){
				DirectoryVisitor directoryVisitor = new DirectoryVisitor(directory.getPath(),this,filesInDirectory);
				directoryVisitor.processFiles();
			}
			return null;
		}    

		@Override
		/** Executed automatically once the doInBackground() method is complete. 
		 */
		public void done() { 
			
			DataDealer.filesToObjectArray();
			
			Main.table.setModel(Main.booleanTableModel);	
			Main.booleanTableModel.fireTableDataChanged();
	    	
	    	//Reset Dataset Directory path
			Main.datasetDirectory = "";
	    	
			// Close the progress window
            WindowEvent wev = new WindowEvent(Main.popupImportProgressWindow, WindowEvent.WINDOW_CLOSING);
            Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(wev);
		}
	}

	/** 
	 * Load Metadata Worker Class
	 * Extends SwingWorker
	 * 
	 * @author Michael Ferguson
	 * 
	 */
	public class LoadMetadataWorker extends SwingWorker<Void, Void> {   
		private String pathToMetadataFile;
		
		public  LoadMetadataWorker(JProgressBar aProgressBar, String pathToMetadataFile) {  
			Main.loadMetadataProgressBar = aProgressBar; 
			this.pathToMetadataFile = pathToMetadataFile;
			
			addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if ("progress".equals(evt.getPropertyName())) {
						Main.loadMetadataProgressBar.setValue((Integer) evt.getNewValue());
					}
				}
			});
	
			Main.transformMetadataProgressBar.setVisible(true);        
			Main.transformMetadataProgressBar.setStringPainted(true);
			Main.transformMetadataProgressBar.setValue(0);
			setProgress(0);
		}
	
		public void loadMetadata(String pathToMetadataFile){
			Deserializer deserializer = new Deserializer(pathToMetadataFile);
			MetadataStore metadataObject = deserializer.deserialzeMetadata();
	
			Main.data = metadataObject.getData();
			Main.listOfFilePathsInDataset = metadataObject.getListOfFilePathsInDataset();
			Main.listOfFileCorrespondingPIDSInDataset = metadataObject.getListOfFileCorrespondingPIDSInDataset();
			Main.metadataFields = metadataObject.getMetadataFields();
			Main.metadata = metadataObject.getMetadata();
			Main.idAndFileTypeList = metadataObject.getIDAndFileTypeList();
			Main.basicMetadataSchema = metadataObject.getBasicMetadataSchema();
			Main.filesInDataset = metadataObject.getFilesInDataset();
			
			String[] automationFields = metadataObject.getAutomationFields();
			// Set automation fields from saved automation fields
			Main.identifierPrefix.setText(automationFields[0]);
			Main.sourceOfData.setText(automationFields[1]);
			Main.coverageSpatialCountry.setText(automationFields[2]);
			Main.coverageSpatialSitename.setText(automationFields[3]);
			Main.coverageSpatialStructure.setText(automationFields[4]);
			
			Main.booleanTableModel.data = metadataObject.getData();
			Main.table.setModel(Main.booleanTableModel);	
			Main.booleanTableModel.fireTableDataChanged();
	    	
			Main.fileIndex = metadataObject.getCurrentPIDCount();
		}
	
		@Override
		/**
		 * Do in background thread. 
		 * @throws  Exception  Throws exceptions 
		 */
		protected Void doInBackground() throws Exception {
			loadMetadata(pathToMetadataFile);
			return null;
		}    
	
		@Override
		/** Executed automatically once the doInBackground() method is complete. 
		 */
		public void done() { 
			// Close the transform window
	        WindowEvent wev = new WindowEvent(Main.popupLoadMetadataProgressWindow, WindowEvent.WINDOW_CLOSING);
	        Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(wev);
		}
	}
	
	/** 
	 * Transform Metadata Worker Class
	 * Extends SwingWorker
	 * 
	 * @author Michael Ferguson
	 * 
	 */
	public class TransformMetadataWorker extends SwingWorker<Void, Void> {   
		private String coverageSpatialCountryText, coverageSpatialSitenameText, siteLatitudeText, siteLongitudeText, sharedDrivePathServerText, sharedDrivePathClientText, alukaMetadataFilePathText, alukaErrorFilePathText, alukaDataDirectoryFilePathText, transformedMetadataPathText;

		public  TransformMetadataWorker(JProgressBar aProgressBar, String coverageSpatialCountryText, String coverageSpatialSitenameText, String siteLatitudeText, String siteLongitudeText, String sharedDrivePathServerText, String sharedDrivePathClientText, String alukaMetadataFilePathText, String alukaErrorFilePathText, String alukaDataDirectoryFilePathText, String transformedMetadataPathText) {  
			Main.transformMetadataProgressBar = aProgressBar; 
			
			this.coverageSpatialCountryText = coverageSpatialCountryText;
			this.coverageSpatialSitenameText = coverageSpatialSitenameText;
			this.siteLatitudeText = siteLatitudeText;
			this.siteLongitudeText = siteLongitudeText;
			this.sharedDrivePathServerText = sharedDrivePathServerText;
			this.sharedDrivePathClientText = sharedDrivePathClientText;
			this.alukaMetadataFilePathText = alukaMetadataFilePathText;
			this.alukaErrorFilePathText = alukaErrorFilePathText;
			this.alukaDataDirectoryFilePathText = alukaDataDirectoryFilePathText;
			this.transformedMetadataPathText = transformedMetadataPathText;
			
			addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if ("progress".equals(evt.getPropertyName())) {
						Main.transformMetadataProgressBar.setValue((Integer) evt.getNewValue());
					}
				}
			});

			Main.transformMetadataProgressBar.setVisible(true);        
			Main.transformMetadataProgressBar.setStringPainted(true);
			Main.transformMetadataProgressBar.setValue(0);
			setProgress(0);
			Main.transformMetadataProgressBar.setMaximum(100);
			
		}
		
		@SuppressWarnings({ "static-access", "unused" })
		public void transformMetadata(String coverageSpatialCountryText, String coverageSpatialSitenameText, String siteLatitudeText, String siteLongitudeText, String sharedDrivePathServerText, String sharedDrivePathClientText, String alukaMetadataFilePathText, String alukaErrorFilePathText, String alukaDataDirectoryFilePathText, String transformedMetadataPathText){

			ErrorFormatter errorFormatter = new ErrorFormatter(alukaErrorFilePathText);
			ArrayList<String> filePathList = errorFormatter.filePathList;
			ArrayList<String> PIDList = errorFormatter.PIDList;
			ArrayList<String> filePathListMetadataRecords = new ArrayList<String>();
			ArrayList<String> correspondingPIDListMetadataRecords = new ArrayList<String>();
			XMLReader xmlReader = new XMLReader(alukaMetadataFilePathText);
			ArrayList<ArrayList<String>> metadataFieldsList = xmlReader.metadataFieldsList;
			ArrayList<ArrayList<String>> metadataList = xmlReader.metadataList;
			ArrayList<String[]> idAndFileTypeList = xmlReader.idAndFileTypeList;
			ArrayList<File> filesInDatasetList = new ArrayList<File>();
			String countryCamelCase = Utility.toCamelCase(coverageSpatialCountryText);

			String site = coverageSpatialSitenameText;
			String siteCamelCase = Utility.toCamelCase(site);

			String transformedMetadataOutputPath = transformedMetadataPathText+"/"+countryCamelCase+"_"+siteCamelCase+"_transformed";
			
			File dir = new File(transformedMetadataOutputPath);
			dir.mkdir();
			
			// Loop through metadata Records
			int metadataRecordIndex = 0;
			int metadataFieldsListSize=metadataFieldsList.size();
			for (ArrayList<String> metadataRecord : metadataFieldsList){					
				String recordPID = metadataList.get(metadataRecordIndex).get(1);

				int fileFormatterPIDIndex = PIDList.indexOf(recordPID);  

				String recordFilePath = "undefined";		
				if (fileFormatterPIDIndex>-1){
					recordFilePath = filePathList.get(fileFormatterPIDIndex);
				}

				String[] temp = idAndFileTypeList.get(metadataRecordIndex);
				if (recordFilePath.equals("undefined")){
					temp[1] = "undefined";
				}
				else{
					temp[1] = Utility.getExtension(recordFilePath);
				}
				idAndFileTypeList.set(metadataRecordIndex, temp);

				filePathListMetadataRecords.add(recordFilePath);
				
				correspondingPIDListMetadataRecords.add(recordPID);
				String[] clientServerFilePaths = ServerPath.calculateNewFilePaths(recordFilePath, alukaDataDirectoryFilePathText, sharedDrivePathServerText, sharedDrivePathClientText);
				String clientFilePath = clientServerFilePaths[0];
				filesInDatasetList.add(new File(clientFilePath));
				
				metadataRecordIndex++;	

				setProgress((int)(95*((double)metadataRecordIndex/(double)metadataFieldsListSize)));
			}	

			String identifierPrefix = xmlReader.identifierPrefix;
			String sourceOfData = xmlReader.sourceOfData;

			String[] automationFields = new String[]{identifierPrefix, sourceOfData, coverageSpatialCountryText, coverageSpatialSitenameText, "", siteLatitudeText, siteLongitudeText, sharedDrivePathClientText, sharedDrivePathServerText};
			
			Serializer serializer = new Serializer(transformedMetadataPathText+"/"+countryCamelCase+"_"+siteCamelCase+"_transformed"+"/"+siteCamelCase+".metafy");
			serializer.serializeMetadata(idAndFileTypeList, metadataFieldsList, metadataList, filesInDatasetList, DataDealer.transformedMetadataToObjectArray(filesInDatasetList, metadataList, idAndFileTypeList), filePathListMetadataRecords, correspondingPIDListMetadataRecords, Main.basicMetadataSchema, automationFields,  xmlReader.currentPIDcount+1);		
			setProgress(100);
		}

		@Override
		/**
		 * Do in background thread. 
		 * @throws  Exception  Throws exceptions 
		 */
		protected Void doInBackground() throws Exception {
			
			transformMetadata(coverageSpatialCountryText, coverageSpatialSitenameText, siteLatitudeText, siteLongitudeText, sharedDrivePathServerText, sharedDrivePathClientText, alukaMetadataFilePathText, alukaErrorFilePathText, alukaDataDirectoryFilePathText, transformedMetadataPathText);
			return null;
		}    

		@Override
		/** Executed automatically once the doInBackground() method is complete. 
		 */
		public void done() { 
			// Close the transform window
            WindowEvent wev = new WindowEvent(Main.transformAlukaMetadataWindow, WindowEvent.WINDOW_CLOSING);
            Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(wev);

            Main.JProgressBarPanel = new JPanel();
		}
	}

	/** 
	 * Validate Metadata Worker Class
	 * Extends SwingWorker
	 * 
	 * @author Michael Ferguson
	 * 
	 */
	public class ValidateMetadataWorker extends SwingWorker<Void, Void> {   

		public  ValidateMetadataWorker(JProgressBar aProgressBar) {  
			Main.validatemetadataProgressBar = aProgressBar; 

			addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if ("progress".equals(evt.getPropertyName())) {
						Main.validatemetadataProgressBar.setValue((Integer) evt.getNewValue());
					}
				}
			});
			
			Main.validatemetadataProgressBar.setVisible(true);        
			Main.validatemetadataProgressBar.setStringPainted(true);
			Main.validatemetadataProgressBar.setValue(0);
			setProgress(0);
		}
		

		public void validateMetadataFilePaths(){
			boolean validMetadata = true;
			
			int totalValidFiles = 0;
			int totalInvalidFiles = 0;
			
			int toalFilesInDataset = Main.filesInDataset.size();
			
			int index = 0;
			for (File file : Main.filesInDataset){
				if (!file.exists()){
					validMetadata = false;
					totalInvalidFiles++;
					//Console Feedback ---------------------------------------- 
					try
					{
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Error: ", Main.errorAttributes);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Metadata Record Invalid:   ", Main.boldAttributes);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "PID: "+Main.listOfFileCorrespondingPIDSInDataset.get(index)+" \n", null);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "File path Does not exist: "+Main.listOfFilePathsInDataset.get(index)+" \n", null);
					}
					catch(Exception ex) { System.out.println(ex); }
				    //---------------------------------------------------------- 
				}
				else{
					totalValidFiles++;
				}
				index++;
				setProgress((int)(100*((double)index/(double)toalFilesInDataset)));
			}
			
			if (validMetadata){
				if (index!=0){
					//Console Feedback ---------------------------------------- 
					try
					{
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Info: ", Main.infoAttributes);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Metadata Successfully Validated.   ", Main.boldAttributes);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Total Files: "+index+"\n", null);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Total Valid Files: "+totalValidFiles+"\n", null);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Total Invalid Files: "+totalInvalidFiles+"\n", null);
					}
					catch(Exception ex) { System.out.println(ex); }
					//---------------------------------------------------------- 
				}
				else{
					//Console Feedback ---------------------------------------- 
					try
					{
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Info: ", Main.infoAttributes);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "No Metadata to Validate.   ", Main.boldAttributes);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Total Files: "+index+"\n", null);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Total Valid Files: "+totalValidFiles+"\n", null);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Total Invalid Files: "+totalInvalidFiles+"\n", null);
					}
					catch(Exception ex) { System.out.println(ex); }
					//---------------------------------------------------------- 
				}
			    
			}
			else{
				//Console Feedback ---------------------------------------- 
				try
				{
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Error: ", Main.errorAttributes);
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Metadata Validation Failed.   \n", Main.boldAttributes);
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Total Files: "+index+"\n", null);
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Total Valid Files: "+totalValidFiles+"\n", null);
					Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Total Invalid Files: "+totalInvalidFiles+"\n", null);
				}
				catch(Exception ex) { System.out.println(ex); }
			    //---------------------------------------------------------- 
			}
		}
		
		@Override
		/**
		 * Do in background thread. 
		 * @throws  Exception  Throws exceptions 
		 */
		protected Void doInBackground() throws Exception {
			validateMetadataFilePaths();
			return null;
		}    

		@Override
		/** Executed automatically once the doInBackground() method is complete. 
		 */
		public void done() { 
			// Close the progress window
            WindowEvent wev = new WindowEvent(Main.popupValidateMetadataProgressWindow, WindowEvent.WINDOW_CLOSING);
            Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(wev);	
		}
	}
	
}	