package zamani.metafy.core.gui;
import javax.swing.table.AbstractTableModel;

/** 
 * BooleanTableModel class
 * Custom table model
 * Boolean data rendered as a check box's
 * Only the first column is editable, boolean values should occur in the first column
 * 
 * @author Michael Ferguson
 * 
 */
public class BooleanTableModel extends AbstractTableModel {
	private static final long serialVersionUID = 3249149063341932697L;
	String[] columns = {"<html>&nbsp;Pub","<html>&nbsp;&nbsp;Metadata","<html>&nbsp;Status"};
    public Object[][] data = {};

	/**
	 * Get Row Count
	 * @return row count
	 */
    public int getRowCount() {
        return data.length;
    }

	/**
	 * Get Column Count
	 * @return column count
	 */
    public int getColumnCount() {
        return columns.length;
    }

	/**
	 * Get Value at specified row and column index's
	 * @param rowIndex int row index
	 * @param columnIndex int column index
	 * @return value
	 */
    public Object getValueAt(int rowIndex, int columnIndex) {
        return data[rowIndex][columnIndex];
    }

	/**
	 * Get Column Name
	 * @param column int column
	 * @return column name
	 */
    @Override
    public String getColumnName(int column) {
        return columns[column];
    }
    
	/**
	 * Check if Cell Is Editable
	 * @param row int row
	 * @param col int column
	 * @return true if cell is editable, false if not
	 */  
    public boolean isCellEditable(int row, int col) {
    	// Only make the first column editable
    	if (col==0){
    		return true;
    	}
    	else{
    		return false;
    	}
        
    }
    
	/**
	 * Set Value At
	 * @param value Object
	 * @param row int row
	 * @param col int col
	 */  
    public void setValueAt(Object value, int row, int col) {
    		data[row][col] = value; // save edits 
    	  fireTableCellUpdated(row, col); // inform table about change
    }
    
	/**
	 * Update Cell
	 * @param row int row
	 * @param col int col
	 */  
    public void updateCell(int row, int col) {
	  fireTableCellUpdated(row, col); // inform table about change
    }

	/**
	 * Get Column Class
	 * This method is used by the JTable to define the default
	 * renderer or editor for each cell. For example if you have
     * a boolean data it will be rendered as a check box. A
     *  number value is right aligned.
	 * @param columnIndex int column index
	 */  
    @Override
    public Class<?> getColumnClass(int columnIndex) {
        return data[0][columnIndex].getClass();
    }
}