package zamani.metafy.core.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Set;
import java.util.Map.Entry;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.UIManager.LookAndFeelInfo;
import javax.swing.border.LineBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.plaf.ColorUIResource;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

import zamani.metafy.core.Main;
import zamani.metafy.core.data.DataDealer;
import zamani.metafy.core.utility.Utility;

/** 
 * GUIBuilder class
 * Contains GUI functions relating to the construction of the GUI
 * 
 * @author Michael Ferguson
 * 
 */
public class GUIBuilder {
	
	// Toolbar vars
	static final private String PREVIOUS = "previous";
	static final private String NEXT = "next";
	static final private String ZOOM_IN = "zoom_in";
	static final private String ZOOM_OUT = "zoom_out";
	static final private String BEST_FIT = "best_fit";
	static final private String NORMAL_SIZE = "normal_size";
	
	// Public Builder Functions
	// ------------------------
	
	/**
	 * Build GUI Look and Feel
	 */
	public void buildGUILookAndFeel(){
		try {       
			Set<Entry<Object, Object>> entries =  UIManager.getLookAndFeelDefaults().entrySet();
        
	        for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
	            if ("Nimbus".equals(info.getName())) {
	                UIManager.setLookAndFeel(info.getClassName());
	                 break;
	             }
	        }        
	    	
	  	    for (Entry<Object, Object> entry : entries)
	  	    {	

	  	    	if (entry.getKey().toString().contains("ToolBar")){
	  	    		UIManager.getLookAndFeelDefaults().put(entry.getKey(), entry.getValue());
	  	    	}
	  	    	else if (entry.getKey().toString().contains("SplitPaneDivider") ){
	  	    		UIManager.getLookAndFeelDefaults().put(entry.getKey(), entry.getValue());
	  	    	}
	  	    	else if (entry.getKey().toString().contains("Table")){
	  	    		UIManager.getLookAndFeelDefaults().put(entry.getKey(), entry.getValue());
	  	    	}
	  	    }
	  	  
	  	    UIManager.put("ToolBar.dockingBackground", new Color(0xE2E2E2));
	  	    UIManager.put("ToolBar.background", new Color(0xE2E2E2));
	    	UIManager.put("Panel.background", new Color(0xE2E2E2));
	    	
	    	ColorUIResource colorResource = new ColorUIResource(Color.green.darker());
	    	UIManager.put("nimbusOrange",colorResource);
	    	
		} catch (ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}
	
	/**
	 * Configure Applet's Frame
	 * @param  appletFrame  Object applet's frame object
	 */
	public void configureMainFrame(Object appletFrame){
		while (! (appletFrame instanceof Frame)){
			appletFrame = ((Component) appletFrame).getParent ();
		}
		Frame frame = (Frame) appletFrame;
		 
		// Get rid of default applet menu
		frame.setMenuBar(null);
		  
		frame.pack();
		  
		Main.screenDimensions = Utility.getScreenDimension(frame);
		  
		frame.setMinimumSize(new Dimension((int)(Main.screenDimensions.width-10), (int)(Main.screenDimensions.height-80)));
		//frame.setExtendedState(Frame.MAXIMIZED_BOTH); 
	}
	
	/**
	 * Build Applet Menu Items
	 */
	public void buildMenuItems(){
		Main.loadMetadataMI = new JMenuItem("Load Metadata...", new ImageIcon(Main.class.getResource("images/load_open.png")));
		Main.openFilesMI = new JMenuItem("Open Files...", new ImageIcon(Main.class.getResource("images/load_open.png")));
		Main.addDirectoriesMI = new JMenuItem("Add Directories...", new ImageIcon(Main.class.getResource("images/add_file_folder.png")));
		Main.addFilesMI = new JMenuItem("Add Files...", new ImageIcon(Main.class.getResource("images/add_file_folder.png")));
		Main.openDirectoriesMI = new JMenuItem("Open Directories...", new ImageIcon(Main.class.getResource("images/load_open.png")));
		Main.importSubMenu = new JMenu("Import");
		Main.importCoordinates = new JMenuItem("Import Co-ords...", new ImageIcon(Main.class.getResource("images/load_open.png")));
		Main.addCameraCalibrationSettings = new JMenuItem("Import Camera Calibration Settings...", new ImageIcon(Main.class.getResource("images/load_open.png")));
		Main.exportSubMenu = new JMenu("Export");
		Main.exportMI = new JMenuItem("Export Metadata...", new ImageIcon(Main.class.getResource("images/save.png")));
		Main.transformAlukaMetadata = new JMenuItem("Transform Aluka Metadata...", new ImageIcon(Main.class.getResource("images/save.png")));
		
		Main.calculateTitles = new JMenuItem("Calculate Titles",  new ImageIcon(Main.class.getResource("images/calculate.png")));
		Main.associateFilesMI = new JMenuItem("Associate Files...", new ImageIcon(Main.class.getResource("images/associate_files.png")));
		Main.validateMetadata = new JMenuItem("Validate Metadata",  new ImageIcon(Main.class.getResource("images/validate.png")));
		
		Main.saveMetadata = new JMenuItem("Save Metadata",  new ImageIcon(Main.class.getResource("images/save.png")));
		Main.saveMetadataAs = new JMenuItem("Save Metadata As...",  new ImageIcon(Main.class.getResource("images/save.png")));
		
		Main.aboutMI = new JMenuItem("About");
		Main.removeCameraCalibrationSettings = new JMenuItem("Remove Camera Calibration Settings", new ImageIcon(Main.class.getResource("images/delete.png")));	
		Main.batchModifyMI = new JMenuItem("Batch Modify", new ImageIcon(Main.class.getResource("images/batch_modify.png")));	
	}
	
	/**
	 * Build Applet GUI Content
	 * @return JSplitPane containing APplets content
	 */
	public JSplitPane buildApplicationContent(){
              
        // Menu Panel (need to add Menu Bar to a panel in a JApplet)
        JPanel menupanel = new JPanel();
        menupanel.setLayout (new BorderLayout ());
        menupanel.add(buildMenuBar(), BorderLayout.WEST);

        // Table and sort panel split with toolbar and image scroll pane
        JSplitPane tableSortPanelAndImageScrollPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, buildTableAndSortPanel(), buildToolbarAndImageScrollpane());
        tableSortPanelAndImageScrollPane.setOneTouchExpandable(true);
        int dividerPosition = (int)((Main.screenDimensions.height*0.65)-80);
        tableSortPanelAndImageScrollPane.setDividerLocation(dividerPosition);
     
        // Metadata Panel
        buildTemporaryMetadataPanel();
        
        
        JPanel tabbedPanel = buildTabbedPanePanel();
       
        //Create a split pane with the tabbed pane panel and metadata table (and image scroll pane)
        JSplitPane contentPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, tableSortPanelAndImageScrollPane, tabbedPanel);
        contentPane.setOneTouchExpandable(true);
       
        // Start Horizontal Screen Divider at 60% of the screens width
        dividerPosition = (int)(Main.screenDimensions.width*0.60);
        
        // Adjust the divider and tabbed Panel Minimum size accordingly
        int tabbedPanelMinimumSize = 0;       
        if (Main.screenDimensions.width<1050){
        	dividerPosition = (int)(Main.screenDimensions.width*0.47);
        	tabbedPanelMinimumSize = (int)(Main.screenDimensions.width*0.47);
        }
        else{
        	tabbedPanelMinimumSize = (int)(Main.screenDimensions.width*0.35);
        }      
        
        contentPane.setDividerLocation(dividerPosition);
        tabbedPanel.setMinimumSize(new Dimension(tabbedPanelMinimumSize,  (int)(Main.screenDimensions.height-80)));

        JSplitPane menuAndContentSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT, menupanel, contentPane);
        menuAndContentSplit.setOneTouchExpandable(false);
        menuAndContentSplit.setDividerLocation(30);
        menuAndContentSplit.setDividerSize(1);
        menuAndContentSplit.setEnabled(false);
        
        return menuAndContentSplit;
	}
	
	/**
	 * Set Progress Bar's Colour
	 */
	public void setProgressBarColour(){
		Main.JProgressBarPanel.setBackground(new Color(0xE2E2E2));
		Main.JProgressBarPanelAssociateFiles.setBackground(new Color(0xE2E2E2));
		Main.JProgressBarPanelExportMetadata.setBackground(new Color(0xE2E2E2));
	}
	
	/**
	 * Set Progress Bar's Size
	 */
	public void setProgressBarSize(){
		Main.transformMetadataProgressBar.setPreferredSize(new Dimension(150,35));
		Main.associateFilesProgressBar.setPreferredSize(new Dimension(150,35));
		Main.exportMetadataProgressBar.setPreferredSize(new Dimension(150,35));
	}

	/**
	 * Configure Console
	 */
	public void configureConsole(){
		Main.console = new JTextPane();
		Main.console.setText("");
		Main.consoleDocument = Main.console.getStyledDocument();
		
		//  Define Console keyword attributes
		defineConsoleKeywordAttributes();
	}
	// ------------------------
	
	// Private Builder Functions
	// -----------------
	
	/**
	 * Build Menu Bar
	 * @return Menu Bar
	 */
	private JMenuBar buildMenuBar(){
		// Menu Bar
        JMenuBar menuBar = new JMenuBar();	
        menuBar.setPreferredSize(new Dimension((int)(Main.screenDimensions.width)+40,40));
        
        // Menus	
        JMenu fileMenu = new JMenu("File");
        JMenu editMenu = new JMenu("Edit");	
        JMenu toolsMenu = new JMenu("Tools");	
        JMenu helpMenu = new JMenu("Help");

        // Add Menus to menu bar
        menuBar.add(fileMenu);
        menuBar.add(editMenu);
        menuBar.add(toolsMenu);
        menuBar.add(helpMenu);
        
        // Add save information label to menu bar
        menuBar.add(Main.saveInformationLabel);

        // File Menu : Add items to file menu
        fileMenu.add(Main.saveMetadata);
        fileMenu.add(Main.saveMetadataAs);
        fileMenu.add(Main.loadMetadataMI);
        fileMenu.addSeparator();
        fileMenu.add(Main.openFilesMI);
        fileMenu.add(Main.openDirectoriesMI);
        fileMenu.addSeparator();
        fileMenu.add(Main.addFilesMI);
        fileMenu.add(Main.addDirectoriesMI);
        fileMenu.addSeparator();
        Main.importSubMenu.add(Main.importCoordinates);
        Main.importSubMenu.add(Main.addCameraCalibrationSettings);
        fileMenu.add(Main.importSubMenu);
        Main.exportSubMenu.add(Main.exportMI);
        fileMenu.add(Main.exportSubMenu);

        // Edit Menu : Add items to action menu
        editMenu.add(Main.removeCameraCalibrationSettings);
        editMenu.add(Main.calculateTitles);
        editMenu.add(Main.associateFilesMI);
        editMenu.add(Main.validateMetadata);
        editMenu.add(Main.batchModifyMI);
        
        // Tool Menu : add items to tool menu
        toolsMenu.add(Main.transformAlukaMetadata);
        
        // Help Menu : add items to help menu
        helpMenu.add(Main.aboutMI);
        
        return menuBar;
	}
	
	/**
	 * Build Image Viewer Toolbar
	 * @return Image viewer toolbar
	 */
	private JToolBar buildImageViewerToolBar(){
        JToolBar toolBar = new JToolBar();
        addToolbarButtons(toolBar);
        return toolBar;
	}
	
	/**
	 * Build Automation Tab
	 * @return Automation SplitPane
	 */
	private JSplitPane buildAutomationTab(){
		// Load fields
		DataDealer.LoadFields();
        
        JLabel itemLabel = new JLabel("Fields");
        Main.fieldComboBox = new JComboBox<String>(Main.fieldLabels);
        Main.fieldComboBox.setMaximumRowCount(5);
         
        Main.fieldComboBox.addActionListener (new ActionListener () {
    	    public void actionPerformed(ActionEvent e) {
    	    	Main.fieldComboBoxSelectedIndex = Main.fieldComboBox.getSelectedIndex();
    	    	GUIModifier.modifyItemFields();
    	    	Main.itemComboBoxSelectedIndex = Main.itemComboBox.getSelectedIndex();
    	    }
    	  });
        
        String[] labels2 = {};
        JLabel itemLabel2 = new JLabel("Items");
        Main.itemComboBox = new JComboBox<String>(labels2);
        Main.itemComboBox.setMaximumRowCount(5);

        Main.itemComboBox.setEditable(true);
        
        Main. previousItemComboBoxValue = Main.itemComboBox.getSelectedItem();
        Main.itemComboBoxSelectedIndex = Main.itemComboBox.getSelectedIndex();
        Main.fieldComboBoxSelectedIndex = Main.fieldComboBox.getSelectedIndex();

        Main.itemComboBox.addActionListener (new ActionListener () {
      	    public void actionPerformed(ActionEvent e) {
      	    	if (Main.itemComboBox.getSelectedIndex()!=-1){
      	    		Main.previousItemComboBoxValue = Main.itemComboBox.getSelectedItem();
      	    		Main.itemComboBoxSelectedIndex = Main.itemComboBox.getSelectedIndex();
      	    	}
      	    }
        });
        GUIModifier.modifyItemFields();

        
        Main.fieldComboBox.setPreferredSize( new Dimension( 200, 26 ) );
        Main.itemComboBox.setPreferredSize( new Dimension( 200, 26 ) );
        
        //--------------------------------------------------------------------------------                      

        
        JButton b2 = new JButton("UPDATE");
        b2.addActionListener (new ActionListener () {
      	    public void actionPerformed(ActionEvent e) {

      	    	Object selectedItem = Main.itemComboBox.getSelectedItem();
      	    	if (selectedItem!=null){
      	    		if (Main.items.get(Main.fieldComboBoxSelectedIndex)!=null){
	      	    		Main.items.get(Main.fieldComboBoxSelectedIndex)[Main.itemComboBoxSelectedIndex] = selectedItem.toString();
	      	    		Main.defaultComboBoxModel.setSelectedItem(Main.itemComboBox.getSelectedItem());
	          	    	
	          	    	DefaultComboBoxModel<String> model = new DefaultComboBoxModel<String>(Main.items.get(Main.fieldComboBoxSelectedIndex));
	          	    	Main.itemComboBox.setModel(model);
	              		
	          	    	Main.itemComboBox.setSelectedItem(selectedItem);
	              		
	          	    	DataDealer.modifyFieldName();
      	    		}
      	    	}
      	    }
      	  });
        
        
        JButton b3 = new JButton("REMOVE");
        b3.addActionListener (new ActionListener () {
    	    public void actionPerformed(ActionEvent e) {
    	    	 
    	    	if (Main.items.get(Main.fieldComboBoxSelectedIndex)!=null){
    	    		if (Main.items.get(Main.fieldComboBoxSelectedIndex).length>1){
            	    	String[] temp = new String[Main.items.get(Main.fieldComboBoxSelectedIndex).length-1];
            	    	int index = 0;
            	    	int index2 = 0;
            	    	for (String s : Main.items.get(Main.fieldComboBoxSelectedIndex)){
            	    		
            	    		if (index2!=Main.itemComboBoxSelectedIndex){
            	    			temp[index] = s;
            	    			index++;
            	    		}
            	    		index2++;
            	    	}

            	    	Main.items.set((Main.fieldComboBoxSelectedIndex) , temp);

          	    	DefaultComboBoxModel<String> model = new DefaultComboBoxModel<String>(Main.items.get(Main.fieldComboBoxSelectedIndex));
          	    	Main.itemComboBox.setModel(model);
              		
          	    	DataDealer.modifyFieldName();
    	    		}
    	    		else{
    	    			Main.items.set((Main.fieldComboBoxSelectedIndex) , new String[]{});
            	  	
            	  		DefaultComboBoxModel<String> model = new DefaultComboBoxModel<String>(Main.items.get(Main.fieldComboBoxSelectedIndex));
            	  		Main.itemComboBox.setModel(model);
            	  		
            	  		DataDealer.modifyFieldName();
    	    		}
    	    	}
    	    }
    	  });

        
        // Automation FIelds
        JPanel automationFieldsPanel = new JPanel(); 
        automationFieldsPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(10,5,5,5),BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Automation Fields"), BorderFactory.createEmptyBorder(20,5,5,5))));
        GroupLayout layout = new GroupLayout( automationFieldsPanel );
        automationFieldsPanel.setLayout( layout );
        layout.setAutoCreateGaps( true );
        
        // Automation Dictionary
        JPanel automationDictionaryPanel = new JPanel(); 
        automationDictionaryPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(0,5,5,5),BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Field Dictionary"), BorderFactory.createEmptyBorder(20,5,5,5))));
        GroupLayout layout2 = new GroupLayout( automationDictionaryPanel );
        automationDictionaryPanel.setLayout( layout2 );
        layout2.setAutoCreateGaps( true );
        
        
        Main.addItemTextField = new JTextField(10);
        JButton addButton = new JButton("ADD New Item");
        addButton.addActionListener (new ActionListener () {
    	  @SuppressWarnings("unused")
		public void actionPerformed(ActionEvent e) {
    		    // Check to see that text has been entered and that an item doesn't already exist with that name
    		  	if (!Main.addItemTextField.getText().equals("") && !Utility.arrayContainsNull(Main.addItemTextField.getText(), Main.items.get(Main.fieldComboBox.getSelectedIndex()))){
        	    	if (Main.items.get(Main.fieldComboBox.getSelectedIndex())==null){
        	    		String[] temp = new String[]{ Main.addItemTextField.getText()};
        	    		Main.items.set(Main.fieldComboBox.getSelectedIndex(), temp);
        	    		DataDealer.modifyFieldName();
        	    		DefaultComboBoxModel<String> model = new DefaultComboBoxModel<String>(Main.items.get(Main.fieldComboBox.getSelectedIndex()));
        	    		Main.itemComboBox.setModel(model);	
        	    	}
        	    	else{
        	    		String[] temp = new String[Main.items.get(Main.fieldComboBox.getSelectedIndex()).length+1];
        	    		temp[0] = Main.addItemTextField.getText();
        	    		int index = 1;
        	    		for (String s : Main.items.get(Main.fieldComboBox.getSelectedIndex())){
        	    			temp[index] = Main.items.get(Main.fieldComboBox.getSelectedIndex())[index-1];
        	    			index++;
        	    		}
        	    		Main.items.set(Main.fieldComboBox.getSelectedIndex(), temp);
        	    		DataDealer.modifyFieldName();
       	    		DefaultComboBoxModel<String> model = new DefaultComboBoxModel<String>(Main.items.get(Main.fieldComboBox.getSelectedIndex()));
       	    		Main.itemComboBox.setModel(model);
        	    	}
    		  	}

    	    }
        });
        
        Main.addItemTextField.setPreferredSize( new Dimension( 100, 27 ) );
        JLabel sharedDrivePathServerLabel = new JLabel("Path to Shared Drive (Server)");
        Main.sharedDrivePathOnServer = new JTextField(10);
        Main.sharedDrivePathOnServer.setPreferredSize( new Dimension( 100, 27 ) );
        Main.sharedDrivePathOnServer.setText("/mnt/shares/zamani");
        
        JLabel identifierLabel = new JLabel("Identifier Prefix");
        Main.identifierPrefix = new JTextField(10);
        Main.identifierPrefix.setPreferredSize( new Dimension( 100, 27 ) );
        Main.identifierPrefix.setText("uctnew");
                      
        JLabel sourceLabel = new JLabel("Source");
        Main.sourceOfData = new JTextField(10);
        Main.sourceOfData.setPreferredSize( new Dimension( 100, 27 ) );
        Main.sourceOfData.setText("University of Cape Town, Geomatics Department");
         
        JLabel coverageSpatialCountryLabel = new JLabel("Coverage Spatial Country");
        Main.coverageSpatialCountry = new JTextField(10);
        Main.coverageSpatialCountry.setPreferredSize( new Dimension( 100, 27 ) );
        Main.coverageSpatialCountry.setText("Sudan");
        
        JLabel coverageSpatialSitenameLabel = new JLabel("Coverage Spatial Name");
        Main.coverageSpatialSitename = new JTextField(10);
        Main.coverageSpatialSitename.setPreferredSize( new Dimension( 100, 27 ) );
        Main.coverageSpatialSitename.setText("Musawwarat es-Sufra");
        
        JLabel coverageSpatialStructureLabel = new JLabel("Coverage Spatial Structure");
        Main.coverageSpatialStructure = new JTextField(10);
        Main.coverageSpatialStructure.setPreferredSize( new Dimension( 100, 27 ) );
        Main.coverageSpatialStructure.setText("Great Enclosure");

        // Add components to Automation Fields Panel
        //----------------------------------------------
        // Horizontally
        layout.setHorizontalGroup( layout.createSequentialGroup()
			.addGroup( layout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( sharedDrivePathServerLabel )								
				.addComponent( identifierLabel )
				.addComponent( sourceLabel )
				.addComponent( coverageSpatialCountryLabel )
				.addComponent( coverageSpatialSitenameLabel )
				.addComponent( coverageSpatialStructureLabel ))
			                    
			.addGroup( layout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( Main.sharedDrivePathOnServer )								
				.addComponent( Main.identifierPrefix )
				.addComponent( Main.sourceOfData )
				.addComponent( Main.coverageSpatialCountry )
				.addComponent( Main.coverageSpatialSitename )
				.addComponent( Main.coverageSpatialStructure ))
        );

        // Vertically
        layout.setVerticalGroup( layout.createSequentialGroup()
         .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )
            .addComponent( sharedDrivePathServerLabel )
            .addComponent( Main.sharedDrivePathOnServer ))
         .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )
            .addComponent( identifierLabel )
            .addComponent( Main.identifierPrefix ))
         .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )
            .addComponent( sourceLabel )
            .addComponent( Main.sourceOfData ))
         .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )
            .addComponent( coverageSpatialCountryLabel )
            .addComponent( Main.coverageSpatialCountry ))
         .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )
            .addComponent( coverageSpatialSitenameLabel )
            .addComponent( Main.coverageSpatialSitename ))
         .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )
            .addComponent( coverageSpatialStructureLabel )
            .addComponent( Main.coverageSpatialStructure )) 	                                                                         
        ); 
        
        
        // Add components to Automation Dictionary Panel
        //----------------------------------------------
        // Horizontally
        layout2.setHorizontalGroup( layout2.createSequentialGroup()
			.addGroup( layout2.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( itemLabel )
				.addComponent( itemLabel2 ))
			                    
			.addGroup( layout2.createParallelGroup( GroupLayout.Alignment.LEADING )	
				.addComponent( Main.fieldComboBox )
				.addComponent( Main.itemComboBox )
				.addComponent( Main.addItemTextField ) )
			                  
			.addGroup( layout2.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( b2 )
				.addComponent( addButton ))
			                  
			.addGroup( layout2.createParallelGroup( GroupLayout.Alignment.LEADING )              
				.addComponent( b3 ))
        );

        // Vertically
        layout2.setVerticalGroup( layout2.createSequentialGroup()
         .addGroup( layout2.createParallelGroup( GroupLayout.Alignment.BASELINE )
            .addComponent( itemLabel )
            .addComponent( Main.fieldComboBox ))
         .addGroup( layout2.createParallelGroup( GroupLayout.Alignment.BASELINE )
            .addComponent( itemLabel2 )
            .addComponent( Main.itemComboBox )
            .addComponent( b2 )
            .addComponent( b3 ))
         .addGroup( layout2.createParallelGroup( GroupLayout.Alignment.BASELINE )
            .addComponent( Main.addItemTextField )
            .addComponent( addButton ))                                                                        
        ); 
        
        JSplitPane automationDictionarySplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, automationDictionaryPanel, new JPanel());
        automationDictionarySplitPane.setOneTouchExpandable(false);
        automationDictionarySplitPane.setDividerLocation(180);
        automationDictionarySplitPane.setEnabled(false);
        
        JSplitPane automationFieldsAndDictionarySplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, automationFieldsPanel, automationDictionarySplitPane);
        automationFieldsAndDictionarySplitPane.setOneTouchExpandable(false);
        automationFieldsAndDictionarySplitPane.setDividerLocation(300);
        automationFieldsAndDictionarySplitPane.setEnabled(false);
        
        return automationFieldsAndDictionarySplitPane;
	}
	
	/**
	 * Build Console
	 * @return Console Panel
	 */
	private JPanel buildConsoleTab(){
        JPanel consolePanel = new JPanel(false);                      
        Main.console.setEditable(false);
        JScrollPane scrollPane2 = new JScrollPane(Main.console,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        consolePanel.setLayout(new GridLayout(1, 1));
        consolePanel.add(scrollPane2);
        return consolePanel;
	}
	
	/**
	 * Build Table
	 * @return Table contained within a JScrollPane
	 */
	private JScrollPane buildTable(){
		Main.booleanTableModel = new BooleanTableModel();
		Main.table = new JTable(Main.booleanTableModel);
		Main.table.setRowHeight(40);
		((DefaultTableCellRenderer)Main.table.getTableHeader().getDefaultRenderer())
	    .setHorizontalAlignment(JLabel.LEFT);
				
		Main.table.getColumnModel().getColumn(0).setMinWidth(35);
		Main.table.getColumnModel().getColumn(0).setMaxWidth(35);
	    
		Main.table.getColumnModel().getColumn(2).setMinWidth(45);
		Main.table.getColumnModel().getColumn(2).setMaxWidth(45);
	    
	    // Table Mouse Listener
		Main.table.addMouseListener(new MouseAdapter() {
	    	  public void mouseClicked(MouseEvent e) {
	    	    
	    	      JTable target = (JTable)e.getSource();
	    	      try {
	    	    	  GUIModifier.setImageInScrollPane(target, 0, false);
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} 
	    	  }
	    });
	    
	    // Overide Tables VK_UP and VK_DOWN to nothing (set in Table Keyboard Listener)
		Main.table.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0), "up");
		Main.table.getActionMap().put("up", new AbstractAction() {
			private static final long serialVersionUID = 2521506560037797874L;

			@Override
            public void actionPerformed(ActionEvent e) {}
        });
		Main.table.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0), "down");
		Main.table.getActionMap().put("down", new AbstractAction() {
            /**
			 * 
			 */
			private static final long serialVersionUID = 5287786111614523408L;

			@Override
            public void actionPerformed(ActionEvent e) {}
        });
	    
	    // Table Keyboard Listener
		Main.table.addKeyListener(new KeyAdapter() {
	        @Override
	        public void keyPressed(KeyEvent e){

		        int keyCode = e.getKeyCode();
		        switch( keyCode ) { 
		            case KeyEvent.VK_UP:
		                JTable target = (JTable)e.getSource();
					try {
						GUIModifier.setImageInScrollPane(target, -1, true);
					} catch (Exception e2) {
						// TODO Auto-generated catch block
						e2.printStackTrace();
					}
		                break;
		            case KeyEvent.VK_DOWN:
		                JTable target2 = (JTable)e.getSource();
					try {
						GUIModifier.setImageInScrollPane(target2, 1, true);
					} catch (Exception e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
		                break;
		         }	
	        }
	    });
	    
		Main.table.setDefaultRenderer(Object.class, new DefaultTableCellRenderer()
		{
			private static final long serialVersionUID = 6900974631464221495L;

			@Override
		    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
		    {
		        final Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);	
		          
		        if (isSelected && column!=2){
		        	setBorder(new LineBorder(Color.RED));
		        }
		        
		    	return GUIModifier.setCellColours(table, value, isSelected, hasFocus, row, column, c);
		    }
		});

		Main.table.setRowSelectionAllowed(true);
		Main.table.setColumnSelectionAllowed(false);
	    
       JScrollPane scroll = new JScrollPane(Main.table);
        
	    return scroll;
}
	
	/**
	 * Build Table and Sort Panel
	 * @return Panel containing Metadata Table and Sort Panel Split Pane
	 */
	private JPanel buildTableAndSortPanel(){
        JPanel metadataTableAndSortPanel = new JPanel();
        metadataTableAndSortPanel.setLayout(new GridLayout(1,1));
        metadataTableAndSortPanel.add(buildMetadataTableAndSortPane());
        return metadataTableAndSortPanel;
	}
	  
	/**
	 * Add Toolbar Buttons to toolbar
	 */
	private void addToolbarButtons(JToolBar toolBar) {
	    JButton button = null;
	 
	    //first button
	    button = buildNavigationButton(Main.class.getResource("images/Back24.png"), PREVIOUS, "Back to previous metadata item","Previous");
		button.addActionListener(new ActionListener() {	 
	        public void actionPerformed(ActionEvent e)
	        {
	        	try {
	        		GUIModifier.setImageInScrollPane(Main.table, -1, true);
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
	        }
	    });  
	    toolBar.add(button);
	 
	    //second button
	    button = buildNavigationButton(Main.class.getResource("images/Forward24.png"), NEXT, "Forward to next metadata item", "Next");
		button.addActionListener(new ActionListener() {	 
	        public void actionPerformed(ActionEvent e)
	        {
	        	try {
	        		GUIModifier.setImageInScrollPane(Main.table, 1, true);
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
	        }
	    }); 
	    toolBar.add(button);
	    
	    //third button
	    button = buildNavigationButton(Main.class.getResource("images/zoom_in.PNG"), ZOOM_IN, "zoom in", "zoom_in");
		button.addActionListener(new ActionListener() {	 
	        public void actionPerformed(ActionEvent e)
	        {
	        	try {
					GUIModifier.zoomIn();
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
	        }
	    }); 
	    toolBar.add(button);
	    
	    //fourth button
	    button = buildNavigationButton(Main.class.getResource("images/zoom_out.PNG"), ZOOM_OUT, "zoom out", "zoom_out");
		button.addActionListener(new ActionListener() {	 
	        public void actionPerformed(ActionEvent e)
	        {
	        	try {
	        		GUIModifier.zoomOut();
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
	        }
	    }); 
	    toolBar.add(button);
	    
	    //fifth button
	    button = buildNavigationButton(Main.class.getResource("images/best_fit.PNG"), BEST_FIT, "best fit", "best_fit");
		button.addActionListener(new ActionListener() {	 
	        public void actionPerformed(ActionEvent e)
	        {
	        	Main.imagePanel.setBestFit(true,Main.imageScrollPane.getSize());
	        }
	    }); 
	    toolBar.add(button);
	    
	    //sixth button
	    button = buildNavigationButton(Main.class.getResource("images/normal_size.PNG"), NORMAL_SIZE, "normal size", "normal_size");
		button.addActionListener(new ActionListener() {	 
	        public void actionPerformed(ActionEvent e)
	        {
	        	Main.imagePanel.setNormalSize();
	        }
	    }); 
	    toolBar.add(button);
	}    
	  
	/**
	 * Build Navigation Button
	 * @return Navigation Button
	 */
	private static JButton buildNavigationButton(URL imageURL,final String actionCommand, String toolTipText, String altText) {

			//Create and initialize the button.
			JButton button = new JButton();
			button.setActionCommand(actionCommand);
			button.setToolTipText(toolTipText);
			
			if (imageURL != null) {                      										//image found
				button.setIcon(new ImageIcon(imageURL, altText));
			}
			else {                                     											//no image found
				button.setText(altText);
			}
			
			return button;
	}
	
	/**
	 * Build Image Scroll Pane
	 * @return JSplitPane containing Toolbar and Image Scroll Pane
	 */
	private JSplitPane buildToolbarAndImageScrollpane(){
		// Create new Image Scaling Panel and add it to the image scroll pane
		Main.imagePanel = new ImageScalingPanel();
		Main.imageScrollPane = new JScrollPane(Main.imagePanel);
		
        // Add ToolBar and Image Scroll Pane to a JSplitPane
        JSplitPane toolBarAndImageScrollPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, buildImageViewerToolBar(), Main.imageScrollPane);
        toolBarAndImageScrollPane.setOneTouchExpandable(false);
        toolBarAndImageScrollPane.setDividerLocation(30);
        toolBarAndImageScrollPane.setEnabled(false);	//Disable resizing
        toolBarAndImageScrollPane.setMinimumSize(new Dimension(300, 150));
        return toolBarAndImageScrollPane;
	}
	
	private void buildTemporaryMetadataPanel(){
        JPanel tempMetaPanel = new JPanel();
        tempMetaPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(10,5,5,5),BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Metadata Record"), BorderFactory.createEmptyBorder(10,30,10,20))));

        Main.metadataPanel = tempMetaPanel;                      
        Main.metadataScrollPane = new JScrollPane(Main.metadataPanel); 
	}
	
	
	/**
	 * Build Tabbed Pane Panel
	 * @return JPanel containing Tabbed Pane 
	 */
	private JPanel buildTabbedPanePanel(){
		
        // Create tabbed Pane panel
        JPanel tabbedPanePanel = new JPanel();
        tabbedPanePanel.setLayout(new GridLayout(1,1));
		
        Main.tabbedPane = new JTabbedPane();
        
        Main.tabbedPane.addTab("Automate", null, buildAutomationTab());
        Main.tabbedPane.setMnemonicAt(0, KeyEvent.VK_1);
        
        Main.tabbedPane.addTab("Metadata", null, Main.metadataScrollPane);
        Main.tabbedPane.setMnemonicAt(1, KeyEvent.VK_2);
        
        Main.tabbedPane.addTab("Console", null, buildConsoleTab());
        Main.tabbedPane.setMnemonicAt(2, KeyEvent.VK_3);
        
        tabbedPanePanel.add(Main.tabbedPane);
        
        tabbedPanePanel.setMinimumSize( new Dimension(300, 50));
        
        return tabbedPanePanel;       
	}
	
	/**
	 * Build Metadata Table and Sort Split Pane
	 * @return JSplitPane containing Metadata Table and Sort Panel
	 */
	private JSplitPane buildMetadataTableAndSortPane(){
        //Overview Panel------------------------------------
        JPanel panel1_1_1 = new JPanel(false); 
        panel1_1_1.setLayout(new GridLayout(2,1));
        
        Main.identityButton   = new JRadioButton("Identity"  , true);
        JRadioButton fileButton    = new JRadioButton("File"   , false);
        JRadioButton statusButton    = new JRadioButton("Status"   , false);
        JRadioButton tagButton = new JRadioButton("Tag", false);
        JRadioButton containsButton = new JRadioButton("Contains", false);
        
        ButtonGroup bgroup = new ButtonGroup();
        bgroup.add(Main.identityButton);
        bgroup.add(fileButton);
        bgroup.add(statusButton);
        bgroup.add(tagButton);
        bgroup.add(containsButton);
        
        // Identity Radio Button Listener
        Main.identityButton.addActionListener (new ActionListener () {
      	    public void actionPerformed(ActionEvent e) {
      	    	Main.currentSort = "Identity";
      	    	Main.tagComboBox.setEnabled(false);  
                Main.tagSortText.setEnabled(false);
				Collections.sort(Main.idAndFileTypeList,new Comparator<String[]>() {
		            public int compare(String[] strings, String[] otherStrings) {
		            	// index 0 contains the ID
		                return strings[0].compareTo(otherStrings[0]);
		            }
		        });
				
				DataDealer.sortData();	
				DataDealer.resetIndexesOfSortedData();
      	    }
        });
        // File Radio Button Listener
        fileButton.addActionListener (new ActionListener () {
    	    public void actionPerformed(ActionEvent e) {
    	    	Main.currentSort = "File";
    	    	Main.tagComboBox.setEnabled(false);  
                Main.tagSortText.setEnabled(false);
				Collections.sort(Main.idAndFileTypeList,new Comparator<String[]>() {
		            public int compare(String[] strings, String[] otherStrings) {
		            	// index 1 contains the filename
		                return strings[1].compareTo(otherStrings[1]);
		            }
		        });
				
				DataDealer.sortData();	
				DataDealer.resetIndexesOfSortedData();
    	    }
        });
        
        // Status Radio Button Listener
        statusButton.addActionListener (new ActionListener () {
    	    public void actionPerformed(ActionEvent e) {
    	    	Main.currentSort = "Status";
    	    	Main.tagComboBox.setEnabled(false);  
                Main.tagSortText.setEnabled(false);
				Collections.sort(Main.idAndFileTypeList,new Comparator<String[]>() {
		            public int compare(String[] strings, String[] otherStrings) {
		            	// index 3 contains the percentage complete (Status)
		            	// ascending order
		                return Integer.parseInt(strings[3]) - Integer.parseInt(otherStrings[3]);
		                
		                //descending order
		                //return Integer.parseInt(otherStrings[3]) - Integer.parseInt(strings[3]);
		            }
		        });
				
				DataDealer.sortData();	
				DataDealer.resetIndexesOfSortedData();
    	    }
        });
        
        // Tag Radio Button Listener
        tagButton.addActionListener (new ActionListener () {
    	    public void actionPerformed(ActionEvent e) {
    	    	Main.currentSort = "Tag";
    	    	Main.tagComboBox.setEnabled(true);  
                Main.tagSortText.setEnabled(true);
    	    	if (Main.tagSortText.getText()!=null){
    	    		String tagSortTextTemp = Main.tagSortText.getText();
    	    		if (!tagSortTextTemp.equals("")){
    	    			Main.tagSortText.setText(tagSortTextTemp);
    	    		}
    	    	}
    	    }
        });
        // Contains Radio Button Listener
        containsButton.addActionListener (new ActionListener () {
    	    public void actionPerformed(ActionEvent e) {
    	    	Main.currentSort = "Contains";
    	    	Main.tagComboBox.setEnabled(true);  
                Main.tagSortText.setEnabled(true);
    	    	if (Main.tagSortText.getText()!=null){
    	    		String tagSortTextTemp = Main.tagSortText.getText();
    	    		if (!tagSortTextTemp.equals("")){
    	    			Main.tagSortText.setText(tagSortTextTemp);
    	    		}
    	    	}
    	    }
        });
        
        
        JPanel radioPanel = new JPanel();
        radioPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Sort by"), BorderFactory.createEmptyBorder(5,5,5,5)));
        radioPanel.setLayout(new GridLayout(1, 4));
        radioPanel.add(Main.identityButton);
        radioPanel.add(fileButton);
        radioPanel.add(statusButton);
        radioPanel.add(tagButton);
        radioPanel.add(containsButton);

        //-----------------------------

        panel1_1_1.add(radioPanel);
        
        DataDealer.LoadFields();

        JPanel tagPanel1 = new JPanel();

        GroupLayout tagLayout = new GroupLayout(tagPanel1);
        tagPanel1.setLayout( tagLayout );
        tagLayout.setAutoCreateGaps( true );
        
        JPanel tagPanel = new JPanel();
        tagPanel.setLayout(new GridLayout(1, 2));
        
        Main.tagComboBox = new JComboBox<String>(Main.tagSortDataset);
        Main.tagComboBox.setMaximumRowCount(5);
        Main.tagComboBox.setPreferredSize( new Dimension( 50, 26 ) );
        tagPanel.add(Main.tagComboBox);
        
        Main.tagSortText = new JTextField();
        Main.tagSortText.setPreferredSize( new Dimension( 50, 27 ) );
        
        Main.tagSortText.getDocument().addDocumentListener(new DocumentListener() {
      	  public void changedUpdate(DocumentEvent e) {
      		  containOrTagSort();
      	  }
      	  public void removeUpdate(DocumentEvent e) {
      		  containOrTagSort();
      	  }
      	  public void insertUpdate(DocumentEvent e) {
      		  containOrTagSort();
      	  }

      	  @SuppressWarnings("unused")
		public void containOrTagSort() {
      	    
      	    int index = 0;
      	    
      	    ArrayList<Integer> indexes = new ArrayList<Integer>();
      	    if (Main.currentSort.equals("Tag")){
      	    	
          	    for (ArrayList<String> al : Main.metadata){
          	    	int i = Main.metadataFields.get(index).indexOf(Main.tagComboBox.getSelectedItem().toString());
          	    	
          	    	if (al.get(i)!=null){
              	    	if (al.get(i).equals(Main.tagSortText.getText())){
              	    		indexes.add(index);
              	    	}
          	    	}
          	    	index++;
          	    }
      	    }
      	    else if (Main.currentSort.equals("Contains")){
      	    	
          	    for (ArrayList<String> al : Main.metadata){
          	    	
          	    	int i = Main.metadataFields.get(index).indexOf(Main.tagComboBox.getSelectedItem().toString());
          	    	
          	    	if (al.get(i)!=null){
              	    	if (al.get(i).contains(Main.tagSortText.getText())){
              	    		indexes.add(index);
              	    	}
          	    	}
          	    	index++;
          	    }
      	    }
      	    if (Main.currentSort.equals("Contains") || Main.currentSort.equals("Tag")){
      	    	
      	    	// If there is actually data
      	    	if (Main.data!=null){
      	    		if (Main.data.length>0){
		  	    		index = 0;
					    ArrayList<File> tempFilesInDataset = new ArrayList<File>();
					    ArrayList<ArrayList<String>> tempMetadataFields = new ArrayList<ArrayList<String>>();
					    ArrayList<ArrayList<String>> tempMetadata = new ArrayList<ArrayList<String>>();
					    Object[][] tempData = new Object[Main.data.length][2];	
					    ArrayList<String[]> tempIDAndFileTypeList = new ArrayList<String[]>();
						ArrayList<String> tempListOfFilePathsInDataset = new ArrayList<String>();
						ArrayList<String> tempListOfFileCorrespondingPIDSInDataset = new ArrayList<String>();
		
					    // Add contained indexes first
				    	for (int i : indexes){ 
				    		tempIDAndFileTypeList.add(Main.idAndFileTypeList.get(i));
				    		tempFilesInDataset.add(Main.filesInDataset.get(i));	
				    		tempMetadataFields.add(Main.metadataFields.get(i));
				    		tempMetadata.add(Main.metadata.get(i));
				    		tempData[index] = Main.data[i]; 
				    		tempListOfFilePathsInDataset.add(Main.listOfFilePathsInDataset.get(i));
				    		tempListOfFileCorrespondingPIDSInDataset.add(Main.listOfFileCorrespondingPIDSInDataset.get(i));
				    		index++;
				    	}
		
				    	int length = index;
				    	Main.length_constant = index;
				    	
				    	index = 0;
				    	// Add the rest of the indexes
				    	for (String[] s : Main.idAndFileTypeList){
				    		if (!indexes.contains(index)){
				    			tempIDAndFileTypeList.add(Main.idAndFileTypeList.get(index));
					    		tempFilesInDataset.add(Main.filesInDataset.get(index));	
					    		tempMetadataFields.add(Main.metadataFields.get(index));
					    		tempMetadata.add(Main.metadata.get(index));
					    		tempData[length] = Main.data[index]; 
					    		tempListOfFilePathsInDataset.add(Main.listOfFilePathsInDataset.get(index));
					    		tempListOfFileCorrespondingPIDSInDataset.add(Main.listOfFileCorrespondingPIDSInDataset.get(index));
					    		length++;
				    		}
				    		index++;	
				    	}
				    	
				    	Main.idAndFileTypeList = tempIDAndFileTypeList;
				    	Main.filesInDataset = tempFilesInDataset;
						Main.metadataFields = tempMetadataFields;
						Main.metadata = tempMetadata;
						Main.data = tempData;
						Main.listOfFilePathsInDataset = tempListOfFilePathsInDataset;
					    Main.listOfFileCorrespondingPIDSInDataset = tempListOfFileCorrespondingPIDSInDataset;
						
					    Main.table.setDefaultRenderer(Object.class, new DefaultTableCellRenderer()
						{
							private static final long serialVersionUID = -4952791163281456613L;
		
							@Override
						    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
						    {
						        final Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
						        
						        if (isSelected && column!=2){
						        	setBorder(new LineBorder(Color.RED));
						        }
							       
						    	if (row < Main.length_constant && column==1){
						    		c.setBackground(Color.LIGHT_GRAY);
						    	}
						    	else if (row < Main.length_constant && column==2){
					    			c.setBackground(Utility.getColor(Double.parseDouble(Main.idAndFileTypeList.get(row)[3])));
						    	}
						    	else if (row >= Main.length_constant && (column==1 || column==2)){
						    		if (column==2){
						    			c.setBackground(Utility.getColor(Double.parseDouble(Main.idAndFileTypeList.get(row)[3])));
						    		}
						    		else{
						    			c.setBackground(Color.WHITE);
						    		}
						    	}
						    	return c;
						    }
						});
						
						Main.booleanTableModel.data = Main.data;
						Main.table.setModel(Main.booleanTableModel);	
				    	Main.booleanTableModel.fireTableDataChanged();
		    	
				    	DataDealer.resetIndexesOfSortedData();
      	    		}
      	    	}
          	}
      	  }
      	});
        
        tagPanel.add(Main.tagSortText);
        
        
        // Horizontally
        tagLayout.setHorizontalGroup( tagLayout.createSequentialGroup()
			.addGroup( tagLayout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( radioPanel )
				.addComponent( tagPanel )) 
				
        );

        // Vertically
        tagLayout.setVerticalGroup( tagLayout.createSequentialGroup()
         .addGroup( tagLayout.createParallelGroup( GroupLayout.Alignment.BASELINE )
            .addComponent( radioPanel ))
         .addGroup( tagLayout.createParallelGroup( GroupLayout.Alignment.BASELINE )
            .addComponent( tagPanel ))  
        );
        
        JScrollPane cc = buildTable();
        
        
        //Split panel1_1---------------
        JSplitPane panel1_1_s = new JSplitPane(JSplitPane.VERTICAL_SPLIT, tagPanel1, cc);
        panel1_1_s.setOneTouchExpandable(false);
        panel1_1_s.setDividerLocation(105);

        Dimension minimumSize2 = new Dimension(100, 30);
        
        //Provide minimum sizes for the two components in the split pane
        minimumSize2 = new Dimension(100, 30);
        panel1_1_1.setMinimumSize(minimumSize2);                
        cc.setMinimumSize(minimumSize2);
        panel1_1_s.setEnabled(false);
        
        return panel1_1_s;
	}
 
	/**
	 * Define Console Keyword Attributes
	 */
	private void defineConsoleKeywordAttributes(){
		Main.infoAttributes = new SimpleAttributeSet();
		StyleConstants.setForeground(Main.infoAttributes, Color.GREEN);
		
		Main.errorAttributes = new SimpleAttributeSet();
		StyleConstants.setForeground(Main.errorAttributes, Color.RED);
		
		Main.boldAttributes = new SimpleAttributeSet();
		StyleConstants.setBold(Main.boldAttributes, true);
	}
	// ------------------
}