package zamani.metafy.core.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.GroupLayout.ParallelGroup;
import javax.swing.GroupLayout.SequentialGroup;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.JTextComponent;

import zamani.metafy.core.Main;
import zamani.metafy.core.utility.FileTypes;
import zamani.metafy.core.utility.Utility;

/** 
 * GUIModifier utility class
 * Contains GUI functions relating to the dynamic modification of the GUI
 * 
 * @author Michael Ferguson
 * 
 */
public class GUIModifier {

    /**
     * Non instantiable Constructor
    */
	private GUIModifier(){}
	
    /**
     * Modify the Item combobox items when an item in the Field combobox is selected
    */
    protected static void modifyItemFields() {
        if (Main.fieldComboBox.getSelectedItem() != null) {

            int index = -1;
           
            for ( int i = 0; i<Main.fieldLabels.length; i++){  
            	if (Main.fieldComboBox.getSelectedItem().toString().equals(Main.fieldLabels[i])){
            		index = i;
            	} 	
            }

            if (index != -1){
            	if (Main.items.get(index)!=null){
            		DefaultComboBoxModel<String> model = new DefaultComboBoxModel<String>(Main.items.get(index));
            		Main.defaultComboBoxModel = model;
            		Main.itemComboBox.setModel( model );
            	}
            	else{
            		String[] empty = {};
               		DefaultComboBoxModel<String> model = new DefaultComboBoxModel<String>(empty);
               		Main.defaultComboBoxModel = model;
               		Main.itemComboBox.setModel( model );
            	}
            } 
        }
    }  
	
	/**
	 * Populate Metadata Panel
	 * @param  labels  String[] String array containing metadata labels
	 * @return JPanel containing populated metadata panel
	 */
	@SuppressWarnings("unchecked")
	public static JPanel populateMetadataPanel(String[] labels){

		int numPairs = labels.length;

	    //Create and populate the panel.
	    JPanel p = new JPanel();

	    p.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(10,5,5,5),BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Metadata Record"), BorderFactory.createEmptyBorder(10,30,10,20))));

	    GroupLayout layout = new GroupLayout(p);
	    p.setLayout(layout);
	    layout.setAutoCreateGaps(true);
	    
	    
	    SequentialGroup horizontalSeqGroup = layout.createSequentialGroup();
	    SequentialGroup verticalSeqGroup = layout.createSequentialGroup();
	    
	    ParallelGroup horizontalParallelGroup = layout.createParallelGroup(GroupLayout.Alignment.LEADING);
	    ParallelGroup horizontalParallelGroup2 = layout.createParallelGroup(GroupLayout.Alignment.LEADING);
	    
	    for (int i = 0; i < numPairs; i++) {
	        JLabel label = new JLabel(labels[i], JLabel.TRAILING);
	        
	        boolean isComboBox = Utility.arrayContains(labels[i], Main.fieldLabels);
	        
	        if (isComboBox){
	        	int indexOfField = Utility.getElementIndex(labels[i], Main.fieldLabels);
	        	String[] listOfFieldItems = Main.items.get(indexOfField);
	        	if (listOfFieldItems==null){
	        		isComboBox = false;
	        	}
	        	// No Combobox for creators
	        	if (labels[i].equals("Creator")){
	        		isComboBox = false;
	        	}
	        }
	        
	        if (isComboBox){
	        	
	        	ParallelGroup verticalParallelGroup = layout.createParallelGroup(GroupLayout.Alignment.BASELINE);
	        	int indexOfField = Utility.getElementIndex(labels[i], Main.fieldLabels);
	        	String[] listOfFieldItems = Main.items.get(indexOfField);
	        	
	        	JComboBox<String> combobox = new JComboBox<String>(Utility.cloneArrayWithBlankFirstIndex(listOfFieldItems));
	        	combobox.setRenderer(new RemoveFirstRowRenderer(combobox.getRenderer()));
	        	 JTextComponent tc = (JTextComponent) combobox.getEditor().getEditorComponent();
	            
	        	 tc.addKeyListener(new KeyListener(){

					@Override
					public void keyPressed(KeyEvent arg0) {}

					@Override
					public void keyReleased(KeyEvent arg0) {
						updateMetadataPanelStatus(Main.table.getSelectedRow());
						Main.stateChanged = true;
					}

					@Override
					public void keyTyped(KeyEvent arg0) {}
	        		
	        	});
	        	
	    	    ItemListener itemListener = new ItemListener() {
	    	        public void itemStateChanged(ItemEvent itemEvent) {
	    	          int state = itemEvent.getStateChange();
	    	           if (state == ItemEvent.SELECTED){
	    	        	   if (Main.stateChanged){
	    	        		   Main.stateChanged = false;
	    	        	   }
	    	        	   else{
	    	        		   updateMetadataPanelStatus(Main.table.getSelectedRow());
	    	        	   }
	    	           }
	    	        }
	    	      };
	    	      combobox.addItemListener(itemListener);

	    	      combobox.setEditable(true);
		        // Set text for textfield if it has already been set
				if (Main.metadata.get(Main.table.getSelectedRow()).size()!=0){
					if (Main.metadata.get(Main.table.getSelectedRow()).get(i)!=""){
						combobox.getEditor().setItem(Main.metadata.get(Main.table.getSelectedRow()).get(i));
					}
				}
	            
		        //horizontally
		        horizontalParallelGroup.addComponent(label);
		        horizontalParallelGroup2.addComponent(combobox);
		        
		        // Vertically
		        verticalSeqGroup.addGroup(verticalParallelGroup.addComponent(label).addComponent(combobox));
	        }
	        else{     
		        JTextField textField = new JTextField(10);
		        if (labels[i].equals("Identifier")){
		        	textField.setEditable(false);
		        }
		        
		        ParallelGroup verticalParallelGroup = layout.createParallelGroup(GroupLayout.Alignment.BASELINE);
		        
		        // Set text for textfield if it has already been set
		        if (Main.table.getSelectedRow()>-1){
					if (Main.metadata.get(Main.table.getSelectedRow()).size()!=0){
						if (Main.metadata.get(Main.table.getSelectedRow()).get(i)!=""){
							textField.setText(Main.metadata.get(Main.table.getSelectedRow()).get(i));
						}
					}
		        }
		        
		        textField.setPreferredSize( new Dimension( 200, 30 ) );
		       
		        textField.getDocument().addDocumentListener(new DocumentListener() {
				  public void changedUpdate(DocumentEvent e) {
					updateMetadata(e);
				  }
				  public void removeUpdate(DocumentEvent e) {
					updateMetadata(e);
				  }
				  public void insertUpdate(DocumentEvent e) {
					updateMetadata(e);
				  }
		      	  
					public void updateMetadata(DocumentEvent e) {
						updateMetadataPanelStatus(Main.table.getSelectedRow());
					}
		      	
		        });
		        
		        //horizontally
		        horizontalParallelGroup.addComponent(label);
		        horizontalParallelGroup2.addComponent(textField);
		        
		        // Vertically
		        verticalSeqGroup.addGroup(verticalParallelGroup.addComponent(label).addComponent(textField));
	        }
	    }
	    
	    horizontalSeqGroup
		.addGroup(horizontalParallelGroup)
		.addGroup(horizontalParallelGroup2);
	    
	    // Horizontally
	    layout.setHorizontalGroup(horizontalSeqGroup);
	    
	    // Vertically
	    layout.setVerticalGroup(verticalSeqGroup); 

	    return p;
	}
	
	/**
	 * Set Table Cell Colours
	 * @param table JTable
	 * @param value Object
	 * @param isSelected boolean
	 * @param hasFocus boolean
	 * @param row int
	 * @param column int
	 * @param c Component
	 * @return Table component containing coloured cells
	 */
	public static Component setCellColours(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column, Component c){       
		if (column==2){				
			c.setBackground(Utility.getColor(Double.parseDouble(Main.idAndFileTypeList.get(row)[3])));
		}
		else{
			c.setBackground(Color.WHITE);
		}
		
		return c;
	}

	/**
	 * Set Image In Scroll Pane when a row in the table is selected
	 * @param tempTable JTable table being selected
	 * @param moveVal int integer amount to move the selected row
	 * @param manualRowSelection boolean set to true to use moveVal to manually move the row selected
	 * @throws Exception
	 */
	public static void setImageInScrollPane(JTable tempTable, int moveVal, boolean manualRowSelection) throws Exception{
	   if (tempTable.getRowCount()>=1){
		
		int row = tempTable.getSelectedRow()+moveVal;
	    int column = tempTable.getSelectedColumn();
	    
	    if (manualRowSelection){
	    	Main.table.changeSelection(row, column, false, false);
	    }
	    
	    if (row < 0){
	    	row = 0;
	    }
	    
	    if (row > Main.filesInDataset.size()-1){
	    	row = Main.filesInDataset.size()-1;	
	    }
	
	    ArrayList<String> data = Main.metadataFields.get(row);   
	    String[] labels = new String[data.size()]; 
	    labels = data.toArray(labels);
	
	    Main.metadataScrollPane.getViewport().remove(Main.metadataPanel);
	    Main.metadataPanel = GUIModifier.populateMetadataPanel(labels);
	    Main.metadataScrollPane.setViewportView(Main.metadataPanel); 
	
	    //Console Feedback ---------------------------------------- 
	    String selectedCellID = Main.metadata.get(row).get(1)+"\n";
		String selectedCellInfo = "["+Main.filesInDataset.get(row).getCanonicalPath()+"]\n";
		try
		{
			//consoleDocumentLength
			Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Info: ", Main.infoAttributes);
			Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Selected ID: ", Main.boldAttributes);
			Main.consoleDocument.insertString(Main.consoleDocument.getLength(), selectedCellID, null);
			Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "         File Path: ", Main.boldAttributes);
			Main.consoleDocument.insertString(Main.consoleDocument.getLength(), selectedCellInfo, null);
		}
		catch(Exception e) { System.out.println(e); }
	    //---------------------------------------------------------- 
	
		try {
			 // Image
			 if (Utility.arrayContains(Utility.getExtension(Main.filesInDataset.get(row).getName()), FileTypes.viewableImageType)){
					if (!Main.filesInDataset.get(row).exists()){
						
						if (!Main.imagePanel.isPanelEmpty()){
							Main.imagePanel.resetPanel();
						}
						
			   			//Console Feedback ---------------------------------------- 
						try
						{
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Error: ", Main.errorAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Image File Does not exist:   ", Main.boldAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), Main.filesInDataset.get(row)+" (PID:"+Main.listOfFileCorrespondingPIDSInDataset.get(row)+") \n", null);
						}
						catch(Exception ex) { System.out.println(ex); }
					    //---------------------------------------------------------- 	
					}
					else{
						Main.imagePanel.setImage(Main.imageLoader.load(Main.filesInDataset.get(row).getCanonicalPath()));
					}
			 }
			 else{
				 if (Main.filesInDataset.get(row).exists()){
					 String tempFilePath = Main.filesInDataset.get(row).getPath();
					 String filePathWithoutExtension = Utility.removeExtension(tempFilePath);
					 String thumbnailPath = filePathWithoutExtension+"_s.jpg";
					 thumbnailPath = tempFilePath.substring(0,tempFilePath.lastIndexOf(File.separator))+"/"+thumbnailPath;
					 System.out.println(thumbnailPath);
					 if ((new File(thumbnailPath)).exists()){
						 Main.imagePanel.setImage(Main.imageLoader.load(thumbnailPath)); 
					 }
					 else{
				   			//Console Feedback ---------------------------------------- 
							try
							{
								Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Error: ", Main.errorAttributes);
								Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Thumbnail Image Does not exist:   ", Main.boldAttributes);
								Main.consoleDocument.insertString(Main.consoleDocument.getLength(), thumbnailPath+" (PID:"+Main.listOfFileCorrespondingPIDSInDataset.get(row)+") \n", null);
							}
							catch(Exception ex) { System.out.println(ex); }
						    //---------------------------------------------------------- 
					 }
				 }
				 else{
					 
						if (!Main.imagePanel.isPanelEmpty()){
							Main.imagePanel.resetPanel();
						}
					 
			   			//Console Feedback ---------------------------------------- 
						try
						{
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Error: ", Main.errorAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "File Does not exist:   ", Main.boldAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), Main.listOfFilePathsInDataset.get(row)+" (PID:"+Main.listOfFileCorrespondingPIDSInDataset.get(row)+") \n", null);
						}
						catch(Exception ex) { System.out.println(ex); }
					    //---------------------------------------------------------- 
				 }
			 }
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		// Ensure Metadata Panel is open
		Main.tabbedPane.setSelectedIndex(1);
	   }
	}

	/**
	 * Zoom into the current image in the image panel
	 * @throws Exception incase there is no image
	 */
	public static void zoomIn() throws Exception{
		Main.imagePanel.setScale(1);
	}
	
	/**
	 * Zoom out of the current image in the image panel
	 * @throws Exception incase there is no image
	 */
	public static void zoomOut() throws Exception{
		Main.imagePanel.setScale(-1);
	}

	/**
	 * Update Metadata Panel Status
	 * @param row int row in table corresponding to metadata panel
	 */
	public static void updateMetadataPanelStatus(int row){
		Component[] components = Main.metadataPanel.getComponents();
		
		int numberFields = 0;
		int fieldsComplete = 0;
		
		ArrayList<String> temp = new ArrayList<String>();
		
		for (Component c : components){
			if (c.getClass() == JTextField.class) { 
				
				JTextField dataItem = (JTextField)c;	
				temp.add(dataItem.getText());
				
				if (dataItem.getText()!=null && !dataItem.getText().equals("")){
					fieldsComplete++;
				}
				
				numberFields++;
			}	
			else if (c.getClass() == JComboBox.class){
				@SuppressWarnings("unchecked")
				JComboBox<String> dataItem = (JComboBox<String>)c;	
				String val = dataItem.getEditor().getItem().toString();
				temp.add(val);
				
				if (val!=null && !val.equals("")){
					fieldsComplete++;
				}
				
				numberFields++;
			}
		}
		
		Main.metadata.set(row, temp);
	
		int percentageComplete = (int)Math.round(((double)fieldsComplete/(double)numberFields)*100.0);
		int lengthOfPadding = 4-(((int)percentageComplete)+"").length();	
		Main.booleanTableModel.data[row][2]="<html>"+Utility.indent(lengthOfPadding)+percentageComplete+"%";
		Main.idAndFileTypeList.get(row)[3] = percentageComplete+"";
		Main.booleanTableModel.updateCell(row, 2);	
	}
	
	/**
	 * Update Status of Metadata Item in the Table
	 * @param index int index of item
	 */
	public static void updateMetadataItemStatus(int index){
		
		ArrayList<String> metadataFields = Main.metadata.get(index);
		
		int numberFields = 0;
		int fieldsComplete = 0;
		
		for (String field : metadataFields){
			if (field!=null && !field.equals("")){
				fieldsComplete++;
			}		
			numberFields++;
		}
		
		int percentageComplete = (int)Math.round(((double)fieldsComplete/(double)numberFields)*100.0);
		int lengthOfPadding = 4-(((int)percentageComplete)+"").length();	
		Main.booleanTableModel.data[index][2]="<html>"+Utility.indent(lengthOfPadding)+percentageComplete+"%";
		Main.idAndFileTypeList.get(index)[3] = percentageComplete+"";
		Main.booleanTableModel.updateCell(index, 2);	
	}

	/**
	 * Get Index's of Selected Table Items
	 * @return int[] containing index's of the selected table items
	 */
	public static int[] getSelectedTableItemIndexes(){
		int[] selectedTableRows = Main.table.getSelectedRows();
		return selectedTableRows;
	}
}