package zamani.metafy.core.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSplitPane;
import javax.swing.JTextField;

import zamani.metafy.core.Main;
import zamani.metafy.core.utility.Utility;

/** 
 * PopupWindow utility class
 * Contains popup window functions
 * 
 * @author Michael Ferguson
 * 
 */
public final class PopupWindow {

    /**
     * Non instantiable Constructor
    */
	private PopupWindow(){}
	
	 // Index of currently selected creator (Batch Modify)
	 static int selectedCreatorIndex = 0;
	 static String[] creatorLabels;
	
	 // Creator Lists (Batch Modify)
	 static ArrayList<String> uniqueCreatorList, newCreatorList;
	 
	// BackgroundWorder instance
	static BackgroundWorker backgroundWorker = new BackgroundWorker();
	 
	/**
	 * Popup Batch Modify Window
	 * @param  screenWidth  int width of screen
	 * @param  screenHeight  int height of screen
	 */
	public static void popupBatchModify(int screenWidth, int screenHeight){
		
		Main.batchModifyWindow = new JFrame("Batch Modify Selected Items");
		
		Main.batchModifyWindow.addWindowListener(new java.awt.event.WindowAdapter() {
		    @Override
		    public void windowClosing(java.awt.event.WindowEvent windowEvent) {
		    	
		        //hide and dispose the frame
		    	Main.batchModifyWindow.setVisible(false);
		    	Main.batchModifyWindow.dispose();
		    }
		});	
		
		Main.batchModifyWindow.setSize(new Dimension((int)(screenWidth*0.4), (int)(screenHeight*0.95)));
		Main.batchModifyWindow.setVisible(true);
		Main.batchModifyWindow.setAlwaysOnTop(true);
		Main.batchModifyWindow.setLocationRelativeTo(null);
	    
		
		JPanel keyAndInputFieldsPanel = new JPanel();
		
		
		JPanel keyPanel = new JPanel();
		keyPanel.setPreferredSize(new Dimension((int)(screenWidth*0.35), 110));
		keyPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(15,10,20,10),BorderFactory.createTitledBorder("Key")));
		
		keyPanel.add(new JLabel("<html><b>CLEAR</b>: Leave field blank &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>IGNORE</b>: Ignore the value in the field</html>"));
		
		
		keyAndInputFieldsPanel.add(keyPanel);
		
		JPanel radioButtonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		
		
		// Radio Button Group
		ButtonGroup radioButtonGroup = new ButtonGroup();
		
		// Radio buttons
		JRadioButton ignore = new JRadioButton("Ignore");
		JRadioButton change = new JRadioButton("Change");
		
		// Set ignore to the selected radio button
		ignore.setSelected(true);

		
		final JCheckBox publishCheckbox = new JCheckBox("Pub");
		// Disable publish checkbox initially
		publishCheckbox.setEnabled(false);
		
		ignore.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent arg0) {
	        	publishCheckbox.setEnabled(false);
	        }
	    });
		
		change.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent arg0) {
	        	publishCheckbox.setEnabled(true);
	        }
	    });

		// Add radio buttons
		radioButtonGroup.add(ignore);
		radioButtonGroup.add(change);
		
		
		radioButtonPanel.add(new JLabel("Publish Status:"));
		radioButtonPanel.add(ignore);
		radioButtonPanel.add(change);
		radioButtonPanel.add(Box.createRigidArea(new Dimension(20,0)));
		radioButtonPanel.add(publishCheckbox);
		
		radioButtonPanel.setBorder(BorderFactory.createEmptyBorder(0,5,0,5));
		
		
		JPanel radioButtonPanelHolder = new JPanel();
		radioButtonPanelHolder.setPreferredSize(new Dimension((int)(screenWidth*0.35), 30));
		radioButtonPanelHolder.setLayout(new BoxLayout(radioButtonPanelHolder, BoxLayout.Y_AXIS));
		
		radioButtonPanelHolder.add(radioButtonPanel);
		radioButtonPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		keyAndInputFieldsPanel.add(radioButtonPanelHolder);
		
		JPanel p1 = new JPanel(); 
		p1.setPreferredSize(new Dimension((int)(screenWidth*0.35), (int)(screenHeight*0.37)));
	    p1.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

	    GroupLayout layout = new GroupLayout(p1);
	    p1.setLayout(layout);
	    layout.setAutoCreateGaps(true);
	     

	    JLabel titleLabel = new JLabel("Title");
	    final JTextField  title = new JTextField(10);
	    title.setPreferredSize( new Dimension( 300, 27 ) );
	    title.setText("IGNORE");
	    
	    JLabel dateLabel = new JLabel("Date");
	    final JTextField  date = new JTextField(10);
	    date.setPreferredSize( new Dimension( 100, 27 ) );
	    date.setText("IGNORE");
	    
	    JLabel sourceLabel = new JLabel("Source");
	    final JTextField source = new JTextField(10);
	    source.setPreferredSize( new Dimension( 100, 27 ) );
	    source.setText("IGNORE");
	    
	    JLabel typeLabel = new JLabel("Type");
	    final JTextField type = new JTextField(10);
	    type.setPreferredSize( new Dimension( 100, 27 ) );
	    type.setText("IGNORE");
	    
	    JLabel coverageSpatialCountryLabel = new JLabel("Coverage Spatial Country");
	    final JTextField coverageSpatialCountry = new JTextField(10);
	    coverageSpatialCountry.setPreferredSize( new Dimension( 100, 27 ) );
	    coverageSpatialCountry.setText("IGNORE");

	    JLabel coverageSpatialNameLabel = new JLabel("Coverage Spatial Name");
	    final JTextField coverageSpatialName = new JTextField(10);
	    coverageSpatialName.setPreferredSize( new Dimension( 100, 27 ) );
	    coverageSpatialName.setText("IGNORE");
	    
	    JLabel coverageSpatialStructureLabel = new JLabel("Coverage Spatial Structure");
	    final JTextField coverageSpatialStructure = new JTextField(10);
	    coverageSpatialStructure.setPreferredSize( new Dimension( 100, 27 ) );
	    coverageSpatialStructure.setText("IGNORE");
	    
	    JLabel coverageSpatialPositionLabel = new JLabel("Coverage Spatial Position");
	    final JTextField coverageSpatialPosition = new JTextField(10);
	    coverageSpatialPosition.setPreferredSize( new Dimension( 100, 27 ) );
	    coverageSpatialPosition.setText("IGNORE");
	    
	    JLabel coverageSpatialPartLabel = new JLabel("Coverage Spatial Part");
	    final JTextField coverageSpatialPart = new JTextField(10);
	    coverageSpatialPart.setPreferredSize( new Dimension( 100, 27 ) );
	    coverageSpatialPart.setText("IGNORE");
	    
	    JLabel coverageSpatialLocalityLabel = new JLabel("Coverage Spatial Locality");
	    final JTextField coverageSpatialLocality = new JTextField(10);
	    coverageSpatialLocality.setPreferredSize( new Dimension( 100, 27 ) );
	    coverageSpatialLocality.setText("IGNORE");
	    
	    final JLabel[] labelArray = {titleLabel, dateLabel, sourceLabel, typeLabel, coverageSpatialCountryLabel, coverageSpatialNameLabel, coverageSpatialStructureLabel, coverageSpatialPositionLabel, coverageSpatialPartLabel, coverageSpatialLocalityLabel};
	    final JTextField[] textFieldArray = {title, date, source, type, coverageSpatialCountry, coverageSpatialName, coverageSpatialStructure, coverageSpatialPosition, coverageSpatialPart, coverageSpatialLocality};
	    
	    // Horizontally
	    layout.setHorizontalGroup( layout.createSequentialGroup()
	    	// Label group	
			.addGroup( layout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent(  titleLabel )
				.addComponent(  dateLabel )
				.addComponent( sourceLabel )
				.addComponent( typeLabel )
				
				.addComponent(  coverageSpatialCountryLabel )
				.addComponent(  coverageSpatialNameLabel )
				.addComponent( coverageSpatialStructureLabel )
				.addComponent( coverageSpatialPositionLabel )
				.addComponent( coverageSpatialPartLabel )
				.addComponent( coverageSpatialLocalityLabel ))
				
			// Textfield group                    
			.addGroup( layout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent(  title )
				.addComponent(  date )
				.addComponent( source )
				.addComponent( type )
				
				.addComponent(  coverageSpatialCountry )
				.addComponent(  coverageSpatialName )
				.addComponent( coverageSpatialStructure )
				.addComponent( coverageSpatialPosition )
				.addComponent( coverageSpatialPart )
				.addComponent( coverageSpatialLocality ))
	    );

	    // Vertically
	    layout.setVerticalGroup( layout.createSequentialGroup()
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent(  titleLabel )
	         .addComponent(  title ))
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )     
	         .addComponent(  dateLabel )
	         .addComponent(  date ))	
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )     
	         .addComponent( sourceLabel )
	         .addComponent( source ))    
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent( typeLabel )
	         .addComponent( type ))   
	          
		 .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent(  coverageSpatialCountryLabel )
	         .addComponent(  coverageSpatialCountry ))
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )     
	         .addComponent(  coverageSpatialNameLabel )
	         .addComponent(  coverageSpatialName ))	
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )     
	         .addComponent( coverageSpatialStructureLabel )
	         .addComponent( coverageSpatialStructure ))    
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent( coverageSpatialPositionLabel )
	         .addComponent( coverageSpatialPosition ))   
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )     
	         .addComponent( coverageSpatialPartLabel )
	         .addComponent( coverageSpatialPart ))    
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent( coverageSpatialLocalityLabel )
	         .addComponent( coverageSpatialLocality ))       
	    ); 
	    
	    JPanel panel2 = new JPanel();
	    panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));
	    
	    
	    // Get list of unique creators from all the selected items
	    int[] selectedTableItems = GUIModifier.getSelectedTableItemIndexes();
	    uniqueCreatorList = new ArrayList<String>();
	    
	    if (selectedTableItems.length!=0){
	    	for(int tableIndex : selectedTableItems){

	    		for (int index=0; index< Main.metadataFields.get(tableIndex).size();index++){
	    			if (Main.metadataFields.get(tableIndex).get(index).equals("Creator")){
	        	    	if (!uniqueCreatorList.contains(Main.metadata.get(tableIndex).get(index))){
		    	    		uniqueCreatorList.add(Main.metadata.get(tableIndex).get(index));
		    	    	}
	    			}
	    		}
	    	}

	    }
	    

		JPanel creatorsPanel = new JPanel(); 
		creatorsPanel.setPreferredSize(new Dimension((int)(screenWidth*0.35), (int)(screenHeight*0.20)));

		creatorsPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

	    GroupLayout creatorsPanelLayout = new GroupLayout(creatorsPanel);
	    creatorsPanel.setLayout(creatorsPanelLayout);
	    creatorsPanelLayout.setAutoCreateGaps(true);
	    
	    
	    creatorLabels = uniqueCreatorList.toArray(new String[uniqueCreatorList.size()]);
        JLabel creatorsLabel = new JLabel("Creators");
        final JComboBox<String> creatorComboBox = new JComboBox<String>(creatorLabels);
         creatorComboBox.setMaximumRowCount(5);

         creatorComboBox.setEditable(true);
       
         creatorComboBox.setPreferredSize( new Dimension( 200, 26 ) );
        
         
         
         creatorComboBox.addActionListener (new ActionListener () {
       	    public void actionPerformed(ActionEvent e) {
       	    	if (creatorComboBox.getSelectedIndex()!=-1){
       	    		selectedCreatorIndex = creatorComboBox.getSelectedIndex();
       	    	}
       	    }
         });
        
        //--------------------------------------------------------------------------------                      

        
        JButton b2 = new JButton("UPDATE");
        b2.addActionListener (new ActionListener () {
      	    public void actionPerformed(ActionEvent e) {

      	    	Object selectedItem = creatorComboBox.getSelectedItem();
      	    	if (selectedItem!=null){
      	    		creatorLabels[selectedCreatorIndex] = selectedItem.toString();
      	    		
      	    		DefaultComboBoxModel<String> model = new DefaultComboBoxModel<String>(creatorLabels);
       	    		creatorComboBox.setModel(model);
      	    	}
      	    }
      	  });
        
        
        JButton b3 = new JButton("DELETE");
        b3.addActionListener (new ActionListener () {
    	    public void actionPerformed(ActionEvent e) {

    	    	ArrayList<String> aList  = new ArrayList<String>(Arrays.asList(creatorLabels));  
    	    	aList.remove(selectedCreatorIndex); 
    	    	String[] temp = new String[creatorLabels.length-1];
    	    	aList.toArray(temp); 
    	    	creatorLabels=temp;
  	    		DefaultComboBoxModel<String> model = new DefaultComboBoxModel<String>(creatorLabels);
   	    		creatorComboBox.setModel(model);
    	    }
    	});

        final JTextField addItemTextField = new JTextField(10);
        JButton addButton = new JButton("ADD New Creator");
        addButton.addActionListener (new ActionListener () {
    	  public void actionPerformed(ActionEvent e) {
    		    // Check to see that text has been entered and that an item doesn't already exist with that name
    		  	if (!addItemTextField.getText().equals("") && !Utility.arrayContainsNull(addItemTextField.getText(), creatorLabels)){
    		  		
    		  		ArrayList<String> aList  = new ArrayList<String>(Arrays.asList(creatorLabels));
        	    	aList.add(addItemTextField.getText());  
        	    	
        	    	String[] temp = new String[creatorLabels.length+1];
        	    	aList.toArray(temp); 
        	    	creatorLabels=temp;
        	    	
      	    		DefaultComboBoxModel<String> model = new DefaultComboBoxModel<String>(creatorLabels);
       	    		creatorComboBox.setModel(model);
    		  	}
    	    }
        });
	    
	    
        // Add components to Automation Dictionary Panel
        //----------------------------------------------
        // Horizontally
        creatorsPanelLayout.setHorizontalGroup( creatorsPanelLayout.createSequentialGroup()
			.addGroup( creatorsPanelLayout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( creatorsLabel ))
			                    
			.addGroup( creatorsPanelLayout.createParallelGroup( GroupLayout.Alignment.LEADING )	
				.addComponent( creatorComboBox )
				.addComponent( addItemTextField ) )
			                  
			.addGroup( creatorsPanelLayout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( b2 )
				.addComponent( addButton ))
			                  
			.addGroup( creatorsPanelLayout.createParallelGroup( GroupLayout.Alignment.LEADING )              
				.addComponent( b3 ))
        );
	    
	    
        // Vertically
        creatorsPanelLayout.setVerticalGroup( creatorsPanelLayout.createSequentialGroup()
         .addGroup( creatorsPanelLayout.createParallelGroup( GroupLayout.Alignment.BASELINE )
            .addComponent( creatorsLabel )
            .addComponent( creatorComboBox )
            .addComponent( b2 )
            .addComponent( b3 ))
         .addGroup( creatorsPanelLayout.createParallelGroup( GroupLayout.Alignment.BASELINE )
            .addComponent( addItemTextField )
            .addComponent( addButton ))                                                                        
        ); 


	    final JButton batchModify = new JButton("Batch Modify");
	    batchModify.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				ArrayList<Integer> batchModifyFieldIndexes = new ArrayList<Integer>();
				
				boolean fieldsModified = false;
				boolean publishStatusValue;
				
				// Get fields to modify
				for (int i=0; i<textFieldArray.length; i++){
					if (!textFieldArray[i].getText().equals("IGNORE")){
						batchModifyFieldIndexes.add(i);
						fieldsModified = true;
					}
				}
				
				// Copy ComboBox values to arraylist for comparison
				newCreatorList = new ArrayList<String>(Arrays.asList(creatorLabels));
				
				int[] selectedTableItems = GUIModifier.getSelectedTableItemIndexes();
				
				// No table items selected
				if (selectedTableItems.length==0){
				    //Console Feedback ----------------------------------------
					try
					{
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Error: ", Main.errorAttributes);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Batch Modify:   \n", Main.boldAttributes);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "No table items selected \n", null);
						Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Select table item/s to Batch Modify \n", null);
					}
					catch(Exception ex) { System.out.println(ex); }
				    //----------------------------------------------------------
				}
				// Fields to modify?, Creators to modify? or publish Status to modify?
				else if (batchModifyFieldIndexes.size()>0 || !newCreatorList.equals(uniqueCreatorList) || publishCheckbox.isEnabled()){
					// Get selected Table items
						for (int tableItemIndex : selectedTableItems){
							// Loop through all field indexes to modify
							for (int fieldToModifyIndex : batchModifyFieldIndexes){
								// Modify metadata field value
								if (Main.metadataFields.get(tableItemIndex).indexOf(labelArray[fieldToModifyIndex].getText())!=-1){
									Main.metadata.get(tableItemIndex).set(Main.metadataFields.get(tableItemIndex).indexOf(labelArray[fieldToModifyIndex].getText()), textFieldArray[fieldToModifyIndex].getText());
								}
							}
							
							// Creators modified?
							if (!newCreatorList.equals(uniqueCreatorList)){
								
								// Only Modify Metadata's creator fields if there is at least one creator
								if (newCreatorList.size()>0){
									fieldsModified = true;
						    		// Create modified metadata and metadataFields lists
						    		ArrayList<String> tempMetadataFieldsList = new ArrayList<String>();
						    		ArrayList<String> tempMetadataList = new ArrayList<String>();
						    		
						    		ArrayList<String> metadataFieldsBeforeCreator = new ArrayList<String>();
						    		ArrayList<String> metadataCreatorFields = new ArrayList<String>();
						    		ArrayList<String> metadataFieldsAfterCreator = new ArrayList<String>();
						    		
						    		ArrayList<String> metadataBeforeCreator = new ArrayList<String>();
						    		ArrayList<String> metadataCreator = new ArrayList<String>();
						    		ArrayList<String> metadataAfterCreator = new ArrayList<String>();
									
									boolean isBeforeCreator = true;
									boolean isCreator = false;
									boolean isAfterCreator = false;
									
									// Populate before and after creator lists
						    		for (int index=0; index< Main.metadataFields.get(tableItemIndex).size();index++){
						    			
						    			if (Main.metadataFields.get(tableItemIndex).get(index).equals("Creator")){
						    				isCreator = true;
						    			}
						    			else{
						    				if (isCreator){
						    					isCreator = false;
						    					isBeforeCreator = false;
						    					isAfterCreator = true;
						    				}
						    			}
						    			// add to fields before creator
						    			if (isBeforeCreator && !isCreator && !isAfterCreator){
						    				metadataFieldsBeforeCreator.add(Main.metadataFields.get(tableItemIndex).get(index));
						    				metadataBeforeCreator.add(Main.metadata.get(tableItemIndex).get(index));
						    			}
						    			// add to fields after creator
						    			else if (isAfterCreator && !isCreator && !isBeforeCreator){
						    				metadataFieldsAfterCreator.add(Main.metadataFields.get(tableItemIndex).get(index));
						    				metadataAfterCreator.add(Main.metadata.get(tableItemIndex).get(index));
						    			}
						    		}
						    		
						    		// Populate creator lists
						    		for (String creator : creatorLabels){
						    			metadataCreatorFields.add("Creator");
						    			metadataCreator.add(creator);
						    		}
						    		
						    		// Modify metada and metadataFields
						    		tempMetadataFieldsList.addAll(metadataFieldsBeforeCreator);
						    		tempMetadataFieldsList.addAll(metadataCreatorFields);
						    		tempMetadataFieldsList.addAll(metadataFieldsAfterCreator);
						    		
						    		tempMetadataList.addAll(metadataBeforeCreator);
						    		tempMetadataList.addAll(metadataCreator);
						    		tempMetadataList.addAll(metadataAfterCreator);
						    		
						    		Main.metadata.set(tableItemIndex,tempMetadataList);
						    		Main.metadataFields.set(tableItemIndex,tempMetadataFieldsList);
								}
							}
							
							// Modify Publish status
							// ----------------------
							
							// Check if publish Checkbox is enabled
							if (publishCheckbox.isEnabled()){
								fieldsModified = true;
								
								// if enabled, change all of the selected records to its value (true if checked, false if unchecked)
								publishStatusValue = publishCheckbox.isSelected();
								
								// Modify metadata items publish status
								Main.data[tableItemIndex][0] = publishStatusValue;
								
								// Update the metadata items table cell
								Main.booleanTableModel.updateCell(tableItemIndex, 0);
							}
							
							
							// If fields were modified (Update percentage complete and provide console feedback)
							if (fieldsModified){
								// Update table items percentage complete
								GUIModifier.updateMetadataItemStatus(tableItemIndex);
								
							    //Console Feedback ----------------------------------------
								try
								{
									Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Info: ", Main.infoAttributes);
									Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Batch Modify:   \n", Main.boldAttributes);
									Main.consoleDocument.insertString(Main.consoleDocument.getLength(), Main.listOfFileCorrespondingPIDSInDataset.get(tableItemIndex)+" Successfully Modified \n", null);
								}
								catch(Exception ex) { System.out.println(ex); }
							    //----------------------------------------------------------
							}
						}
				}
				
				// Close the window
	            WindowEvent wev = new WindowEvent(Main.batchModifyWindow, WindowEvent.WINDOW_CLOSING);
	            Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(wev);
			}
	    });

	    batchModify.setPreferredSize(new Dimension((int)(screenWidth*0.15), (int)(screenHeight*0.07)));
	    batchModify.setMaximumSize(new Dimension((int)(screenWidth*0.15), (int)(screenHeight*0.07)));
	    batchModify.setMinimumSize(new Dimension((int)(screenWidth*0.15), (int)(screenHeight*0.07)));
	    panel2.add(Box.createVerticalGlue());
	    JPanel batchModifyPanel = new JPanel();
	    batchModifyPanel.add(batchModify);
	    panel2.add(batchModifyPanel);
	    panel2.add(Box.createVerticalGlue());
	    
	    keyAndInputFieldsPanel.add(p1);
	    keyAndInputFieldsPanel.add(creatorsPanel);
	    
	    JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, keyAndInputFieldsPanel, panel2);
	    splitPane.setOneTouchExpandable(false);
	    splitPane.setDividerLocation( (int)(screenHeight*0.77));
	    splitPane.setEnabled(false);
	    splitPane.setDividerSize(0);
		
	    Main.batchModifyWindow.add(splitPane);	
	}
	
	/**
	 * Popup Load Metadata Progress Window
	 * @param  screenWidth  int width of screen
	 * @param  screenHeight  int height of screen
	 */
	public static void popupLoadMetadataProgressWindow(int screenWidth, int screenHeight, String pathToMetafyFile){
		Main.popupLoadMetadataProgressWindow = new JFrame("Load Metadata");
		
		Main.popupLoadMetadataProgressWindow.addWindowListener(new java.awt.event.WindowAdapter() {
		    @Override
		    public void windowClosing(java.awt.event.WindowEvent windowEvent) {
		    	
		        //hide and dispose the frame
		    	Main.popupLoadMetadataProgressWindow.setVisible(false);
		    	Main.popupLoadMetadataProgressWindow.dispose();
		    }
		});	
		
		Main.popupLoadMetadataProgressWindow.setSize(new Dimension((int)(screenWidth*0.5), (int)(screenHeight*0.15)));
		Main.popupLoadMetadataProgressWindow.setVisible(true);
		Main.popupLoadMetadataProgressWindow.setAlwaysOnTop(true);
		Main.popupLoadMetadataProgressWindow.setLocation(screenWidth/2-(int)((screenWidth*0.5)/2), (int)(screenHeight/2 - (screenHeight*0.15)/4));
		
		// Set Progress bar settings
		Main.loadMetadataProgressBar.setStringPainted(true);
		Main.loadMetadataProgressBar.setBackground(Color.white);
		Main.loadMetadataProgressBar.setForeground(Color.blue);
		Main.loadMetadataProgressBar.setPreferredSize(new Dimension(100, 15));
		Main.loadMetadataProgressBar.setVisible(true);
		Main.loadMetadataProgressBar.setValue(0);

		Main.popupLoadMetadataProgressWindow.add(Main.loadMetadataProgressBar);
		
		// Create a SwingWorker of the LoadMetadataWorker and execute it
		BackgroundWorker.LoadMetadataWorker aw = backgroundWorker.new LoadMetadataWorker(Main.loadMetadataProgressBar, pathToMetafyFile);	
		aw.execute();	
	}

	/**
	 * Popup Export Metadata Window
	 * @param  screenWidth  int width of screen
	 * @param  screenHeight  int height of screen
	 */
	public static void popupExportMetadataWindow(int screenWidth, int screenHeight){
		Main.exportMetadataWindow = new JFrame("Export Metadata");
		
		Main.exportMetadataWindow.addWindowListener(new java.awt.event.WindowAdapter() {
		    @Override
		    public void windowClosing(java.awt.event.WindowEvent windowEvent) {
		    	
		        //hide and dispose the frame
		    	Main.exportMetadataWindow.setVisible(false);
		    	Main.exportMetadataWindow.dispose();
		    }
		});	
		
		Main.exportMetadataWindow.setSize(new Dimension((int)(screenWidth*0.4), (int)(screenHeight*0.53)));
		Main.exportMetadataWindow.setVisible(true);
		Main.exportMetadataWindow.setAlwaysOnTop(true);
		Main.exportMetadataWindow.setLocationRelativeTo(null);
	    
		JPanel p1 = new JPanel(); 
	    p1.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

	    GroupLayout layout = new GroupLayout(p1);
	    p1.setLayout(layout);
	    layout.setAutoCreateGaps(true);
	     

	    JLabel siteLatitudeLabel = new JLabel("Site Latitude");
	    final JTextField siteLatitude = new JTextField(10);
	    siteLatitude.setPreferredSize( new Dimension( 50, 27 ) );
	    siteLatitude.setText("16.488064");
	    
	    JLabel siteLongitudeLabel = new JLabel("Longitude");
	    final JTextField siteLongitude = new JTextField(10);
	    siteLongitude.setPreferredSize( new Dimension( 50, 27 ) );
	    siteLongitude.setText("33.415227");
	    
	    JLabel sharedDrivePathServerLabel = new JLabel("Path to Shared Drive (Server)");
	    final JTextField sharedDrivePathServer = new JTextField(10);
	    sharedDrivePathServer.setPreferredSize( new Dimension( 100, 27 ) );
	    // Set value to start as that of the automation field
	    sharedDrivePathServer.setText(Main.sharedDrivePathOnServer.getText());
	    
	    JLabel sharedDrivePathClientLabel = new JLabel("Path to Shared Drive (Client)");
	    final JTextField sharedDrivePathClient = new JTextField(10);
	    sharedDrivePathClient.setPreferredSize( new Dimension( 100, 27 ) );
	    JButton sharedDrivePathBrowseButton = new JButton("Browse");
	    sharedDrivePathBrowseButton.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {
		  		sharedDrivePathClient.setText(Utility.getDirectoryPath("Path to Shared Drive (Client)", Main.exportMetadataWindow));
			}
	    });
	    
	    JLabel outputMetadataPathLabel = new JLabel("Path to Output Directory");
	    final JTextField outputMetadataPath = new JTextField(10);
	    outputMetadataPath.setPreferredSize( new Dimension( 100, 27 ) );
	    JButton outputMetadataPathBrowseButton = new JButton("Browse");
	    outputMetadataPathBrowseButton.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				outputMetadataPath.setText(Utility.getDirectoryPath("Path to Output Directory", Main.exportMetadataWindow));
			}
	    });
	    
	    // Horizontally
	    layout.setHorizontalGroup( layout.createSequentialGroup()
	    	// Label group	
			.addGroup( layout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( siteLatitudeLabel )
				.addComponent( siteLongitudeLabel )
				.addComponent( sharedDrivePathServerLabel )
				.addComponent( sharedDrivePathClientLabel )
				.addComponent( outputMetadataPathLabel ))
				
			// Textfield group                    
			.addGroup( layout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( siteLatitude )
				.addComponent( siteLongitude )
				.addComponent( sharedDrivePathServer )
				.addComponent( sharedDrivePathClient )
				.addComponent( outputMetadataPath ))
				
			// Button group                  
			.addGroup( layout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( sharedDrivePathBrowseButton )
				.addComponent( outputMetadataPathBrowseButton ))
	    );

	    // Vertically
	    layout.setVerticalGroup( layout.createSequentialGroup()
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent( siteLatitudeLabel )
	         .addComponent( siteLatitude ))
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )     
	         .addComponent( siteLongitudeLabel )
	         .addComponent( siteLongitude ))	
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )     
	         .addComponent( sharedDrivePathServerLabel )
	         .addComponent( sharedDrivePathServer ))    
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent( sharedDrivePathClientLabel )
	         .addComponent( sharedDrivePathClient )
	         .addComponent( sharedDrivePathBrowseButton ))       
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent( outputMetadataPathLabel )
	         .addComponent( outputMetadataPath )
	         .addComponent( outputMetadataPathBrowseButton ))   
	    ); 
	    
	    JPanel panel2 = new JPanel();
	    panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));

	    final JButton exportMetadata = new JButton("Export Metadata");
	    exportMetadata.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {

				String validatorInfo = "";
				int missingFieldCount = 0;
				
				// Validation
				if (siteLatitude.getText().equals("")){
					missingFieldCount++;
				}
				else if (siteLongitude.getText().equals("")){
					missingFieldCount++;
				}
				else if (sharedDrivePathServer.getText().equals("")){
					missingFieldCount++;
				}
				else if (sharedDrivePathClient.getText().equals("")){
					missingFieldCount++;
				}
				else if (outputMetadataPath.getText().equals("")){
					missingFieldCount++;
				}
				
				if (missingFieldCount>0){
					validatorInfo = "Please complete all the fields.";
				}
				else{
					
					String sharedDrivePathClientText = sharedDrivePathClient.getText();
					String outputMetadataPathText = outputMetadataPath.getText();

					boolean validated = false;
					String[] pathList = new String[]{sharedDrivePathClientText, outputMetadataPathText};
					String validationString = Utility.getValidationString(pathList);
					
					if (validationString.equals("")){
						validated = true;
					}
					else{
						validatorInfo = "Path Error: Path Does not exist.";
					}
					
					if (validated){
						exportMetadata.setEnabled(false);
						Main.exportMetadataProgressBar.setPreferredSize(new Dimension(150,35));
						Main.JProgressBarPanelExportMetadata.add(Main.exportMetadataProgressBar);
						
						validatorInfo = "";
						
						String siteLatitudeText = siteLatitude.getText();
						String siteLongitudeText = siteLongitude.getText();
						String sharedDrivePathServerText = sharedDrivePathServer.getText();

						// Create a SwingWorker of the LoadMetadataWorker and execute it
						BackgroundWorker.ExportMetadataWorker exportMetadataWorker = backgroundWorker.new ExportMetadataWorker(Main.exportMetadataProgressBar, siteLatitudeText, siteLongitudeText, sharedDrivePathServerText, sharedDrivePathClientText, outputMetadataPathText);	
						exportMetadataWorker.execute();
					}
					else{
					    //Console Feedback ----------------------------------------
						try
						{
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Error: ", Main.errorAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Export Metadata:   \n", Main.boldAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), validationString, null);
						}
						catch(Exception ex) { System.out.println(ex); }
					    //----------------------------------------------------------
					}
				}
				
				Main.exportMetadataValidationLabel.setText("<html><div style=\"text-align: center;\">" + validatorInfo + "</html>");
			}
	    });
	    
	    Main.exportMetadataValidationLabel = new JLabel("");
	    JPanel labelPanel = new JPanel();
	    labelPanel.add(Main.exportMetadataValidationLabel);
	    panel2.add(labelPanel);
	    exportMetadata.setPreferredSize(new Dimension((int)(screenWidth*0.15), (int)(screenHeight*0.07)));
	    exportMetadata.setMaximumSize(new Dimension((int)(screenWidth*0.15), (int)(screenHeight*0.07)));
	    exportMetadata.setMinimumSize(new Dimension((int)(screenWidth*0.15), (int)(screenHeight*0.07)));
	    Main.JProgressBarPanelExportMetadata = new JPanel();
	    Main.JProgressBarPanelExportMetadata.setBackground(new Color(0xE2E2E2));
	    
		
	    panel2.add(Main.JProgressBarPanelExportMetadata);
	    panel2.add(Box.createVerticalGlue());
	    JPanel exportMetadataPanel = new JPanel();
	    exportMetadataPanel.add(exportMetadata);
	    panel2.add(exportMetadataPanel);
	    panel2.add(Box.createVerticalGlue());
	    
	    JSplitPane editSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, p1, panel2);
	    editSplitPane.setOneTouchExpandable(false);
	    editSplitPane.setDividerLocation( (int)(screenHeight*0.3));
	    editSplitPane.setEnabled(false);
	    editSplitPane.setDividerSize(0);
		
	    Main.exportMetadataWindow.add(editSplitPane);
	}

	/**
	 * Popup Associate Files Window
	 * @param  screenWidth  int width of screen
	 * @param  screenHeight  int height of screen
	 */
	public static void popupAssociateFilesWindow(int screenWidth, int screenHeight){
		Main.associateFilesWindow = new JFrame("Associate Files");
		
		Main.associateFilesWindow.addWindowListener(new java.awt.event.WindowAdapter() {
		    @Override
		    public void windowClosing(java.awt.event.WindowEvent windowEvent) {
		    	
		        //hide and dispose the frame
		    	Main.associateFilesWindow.setVisible(false);
		    	Main.associateFilesWindow.dispose();
		    }
		});	

		Main.associateFilesWindow.setSize(new Dimension((int)(screenWidth*0.4), (int)(screenHeight*0.4)));
		Main.associateFilesWindow.setVisible(true);
		Main.associateFilesWindow.setAlwaysOnTop(true);
		Main.associateFilesWindow.setLocationRelativeTo(null);
		
		JPanel p1 = new JPanel(); 
	    p1.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

	    GroupLayout layout = new GroupLayout(p1);
	    p1.setLayout(layout);
	    layout.setAutoCreateGaps(true);
		
	    JLabel sharedDrivePathServerLabel = new JLabel("Path to Shared Drive (Server)");
	    final JTextField sharedDrivePathServer = new JTextField(10);
	    sharedDrivePathServer.setPreferredSize( new Dimension( 100, 27 ) );
	    // Set value to start as that of the automation field
	    sharedDrivePathServer.setText(Main.sharedDrivePathOnServer.getText());
	    
	    JLabel associateFilesLabel = new JLabel("Select Associate Files Directory");
	    final JTextField associateFile = new JTextField(10);
	    associateFile.setPreferredSize( new Dimension( 100, 27 ) );
	    JButton associateFileBrowseButton = new JButton("Browse");
	    associateFileBrowseButton.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				associateFile.setText(Utility.getDirectoryPath("Select Associate Files Directory", Main.associateFilesWindow));
			}
	    });
		
		
	 // Horizontally
	    layout.setHorizontalGroup( layout.createSequentialGroup()
	    	// Label group	
			.addGroup( layout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( sharedDrivePathServerLabel )
				.addComponent( associateFilesLabel ))
				
			// Textfield group                    
			.addGroup( layout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( sharedDrivePathServer )
				.addComponent( associateFile ))
				
			// Button group                  
			.addGroup( layout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( associateFileBrowseButton ))
	    );

	    // Vertically
	    layout.setVerticalGroup( layout.createSequentialGroup()
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )     
	         .addComponent( sharedDrivePathServerLabel )
	         .addComponent( sharedDrivePathServer ))    
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent( associateFilesLabel )
	         .addComponent( associateFile )
	         .addComponent( associateFileBrowseButton ))       
	    ); 
	    
	    JPanel panel2 = new JPanel();
	    panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));

	    final JButton associate = new JButton("Associate");
	    associate.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {

				String validatorInfo = "";
				int missingFieldCount = 0;
				
				// Validation
				if (sharedDrivePathServer.getText().equals("")){
					missingFieldCount++;
				}
				else if (associateFile.getText().equals("")){
					missingFieldCount++;
				}

				
				if (missingFieldCount>0){
					validatorInfo = "Please complete all the fields.";
				}
				else{
					
					String associateFileDir = associateFile.getText();
					
					boolean validated = false;
					String[] pathList = new String[]{associateFileDir};
					String validationString = Utility.getValidationString(pathList);
					
					if (validationString.equals("")){
						validated = true;
					}
					else{
						validatorInfo = "Path Error: Path Does not exist.";
					}
					
					if (validated){
						associate.setEnabled(false);
						Main.associateFilesProgressBar.setPreferredSize(new Dimension(150,35));
						Main.JProgressBarPanelAssociateFiles.add(Main.associateFilesProgressBar);
						
						validatorInfo = "";
						
						String sharedDrivePathServerText = sharedDrivePathServer.getText();
						
						// Create a SwingWorker of the AssociateFilesWorker and execute it
						BackgroundWorker.AssociateFilesWorker aw = backgroundWorker.new AssociateFilesWorker(Main.associateFilesProgressBar, associateFileDir, sharedDrivePathServerText);	
						aw.execute();	
					}
					else{
					    //Console Feedback ---------------------------------------- 
						try
						{
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Error: ", Main.errorAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Associate Files:   \n", Main.boldAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), validationString, null);
						}
						catch(Exception ex) { System.out.println(ex); }
					    //---------------------------------------------------------- 
					}
				}
				
				Main.associateFilesValidationLabel.setText("<html><div style=\"text-align: center;\">" + validatorInfo + "</html>");
			}
	    });
	    
	    Main.associateFilesValidationLabel = new JLabel("");
	    
	    JPanel labelPanel = new JPanel();
	    labelPanel.add(Main.associateFilesValidationLabel);
	    panel2.add(labelPanel);
	    associate.setPreferredSize(new Dimension((int)(screenWidth*0.15), (int)(screenHeight*0.07)));
	    associate.setMaximumSize(new Dimension((int)(screenWidth*0.15), (int)(screenHeight*0.07)));
	    associate.setMinimumSize(new Dimension((int)(screenWidth*0.15), (int)(screenHeight*0.07)));
	    Main.JProgressBarPanelAssociateFiles = new JPanel();
		Main.JProgressBarPanelAssociateFiles.setBackground(new Color(0xE2E2E2));
		
		
	    panel2.add(Main.JProgressBarPanelAssociateFiles);
	    panel2.add(Box.createVerticalGlue());
	    JPanel associatePanel = new JPanel();
	    associatePanel.add(associate);
	    panel2.add(associatePanel);
	    panel2.add(Box.createVerticalGlue());
	    
	    JSplitPane editSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, p1, panel2);
	    editSplitPane.setOneTouchExpandable(false);
	    editSplitPane.setDividerLocation( (int)(screenHeight*0.17));
	    editSplitPane.setEnabled(false);
	    editSplitPane.setDividerSize(0);
		
	    Main.associateFilesWindow.add(editSplitPane);
	}

	/**
	 * Popup Transform Metadata Window
	 * @param  screenWidth  int width of screen
	 * @param  screenHeight  int height of screen
	 */
	public static void popupTransformAlukaMetadataWindow(final int screenWidth, final int screenHeight){
		Main.transformAlukaMetadataWindow = new JFrame("Transform Aluka Metadata...");
		
		Main.transformAlukaMetadataWindow.addWindowListener(new java.awt.event.WindowAdapter() {
		    @Override
		    public void windowClosing(java.awt.event.WindowEvent windowEvent) {
		    	
		        //hide and dispose the frame
		    	Main.transformAlukaMetadataWindow.setVisible(false);
		    	Main.transformAlukaMetadataWindow.dispose();
		    }
		});	
		
		Main.transformAlukaMetadataWindow.setVisible(true);
		Main.transformAlukaMetadataWindow.setAlwaysOnTop(true);
		

		JPanel p1 = new JPanel(); 
	    p1.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

	    GroupLayout layout = new GroupLayout(p1);
	    p1.setLayout(layout);
	    layout.setAutoCreateGaps(true);
	     
	    JLabel coverageSpatialCountryLabel = new JLabel("Coverage Spatial Country");
	    final JTextField coverageSpatialCountry = new JTextField(10);
	    coverageSpatialCountry.setPreferredSize( new Dimension( 100, 27 ) );
	    coverageSpatialCountry.setText("Sudan");
	    
	    JLabel coverageSpatialSitenameLabel = new JLabel("Coverage Spatial Name");
	    final JTextField coverageSpatialSitename = new JTextField(10);
	    coverageSpatialSitename.setPreferredSize( new Dimension( 100, 27 ) );
	    coverageSpatialSitename.setText("Musawwarat es-Sufra");
	    
	    JLabel siteLatitudeLabel = new JLabel("Site Latitude");
	    final JTextField siteLatitude = new JTextField(10);
	    siteLatitude.setPreferredSize( new Dimension( 50, 27 ) );
	    siteLatitude.setText("16.488064");
	    
	    JLabel siteLongitudeLabel = new JLabel("Longitude");
	    final JTextField siteLongitude = new JTextField(10);
	    siteLongitude.setPreferredSize( new Dimension( 50, 27 ) );
	    siteLongitude.setText("33.415227");
	    
	    JLabel sharedDrivePathServerLabel = new JLabel("Path to Shared Drive (Server)");
	    final JTextField sharedDrivePathServer = new JTextField(10);
	    sharedDrivePathServer.setPreferredSize( new Dimension( 100, 27 ) );
	    // Set value to start as that of the automation field
	    sharedDrivePathServer.setText(Main.sharedDrivePathOnServer.getText());
	    
	    JLabel sharedDrivePathClientLabel = new JLabel("Path to Shared Drive (Client)");
	    final JTextField sharedDrivePathClient = new JTextField(10);
	    sharedDrivePathClient.setPreferredSize( new Dimension( 100, 27 ) );
	    JButton sharedDrivePathBrowseButton = new JButton("Browse");
	    sharedDrivePathBrowseButton.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {
		  		sharedDrivePathClient.setText(Utility.getDirectoryPath("Path to Shared Drive (Client)", Main.transformAlukaMetadataWindow));
			}
	    });
	    
	    JLabel alukaMetadataFilePathLabel = new JLabel("Path to Aluka Metadata File");
	    final JTextField alukaMetadataFilePath = new JTextField(10);
	    alukaMetadataFilePath.setPreferredSize( new Dimension( 100, 27 ) );
	    JButton alukaMetadataFilePathBrowseButton = new JButton("Browse");
	    alukaMetadataFilePathBrowseButton.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				alukaMetadataFilePath.setText(Utility.getFilePath("Path to Aluka Metadata File", Main.transformAlukaMetadataWindow));
			}
	    });
	    
	    JLabel alukaErrorFilePathLabel = new JLabel("Path to Aluka Error File");
	    final JTextField alukaErrorFilePath = new JTextField(10);
	    alukaErrorFilePath.setPreferredSize( new Dimension( 100, 27 ) );
	    JButton alukaErrorFilePathBrowseButton = new JButton("Browse");
	    alukaErrorFilePathBrowseButton.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				alukaErrorFilePath.setText(Utility.getFilePath("Path to Aluka Error File", Main.transformAlukaMetadataWindow));
			}
	    });

	    JLabel alukaDataDirectoryFilePathLabel = new JLabel("Path to Aluka Data Directory");
	    final JTextField alukaDataDirectoryFilePath = new JTextField(10);
	    alukaDataDirectoryFilePath.setPreferredSize( new Dimension( 100, 27 ) );
	    JButton alukaDataDirectoryFilePathBrowseButton = new JButton("Browse");
	    alukaDataDirectoryFilePathBrowseButton.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				alukaDataDirectoryFilePath.setText(Utility.getDirectoryPath("Path to Aluka Data Directory", Main.transformAlukaMetadataWindow));
			}
	    });
	    
	    JLabel transformedMetadataPathLabel = new JLabel("Path to Transformed Metadata");
	    final JTextField transformedMetadataPath = new JTextField(10);
	    transformedMetadataPath.setPreferredSize( new Dimension( 100, 27 ) );
	    JButton transformedMetadataPathBrowseButton = new JButton("Browse");
	    transformedMetadataPathBrowseButton.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				transformedMetadataPath.setText(Utility.getDirectoryPath("Path to Transformed Metadata", Main.transformAlukaMetadataWindow));
			}
	    });
	    
	    // Horizontally
	    layout.setHorizontalGroup( layout.createSequentialGroup()
	    	// Label group	
			.addGroup( layout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( coverageSpatialCountryLabel )
				.addComponent( coverageSpatialSitenameLabel )
				.addComponent( siteLatitudeLabel )
				.addComponent( siteLongitudeLabel )
				.addComponent( sharedDrivePathServerLabel )
				.addComponent( sharedDrivePathClientLabel )
				.addComponent( alukaMetadataFilePathLabel )
				.addComponent( alukaErrorFilePathLabel )	
				.addComponent( alukaDataDirectoryFilePathLabel )	
				.addComponent( transformedMetadataPathLabel ))
				
			// Textfield group                    
			.addGroup( layout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( coverageSpatialCountry )
				.addComponent( coverageSpatialSitename )
				.addComponent( siteLatitude )
				.addComponent( siteLongitude )
				.addComponent( sharedDrivePathServer )
				.addComponent( sharedDrivePathClient )
				.addComponent( alukaMetadataFilePath )
				.addComponent( alukaErrorFilePath )
				.addComponent( alukaDataDirectoryFilePath )
				.addComponent( transformedMetadataPath ))
				
			// Button group                  
			.addGroup( layout.createParallelGroup( GroupLayout.Alignment.LEADING )
				.addComponent( sharedDrivePathBrowseButton )
				.addComponent( alukaMetadataFilePathBrowseButton )
				.addComponent( alukaErrorFilePathBrowseButton )
				.addComponent( alukaDataDirectoryFilePathBrowseButton )
				.addComponent( transformedMetadataPathBrowseButton ))
	    );

	    // Vertically
	    layout.setVerticalGroup( layout.createSequentialGroup()
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )
	        .addComponent( coverageSpatialCountryLabel )
	        .addComponent( coverageSpatialCountry ))
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )
	        .addComponent( coverageSpatialSitenameLabel )
	        .addComponent( coverageSpatialSitename ))
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent( siteLatitudeLabel )
	         .addComponent( siteLatitude ))
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )     
	         .addComponent( siteLongitudeLabel )
	         .addComponent( siteLongitude ))	
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )     
	         .addComponent( sharedDrivePathServerLabel )
	         .addComponent( sharedDrivePathServer ))    
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent( sharedDrivePathClientLabel )
	         .addComponent( sharedDrivePathClient )
	         .addComponent( sharedDrivePathBrowseButton ))       
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent( alukaMetadataFilePathLabel )
	         .addComponent( alukaMetadataFilePath )
	         .addComponent( alukaMetadataFilePathBrowseButton ))  
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent( alukaErrorFilePathLabel )
	         .addComponent( alukaErrorFilePath )
	         .addComponent( alukaErrorFilePathBrowseButton ))   
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent( alukaDataDirectoryFilePathLabel )
	         .addComponent( alukaDataDirectoryFilePath )
	         .addComponent( alukaDataDirectoryFilePathBrowseButton ))  
	     .addGroup( layout.createParallelGroup( GroupLayout.Alignment.BASELINE )   
	         .addComponent( transformedMetadataPathLabel )
	         .addComponent( transformedMetadataPath )
	         .addComponent( transformedMetadataPathBrowseButton ))   
	    ); 
	    
	    JPanel panel2 = new JPanel();
	    panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));

	    final JButton transform = new JButton("Transform");
	    transform.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {

				String validatorInfo = "";
				int missingFieldCount = 0;
				
				// Validation
				if (coverageSpatialCountry.getText().equals("")){
					missingFieldCount++;
				}
				else if (coverageSpatialSitename.getText().equals("")){
					missingFieldCount++;
				}
				else if (siteLatitude.getText().equals("")){
					missingFieldCount++;
				}
				else if (siteLongitude.getText().equals("")){
					missingFieldCount++;
				}
				else if (sharedDrivePathServer.getText().equals("")){
					missingFieldCount++;
				}
				else if (sharedDrivePathClient.getText().equals("")){
					missingFieldCount++;
				}
				else if (alukaMetadataFilePath.getText().equals("")){
					missingFieldCount++;
				}
				else if (alukaErrorFilePath.getText().equals("")){
					missingFieldCount++;
				}
				else if (alukaDataDirectoryFilePath.getText().equals("")){
					missingFieldCount++;
				}
				else if (transformedMetadataPath.getText().equals("")){
					missingFieldCount++;
				}
				
				if (missingFieldCount>0){
					validatorInfo = "Please complete all the fields.";
				}
				else{
					
					String sharedDrivePathClientText = sharedDrivePathClient.getText();
					String alukaMetadataFilePathText = alukaMetadataFilePath.getText();
					String alukaErrorFilePathText = alukaErrorFilePath.getText();
					String alukaDataDirectoryFilePathText = alukaDataDirectoryFilePath.getText();
					String transformedMetadataPathText = transformedMetadataPath.getText();
					
					boolean validated = false;
					String[] pathList = new String[]{sharedDrivePathClientText, alukaMetadataFilePathText, alukaErrorFilePathText, alukaDataDirectoryFilePathText, transformedMetadataPathText};
					String validationString = Utility.getValidationString(pathList);
					
					if (validationString.equals("")){
						validated = true;
					}
					else{
						validatorInfo = "Path Error: Path Does not exist.";
					}
					
					if (validated){
						transform.setEnabled(false);
						Main.transformMetadataProgressBar.setPreferredSize(new Dimension(150,35));
						Main.JProgressBarPanel.add(Main.transformMetadataProgressBar);
						
						validatorInfo = "";
						
						String coverageSpatialCountryText = coverageSpatialCountry.getText();
						String coverageSpatialSitenameText = coverageSpatialSitename.getText();
						String siteLatitudeText = siteLatitude.getText();
						String siteLongitudeText = siteLongitude.getText();
						String sharedDrivePathServerText = sharedDrivePathServer.getText();
						
						// Create a SwingWorker of the TransformMetadataWorker and execute it
			    		BackgroundWorker.TransformMetadataWorker aw = backgroundWorker.new TransformMetadataWorker(Main.transformMetadataProgressBar, coverageSpatialCountryText, coverageSpatialSitenameText, siteLatitudeText, siteLongitudeText, sharedDrivePathServerText, sharedDrivePathClientText, alukaMetadataFilePathText, alukaErrorFilePathText, alukaDataDirectoryFilePathText, transformedMetadataPathText);	
						aw.execute();	
					}
					else{
					    //Console Feedback ---------------------------------------- 
						try
						{
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Error: ", Main.errorAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), "Export Transformed Metadata:   \n", Main.boldAttributes);
							Main.consoleDocument.insertString(Main.consoleDocument.getLength(), validationString, null);
						}
						catch(Exception ex) { System.out.println(ex); }
					    //---------------------------------------------------------- 
					}
				}
				
				Main.transformMetadataValidationLabel.setText("<html><div style=\"text-align: center;\">" + validatorInfo + "</html>");
			}
	    });
	    
	    Main.transformMetadataValidationLabel = new JLabel("");
	    
	    JPanel labelPanel = new JPanel();
	    labelPanel.add(Main.transformMetadataValidationLabel);
	    panel2.add(labelPanel);
	    transform.setPreferredSize(new Dimension((int)(screenWidth*0.15), (int)(screenHeight*0.07)));
	    transform.setMaximumSize(new Dimension((int)(screenWidth*0.15), (int)(screenHeight*0.07)));
	    transform.setMinimumSize(new Dimension((int)(screenWidth*0.15), (int)(screenHeight*0.07)));

	    panel2.add(Main.JProgressBarPanel);
	    panel2.add(Box.createVerticalGlue());
	    JPanel transformPanel = new JPanel();
	    transformPanel.add(transform);
	    panel2.add(transformPanel);
	    panel2.add(Box.createVerticalGlue());
	    
	    JSplitPane editSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, p1, panel2);
	    editSplitPane.setOneTouchExpandable(false);
	    editSplitPane.setDividerLocation( (int)(screenHeight*0.5));
	    editSplitPane.setEnabled(false);
	    editSplitPane.setDividerSize(0);
		
	    Main.transformAlukaMetadataWindow.add(editSplitPane);
	    Main.transformAlukaMetadataWindow.setSize(new Dimension((int)(screenWidth*0.4),(int)(screenHeight*0.75)));
	    Main.transformAlukaMetadataWindow.setLocationRelativeTo(null);
	}

	/**
	 * Popup Import Progress Window
	 * @param  screenWidth  int width of screen
	 * @param  screenHeight  int height of screen
	 */
	public static void popupImportProgress(int screenWidth, int screenHeight, File[] files){
		Main.popupImportProgressWindow = new JFrame("Import Progress");
		
		Main.popupImportProgressWindow.addWindowListener(new java.awt.event.WindowAdapter() {
		    @Override
		    public void windowClosing(java.awt.event.WindowEvent windowEvent) {
		    	
		        //hide and dispose the frame
		    	Main.popupImportProgressWindow.setVisible(false);
		    	Main.popupImportProgressWindow.dispose();
		    }
		});	
		
		Main.popupImportProgressWindow.setSize(new Dimension((int)(screenWidth*0.5), (int)(screenHeight*0.15)));
		Main.popupImportProgressWindow.setVisible(true);
		Main.popupImportProgressWindow.setAlwaysOnTop(true);
		Main.popupImportProgressWindow.setLocationRelativeTo(null);
		
	 	// Set Progress bar settings
		Main.progressImport.setStringPainted(true);
		Main.progressImport.setBackground(Color.white);
		Main.progressImport.setForeground(Color.blue);
		Main.progressImport.setPreferredSize(new Dimension(100, 15));
		Main.progressImport.setVisible(true);
		Main.progressImport.setValue(0);

		Main.popupImportProgressWindow.add(Main.progressImport);
		
		// Create a SwingWorker of the ImportWorker and execute it
		BackgroundWorker.ImportWorker aw = backgroundWorker.new ImportWorker(Main.progressImport, files);	
		aw.execute();
	}

	/**
	 * Popup Validate Metadata Progress Window
	 * @param  screenWidth  int width of screen
	 * @param  screenHeight  int height of screen
	 */
	public static void popupValidateMetadataProgress(int screenWidth, int screenHeight){
		Main.popupValidateMetadataProgressWindow = new JFrame("Validate Metadata Progress");
		
		Main.popupValidateMetadataProgressWindow.addWindowListener(new java.awt.event.WindowAdapter() {
		    @Override
		    public void windowClosing(java.awt.event.WindowEvent windowEvent) {
		    	
		        //hide and dispose the frame
		    	Main.popupValidateMetadataProgressWindow.setVisible(false);
		    	Main.popupValidateMetadataProgressWindow.dispose();
		    }
		});	
		
		Main.popupValidateMetadataProgressWindow.setSize(new Dimension((int)(screenWidth*0.5), (int)(screenHeight*0.15)));
		Main.popupValidateMetadataProgressWindow.setVisible(true);
		Main.popupValidateMetadataProgressWindow.setAlwaysOnTop(true);
		Main.popupValidateMetadataProgressWindow.setLocationRelativeTo(null);
		
	 	// Set Progress bar settings
		Main.validatemetadataProgressBar.setStringPainted(true);
		Main.validatemetadataProgressBar.setBackground(Color.white);
		Main.validatemetadataProgressBar.setForeground(Color.blue);
		Main.validatemetadataProgressBar.setPreferredSize(new Dimension(100, 15));
		Main.validatemetadataProgressBar.setVisible(true);
		Main.validatemetadataProgressBar.setValue(0);

		Main.popupValidateMetadataProgressWindow.add(Main.validatemetadataProgressBar);
		
		// Create a SwingWorker of the ValidateMetadataWorker and execute it
		BackgroundWorker.ValidateMetadataWorker validateMetadataWorker = backgroundWorker.new ValidateMetadataWorker(Main.validatemetadataProgressBar);	
		validateMetadataWorker.execute();	
	}
}
