package zamani.metafy.core.persistence;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.ObjectInputStream;

import zamani.metafy.core.Main;

/** 
 * Deserializer class
 * Used to deserialize stored metadata
 * 
 * @author Michael Ferguson
 * 
 */
public class Deserializer{ 
	
	String serializedFilePath = "";
	
    /**
     * Deserializer Constructor
     * @param serializedFilePath String path to serialized file
    */
	public Deserializer(String serializedFilePath){
		this.serializedFilePath = serializedFilePath;
	}	
	
    /**
     * Deserialize Metadata
     * @return MetadataStore object containing the deserialized metadata data structures
    */
   public MetadataStore deserialzeMetadata(){
 
	   MetadataStore metadataObject = null;
 
	   try{
		   long fileSize = new File(serializedFilePath).length();
		   FileInputStream fin = new FileInputStream(serializedFilePath);
		   metadataObject = readAndDisplayProgress(fin, fileSize);
 
		   return metadataObject;
 
	   }catch(Exception ex){
		   ex.printStackTrace();
		   return null;
	   } 
   } 
   
   /**
    * Deserialize and display progress
    * @param input InputStream inputstream of serialized file
    * @param fileSize double size of the serialized file
    * @return MetadataStore object containing the deserialized metadata data structures
   */
   public static MetadataStore readAndDisplayProgress(InputStream input, double fileSize) throws Exception{
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		int totalRead = 0;
		int read;
		byte[] buffer = new byte[100];

		while ((read = input.read(buffer)) > -1) {
			Main.loadMetadataProgressBar.setValue((int)(Math.ceil(100*(totalRead/(double)fileSize))));
			bos.write(buffer, 0, read);
			totalRead += read;
		}
		
		bos.flush();
		ByteArrayInputStream byteInput = new ByteArrayInputStream(bos.toByteArray());
		ObjectInputStream in = new ObjectInputStream(byteInput);

		MetadataStore metadataObject = (MetadataStore)in.readObject();
		in.close();
		return metadataObject;
	}
}