package zamani.metafy.core.persistence;
import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;

/** 
 * MetadataStore class
 * Used to store serializable metadata
 * 
 * @author Michael Ferguson
 * 
 */
public class MetadataStore implements Serializable{

		private static final long serialVersionUID = -5140043839426320378L;

		ArrayList<String[]> idAndFileTypeList;
		
		ArrayList<ArrayList<String>> metadataFields = new ArrayList<ArrayList<String>>();
		ArrayList<ArrayList<String>> metadata = new ArrayList<ArrayList<String>>();
		
		ArrayList<File> filesInDataset = new ArrayList<File>();
		ArrayList<String> listOfFilePathsInDataset = new ArrayList<String>();
		ArrayList<String> listOfFileCorrespondingPIDSInDataset = new ArrayList<String>();
		Object[][] data;
		ArrayList<String> basicMetadataSchema = new ArrayList<String>();
		String[] automationFields;
		
		int currentPIDCount;
		
		/**
		 * Set Current PID Count
		 * @param currentPIDCount int
		 */
		public void setCurrentPIDCount(int currentPIDCount){
			this.currentPIDCount = currentPIDCount;
		}

		/**
		 * Set Automation Fields
		 * @param automationFields String[]
		 */
		public void setAutomationFields(String[] automationFields){
			this.automationFields = automationFields;
		}
		
		/**
		 * Set Basic Metadata Schema
		 * @param basicMetadataSchema ArrayList<String>
		 */
		public void setBasicMetadataSchema(ArrayList<String> basicMetadataSchema){
			this.basicMetadataSchema = basicMetadataSchema;
		}
		
		/**
		 * Set ID and File Type List
		 * @param idAndFileTypeList ArrayList<String[]>
		 */
	   public void setIDAndFileTypeList(ArrayList<String[]> idAndFileTypeList){
		   this.idAndFileTypeList = idAndFileTypeList;
	   }
 
		/**
		 * Set Metadata Fields
		 * @param metadataFields ArrayList<ArrayList<String>>
		 */
	   public void setMetadataFields(ArrayList<ArrayList<String>> metadataFields){
		   this.metadataFields = metadataFields;
	   }
	   
		/**
		 * Set Metadata
		 * @param metadata ArrayList<ArrayList<String>>
		 */
	   public void setMetadata(ArrayList<ArrayList<String>> metadata){
		   this.metadata = metadata;
	   }
	   
		/**
		 * Set Files in Dataset
		 * @param filesInDataset ArrayList<File>
		 */
	   public void setFilesInDataset(ArrayList<File> filesInDataset){
		   this.filesInDataset = filesInDataset;
	   }
	   
		/**
		 * Set Data
		 * @param data Object[][]
		 */
	   public void setData(Object[][] data){
		   this.data = data;
	   }
	   
		/**
		 * Set List of File Paths in Dataset
		 * @param listOfFilePathsInDataset ArrayList<String>
		 */
	   public void setListOfFilePathsInDataset(ArrayList<String> listOfFilePathsInDataset){
		   this.listOfFilePathsInDataset = listOfFilePathsInDataset;
	   }
	   
		/**
		 * Set List of Corresponding PIDS in Dataset
		 * @param listOfFileCorrespondingPIDSInDataset ArrayList<String>
		 */
	   public void setListOfFileCorrespondingPIDSInDataset(ArrayList<String> listOfFileCorrespondingPIDSInDataset){
		   this.listOfFileCorrespondingPIDSInDataset = listOfFileCorrespondingPIDSInDataset;
	   }
	   
		/**
		 * Get Current PID Count
		 * @return current PID count
		 */
		public int getCurrentPIDCount(){
			return this.currentPIDCount;
		}
	   
		/**
		 * Get Automation Fields
		 * @return automation fields
		 */
	   public String[] getAutomationFields(){
		   return this.automationFields;
	   }

		/**
		 * Get Basic Metadata Schema
		 * @return basic metadata schema 
		 */
	   public ArrayList<String> getBasicMetadataSchema(){
		   return this.basicMetadataSchema;
	   }
	   
		/**
		 * Get ID and File Type List
		 * @return id and file type list
		 */
	   public ArrayList<String[]> getIDAndFileTypeList(){
		   return this.idAndFileTypeList;
	   }
 
		/**
		 * Get Metadata Fields
		 * @return metadata fields
		 */
	   public ArrayList<ArrayList<String>> getMetadataFields(){
		   return this.metadataFields;
	   }
	   
		/**
		 * Get Metadata
		 * @return metadata
		 */
	   public ArrayList<ArrayList<String>> getMetadata(){
		   return this.metadata;
	   }
	   
		/**
		 * Get Files in Dataset
		 * @return files in dataset
		 */
	   public ArrayList<File> getFilesInDataset(){
		   return this.filesInDataset;
	   }
 
		/**
		 * Get Data
		 * @return data 
		 */
	   public Object[][] getData(){
		   return this.data;
	   }
		/**
		 * Get List of File Paths in Dataset
		 * @return list of file paths in dataset 
		 */
	   public ArrayList<String> getListOfFilePathsInDataset(){
		   return this.listOfFilePathsInDataset;
	   }
	   
		/**
		 * Get List of Corresponding PIDS in Dataset
		 * @return list of corresponding PIDs in dataset
		 */
	   public ArrayList<String> getListOfFileCorrespondingPIDSInDataset(){
		   return this.listOfFileCorrespondingPIDSInDataset;
	   }
}