package zamani.metafy.core.utility;
import java.io.File;
import java.text.DecimalFormat;

/** 
 * FileSize  class
 * Converts a file into a human readable file size
 * 
 * @author Michael Ferguson
 * 
 */
public class FileSize
{

    /**
     * Non instantiable Constructor
    */
	private FileSize(){}
	
    /**
     * Convert file to a Human Readable size
     * @param file File 
    */
	public static String humanReadable(File file) {
		long fileBytes = file.length();
		
	    if(fileBytes <= 0) return "0";
	    final String[] units = new String[] { "B", "KB", "MB", "GB", "TB" };
	    int digitGroups = (int) (Math.log10(fileBytes)/Math.log10(1024));
	    return new DecimalFormat("#,##0.#").format(fileBytes/Math.pow(1024, digitGroups)) + " " + units[digitGroups];
	}
}
