package zamani.metafy.core.utility;

/** 
 * FileTypes utility class
 * Deals with the different file types associated and their relevant mime types
 * 
 * @author Michael Ferguson
 * 
 */
public final class FileTypes {
	
    /**
     * Non instantiable Constructor
    */
	private FileTypes(){}
	
	public static String[] supportedFileTypes = {"ply", "wrl", "qs", "ptx", "zip", "mp4", "mpeg", "qt", "dif", "dv", "wmv", "avi", "AVI", "mp3", "jpg", "jpeg", "JPG", "png", "PNG", "tiff", "tif", "TIF", "html", "txt", "TXT", "rtf", "xml", "XML", "imp", "zfs"};
	public static String[] modelType = {"ply", "wrl", "qs"};
	public static String[] individualLaserScansType = {"ptx"};
	public static String[] GISType = {"zip"};
	public static String[] videoType = {"mp4", "mpeg", "qt", "dif", "dv", "wmv", "avi", "AVI"};
	public static String[] imageType = {"jpg", "jpeg", "JPG", "png", "PNG", "tiff", "tif", "TIF"};
	public static String[] cameraCalibrationImageType = { "tiff", "tif", "TIF"};
	public static String[] textType = {"html", "txt", "TXT", "rtf", "xml", "XML"};
	public static String[] rawLaserScansType = {"imp", "zfs"};	
	public static String[] viewableImageType = {"jpg", "jpeg", "JPG", "png", "PNG"};
	
    /**
     * Get Format from file extension
     * @param fileExtension String
     * @return mime type
    */
	public static String getFormat(String fileExtension){
		switch(fileExtension){
			case "jpg":
				return "image/jpeg";
			case "jpeg":
				return "image/jpeg";
			case "tif":			
				return "image/tiff";
			case "tiff":			
				return "image/tiff";
			case "png":
				return "image/png";
			case "gif":
				return "image/gif";
				
			case "ptx":
				return "text/plain";
			case "html":
				return "text/html";	
			case "txt":
				return "text/plain";
			case "rtf":
				return "text/rtf";			
				
			case "ply":
				return "model/mesh";
			case "qs":
				return "model/mesh";
			case "wrl":
				return "model/vrml";
				
			case "pdf":
				return "application/pdf";
			case "zip":
				return "application/zip";			
			case "zipx":
				return "application/zip";				
			case "xml":
				return "application/xml";
				
			case "mp4":
				return "video/mp4";			
			case "mpeg":
				return "video/mpeg";				
			case "qt":
				return "video/quicktime";				
			case "dif":
				return "video/x-dv";
			case "dv":
				return "video/x-dv";			
			case "wmv":
				return "video/x-ms-wmv";
			case "avi":
				return "video/x-msvideo";
		}	
		return "";
	}
}