package zamani.metafy.core.utility;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

/** 
 * HttpGet utility class
 * Used to process HTTP Get requests, required to quickly get directory count from server opposed to calculating it on the client side
 * 
 * @author Michael Ferguson
 * 
 */
public class HttpGet {

	    /**
	     * Non instantiable Constructor
	    */
		private HttpGet(){}
		
		private final static String USER_AGENT = "Mozilla/5.0";
	  
		
		// HTTP GET request
		/**
		 * Get Directory Count from server using an HTTP GET request
		 * @param url String get url
		 * @return directory count
		 */
		public static int getDirectoryCount(String url) throws Exception {
			
			URL obj = new URL(url);
			HttpURLConnection con = (HttpURLConnection) obj.openConnection();
	 
			// optional default is GET
			con.setRequestMethod("GET");
	 
			//add request header
			con.setRequestProperty("User-Agent", USER_AGENT);
	 
			int responseCode = con.getResponseCode();

			if (responseCode==200){
				BufferedReader in = new BufferedReader(
				        new InputStreamReader(con.getInputStream()));
				String inputLine;
				StringBuffer response = new StringBuffer();
		 
				while ((inputLine = in.readLine()) != null) {
					response.append(inputLine);
				}
				in.close();
				
				if (response.toString().equals("")){
					return -1;
				}
		 
				return Integer.parseInt(response.toString());
			}
			else{
				return -1;
			}
		}
}