package zamani.metafy.core.utility;
import java.io.File;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/** 
 * ServerPath utility class
 * Deals with conversions to Server Path representations
 * 
 * @author Michael Ferguson
 * 
 */
public class ServerPath {
	
    /**
     * Non instantiable Constructor
    */
	private ServerPath(){}
	
    /**
     * Get File Path On Server
     * @param filePath String
     * @param sharedDrivePathServer String
     * @param sharedDrivePathClient String
    */
	public static String getFilepathOnServer(String filePath, String sharedDrivePathServer, String sharedDrivePathClient){
		
		// Build files path on server
		String filePathOnServer = "";
		String formattedFilePath = "";
		
		// Remove root (clients shared drive path) from filepath (e.g. C:\Zamani)
		formattedFilePath = filePath.replace(sharedDrivePathClient, "");
		// Format windows path to ubuntu path
		formattedFilePath = formattedFilePath.replace("\\", "/");
		// Construct servers path 
		
		// Ensure that concatenation contains "/"
		if(!formattedFilePath.substring(0,1).equals("/") && !sharedDrivePathServer.substring(sharedDrivePathServer.length() - 1).equals("/")){
			filePathOnServer = sharedDrivePathServer+"/"+formattedFilePath;
		}
		else{
			filePathOnServer = sharedDrivePathServer + formattedFilePath;
		}
		
		return filePathOnServer;
	}
	
    /**
     * Find Longest Prefix Suffix
     * @param s1 String suffix string
     * @param s2 String prefix string
    */
	public static String findLongestPrefixSuffix(String s1, String s2) {
		
		   for( int i = Math.min(s1.length(), s2.length()); ; i--) {
		      if(s2.endsWith(s1.substring(0, i))) {
		         return s1.substring(0, i);
		      }
		   }		
	}
	
    /**
     * Find Longest Prefix Suffix
     * @param dirToAssociate String
     * @param pathToSharedDriveServer String
    */
	public static String calculateServerDirectoryPath(String dirToAssociate, String pathToSharedDriveServer){
		String driveLetter = (new File(dirToAssociate)).toPath().getRoot().toString();

		// Remove drive Letter from directory to associate
		String serverDirectoryPath = dirToAssociate.replace(driveLetter, "");
		
		// Format path to ubuntu path
		serverDirectoryPath = serverDirectoryPath.replace("\\","/");
		
		if(serverDirectoryPath.substring(0,1).equals("/") && pathToSharedDriveServer.substring(pathToSharedDriveServer.length() - 1).equals("/")){
			serverDirectoryPath = pathToSharedDriveServer.substring(pathToSharedDriveServer.length() - 1)+serverDirectoryPath;
		}
		else if(serverDirectoryPath.substring(0,1).equals("/") || pathToSharedDriveServer.substring(pathToSharedDriveServer.length() - 1).equals("/")){
			serverDirectoryPath = pathToSharedDriveServer+serverDirectoryPath;
		}
		else{
			serverDirectoryPath = pathToSharedDriveServer+"/"+serverDirectoryPath;
		}
		
		return serverDirectoryPath;
	}
	
    /**
     * Calculate New File Paths
     * Calculate new Server and Client paths using Aluka Data Directory path
     * @param filePath String
     * @param alukaDataDirectoryPath String
     * @param sharedDrivePathServer String
     * @param sharedDrivePathClient String
    */
	public static String[] calculateNewFilePaths(String filePath, String alukaDataDirectoryPath, String sharedDrivePathServer, String sharedDrivePathClient){
		
		if (!filePath.equals("undefined")){
			// Build files path on server
			String filePathOnServer = "";
			String formattedFilePath = "";
			String formattedDataDirectoryPath = "";
	
			// Remove root (clients shared drive path) from filepath (e.g. C:\Zamani)
			formattedDataDirectoryPath = alukaDataDirectoryPath.replace(sharedDrivePathClient, "");
			
			// Format windows path to ubuntu path
			formattedDataDirectoryPath = formattedDataDirectoryPath.replace("\\", "/");
			
			formattedFilePath = filePath.replace("\\", "/");
			
			// Remove C:/
			Pattern p = Pattern.compile("^([CDEFGHIJKLMNOPQRSTUVWXYZ]):/");
			Matcher m = p.matcher(formattedFilePath);
			formattedFilePath = m.replaceAll("");
			
			/* E.g of prefix suffix
			* s1 = Sudan/Musawwarat/images/Photogrammetric Lion Temple Submit/SUD-8080.tif
			* s2 = Zamani-Core/Sudan/Musawwarat
			* LongestPrefixSuffix = Sudan/Musawwarat
			*/
			String longestPrefixSuffix = findLongestPrefixSuffix(formattedFilePath, formattedDataDirectoryPath);
			
			//Remove LongestPrefixSuffix from formatted Data Directory path
			formattedDataDirectoryPath = formattedDataDirectoryPath.replace(longestPrefixSuffix, "");
	
			// Construct new client path
			String filePathOnClient = sharedDrivePathClient + formattedDataDirectoryPath + formattedFilePath;
			// Format path to windows path
			filePathOnClient = filePathOnClient.replace("/", "\\");
			
			// Construct servers path 
			filePathOnServer = sharedDrivePathServer + formattedDataDirectoryPath + formattedFilePath;
			
			String[] clientAndServerPath = new String[]{filePathOnClient, filePathOnServer};
			return clientAndServerPath;
		}
		else{
			String[] clientAndServerPath = new String[]{"undefined", "undefined"};
			return clientAndServerPath;
		}
		
	}
}
