package zamani.metafy.core.utility;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GraphicsEnvironment;
import java.awt.Insets;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.URL;
import java.util.ArrayList;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

/** 
 * Utility Enum with no instance 
 * Contains utility functions
 * 
 * @author Michael Ferguson
 * 
 */
public enum Utility {;

	/**
	 * Array Contains
	 * @param  string  String string to check
	 * @param  array  String[] array used to check if string exists in it 
	 * @return True if contained, False if not
	 */
	public static boolean arrayContains(String string, String[] array){
		for (String s : array){
			if (s.equals(string)){
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Array Contains Null
	 * @param  string  String string to check
	 * @param  array  String[] array used to check if string exists in it 
	 * @return True if contained, False if not contained or array null or of 0 length
	 */
	public static boolean arrayContainsNull(String string, String[] array){
		if (array==null || array.length==0){
			return false;	
		}
		else{
			return arrayContains(string,array);
		}
	}
	
	/**
	 * Get Index of Element in array
	 * @param  string  String string to check
	 * @param  array  String[] array used to find strings index 
	 * @return Index of element in array, -1 if element does not exist
	 */
	public static int getElementIndex(String string, String[] array){
		for (int i=0; i<array.length; i++){
			if (array[i].equals(string)){
				return i;
			}
		}
		return -1;
	}
	
	/**
	 * Round double to a certain number of decimals
	 * @param  value  double value to round
	 * @param  places  int number of decimal places to round to
	 * @return rounded double
	 */
	public static double round(double value, int places) {
	    if (places < 0) throw new IllegalArgumentException();

	    BigDecimal bd = new BigDecimal(value);
	    bd = bd.setScale(places, RoundingMode.HALF_UP);
	    return bd.doubleValue();
	}

	/**
	 * Get Indent of a given length
	 * @param  length  int string to check
	 * @return indented string of length
	 */
	public static String indent(int length){
		return new String(new char[length]).replace("\0", "&nbsp;");
	}
	
	/**
	 * Get extension from filename
	 * @param  fileName  String filename string
	 * @return extension of given filename, null if no extension exists
	 */
	public static String getExtension(String fileName){
		int i = fileName.lastIndexOf('.');
		if (i > 0) {
		    return fileName.substring(i+1);
		}
		
		return null;
	}

	/**
	 * Get Formatted Path
	 * @param  filePath  String
	 * @return formatted filepath
	 */
	public static String getFormattedPath(String filePath){
		int i = filePath.lastIndexOf('\\');
		int t = 0;
		String fileName = "<b>"+filePath.substring(i+1)+"</b>";
		if (i >= 0) {
			t = filePath.substring(0, i).lastIndexOf('\\');	   
		}
		if (i >=0){
			 return filePath.substring(t+1,i+1).replace("\\","/")+fileName;
		}

		return null;
	}
	
	/**
	 * Get Screen Dimensions
	 * @param  frame  Frame
	 * @return Screen Dimensions
	 */
	public static Dimension getScreenDimension(Frame frame){
		GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
		Dimension screenDimension = env.getMaximumWindowBounds().getSize();
		
		Insets insets = frame.getInsets();
		final int left = insets.left;
		final int right = insets.right;
		final int top = insets.top;
		final int bottom = insets.bottom;
	
		final int width = screenDimension.width - left - right;
		final int height = screenDimension.height - top - bottom;
		
		return new Dimension(width,height);
	}
	
	/**
	 * Get File Path JFileChooser
	 * @param  dialogTitle  String
	 * @param  parent  Component parent component of the JFileChooser
	 * @return file path
	 */
    public static String getFilePath(String dialogTitle, Component parent){
	    JFileChooser chooser = new JFileChooser(); 
	    chooser.setCurrentDirectory(new java.io.File("."));
	    chooser.setDialogTitle(dialogTitle);
	    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY); 
	    chooser.setAcceptAllFileFilterUsed(false);   									// disable the "All files" option.
	    
	    String pathToSharedDrive = "";
	    
	    if (chooser.showOpenDialog(parent) == JFileChooser.APPROVE_OPTION) { 
	    	pathToSharedDrive = chooser.getSelectedFile().toString();
	    }
	    
	    return pathToSharedDrive;
    }
    
	/**
	 * Get File Save Path JFileChooser
	 * Limited to .metafy file extensions
	 * @param  dialogTitle  String
	 * @param  parent  Component parent component of the JFileChooser
	 * @return file path
	 */
    public static String getFileSavePath(String dialogTitle, Component parent){
    	FileFilter ff = new FileNameExtensionFilter("Metafy File", "metafy");
	    JFileChooser chooser = new JFileChooser(); 
	    chooser.setCurrentDirectory(new java.io.File("."));
	    chooser.setDialogTitle(dialogTitle);
	    chooser.addChoosableFileFilter(ff);  
    	chooser.setAcceptAllFileFilterUsed(false);
    	chooser.setFileFilter(ff);
	    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY); 
	    chooser.setAcceptAllFileFilterUsed(false);   									// disable the "All files" option.
	    
	    String pathToSharedDrive = "";
	    
	    if (chooser.showSaveDialog(parent) == JFileChooser.APPROVE_OPTION) { 
	    	pathToSharedDrive = chooser.getSelectedFile().toString();
	    }
	    
	    return pathToSharedDrive;
    }
    
	/**
	 * Get Directory Path JFileChooser
	 * Limited to .metafy file extensions
	 * @param  dialogTitle  String
	 * @param  parent  Component parent component of the JFileChooser
	 * @return directory path
	 */
    public static String getDirectoryPath(String dialogTitle, Component parent){
	    JFileChooser chooser = new JFileChooser(); 
	    chooser.setCurrentDirectory(new java.io.File("."));
	    chooser.setDialogTitle(dialogTitle);
	    chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY); 
	    chooser.setAcceptAllFileFilterUsed(false);  
	    
	    String directoryPath = "";
	    
	    if (chooser.showOpenDialog(parent) == JFileChooser.APPROVE_OPTION) { 
	    	directoryPath = chooser.getSelectedFile().toString();
	    }
	    
	    return directoryPath;
    }
    
	/**
	 * Get Load Path JFileChooser
	 * @param dialogTitle String
	 * @param  parent  Component parent component of the JFileChooser
	 * @return save path
	 */
    public static String getLoadPath(String dialogTitle, Component parent){
    	FileFilter ff = new FileNameExtensionFilter("Metafy File", "metafy");
    	JFileChooser fileChooser = new JFileChooser();
    	fileChooser.setDialogTitle(dialogTitle);
    	fileChooser.addChoosableFileFilter(ff);  
    	fileChooser.setAcceptAllFileFilterUsed(false);
    	fileChooser.setFileFilter(ff);
    	fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY); 
    	fileChooser.setAcceptAllFileFilterUsed(false);  
    	
    	int userSelection = fileChooser.showOpenDialog(parent);
    	 
    	if (userSelection == JFileChooser.APPROVE_OPTION) {
    	    File fileToSave = fileChooser.getSelectedFile();
    	    return fileToSave.getPath();
    	}
    	else{
    		return "";
    	}
    }
    
	/**
	 * Capitalize a string
	 * @param  string  String string to capitalize
	 * @return string with first character capitalized
	 */
    public static String capitalize(final String string)
    {
       if (string == null){
      	 return "";
       }
       if (string.equals("")){
      	 return "";
       }

       return Character.toUpperCase(string.charAt(0)) + string.substring(1);
    }
    
	/**
	 * Pad String with specified padding and length
	 * @param  string  String string to capitalize
	 * @param  padAmount  int amount to pad
	 * @param  padSymbol  String character used for padding
	 * @return padded string
	 */
	public static String padString(String string,int padAmount, String padSymbol){
		String padding = "";
		for (int i=0; i<padAmount-string.length();i++){
			padding += padSymbol;
		}
		
		return padding+string;
	}
	
	/**
	 * Convert String to Camel Case
	 * @param  sentence  String sentence to convert to camel case
	 * @return camel case converted string
	 */
	public static String toCamelCase(String sentence) {
		
		String sentenceFormatted = sentence.replaceAll("[^a-zA-Z0-9\\s]", " ");
		
	    String out = "";
	    String[] splitSentence = sentenceFormatted.split(" ");
	    String temp;
	    
	    for (int i=0; i<splitSentence.length; i++){
	    	temp = splitSentence[i];
	    	temp = temp.substring(0, 1).toUpperCase() + temp.substring(1);	
	    	out += temp;
	    }
	    return out;
	}
	
	/**
	 * Remove filename/filepath extension
	 * @param  s  String filename/filepath string
	 * @return filename/filepath without extension
	 */
	public static String removeExtension(String s) {

	    String separator = System.getProperty("file.separator");
	    String filename;

	    // Remove the path upto the filename.
	    int lastSeparatorIndex = s.lastIndexOf(separator);
	    if (lastSeparatorIndex == -1) {
	        filename = s;
	    } else {
	        filename = s.substring(lastSeparatorIndex + 1);
	    }

	    // Remove the extension.
	    int extensionIndex = filename.lastIndexOf(".");
	    if (extensionIndex == -1)
	        return filename;

	    return filename.substring(0, extensionIndex);
	}
	
	/**
	 * Clone Array with blank first index
	 * @param  array  String[] array to clone
	 * @return array with blank first index
	 */
	public static String[] cloneArrayWithBlankFirstIndex(String[] array){
		int length = array.length+1;
		String[] newArray = new String[length];
		
		newArray[0] = "";
		int index = 1;
		for (String s : array){
			newArray[index] = s;
			index++;
		}
		
		return newArray;
	}
	
	/**
	 * Read Text File
	 * @param  filePath  URL filepaths URL
	 * @return arraylist of strings containing each line in text file
	 */
    public static ArrayList<String> readTextFile(URL filePath){
    	ArrayList<String> listLines = null;
    	try{
    		  listLines = new ArrayList<String>();
    		  InputStream fstream = filePath.openStream();
    		  DataInputStream in = new DataInputStream(fstream);
    		  BufferedReader br = new BufferedReader(new InputStreamReader(in));
    		  String strLine;
    		  //Read file line by line
    		  while ((strLine = br.readLine()) != null)   {
    			  listLines.add(strLine);
    		  }
    		  //Close the input stream
    		  in.close();
    	}catch (Exception e){
    		  System.err.println("Error: " + e.getMessage());
    	}   
    	
    	return listLines;
    }
    
	/**
	 * Validate Filepath
	 * @param  path  String filepath to validate
	 * @return true if filepath exists and is valid, false if not
	 */
    public static boolean validatePath(String path){
    	if ((new File(path)).exists()){
    		return true;
    	}
    	else{
    		return false;
    	}
    }

	/**
	 * Get Validation String
	 * @param  paths  String[] array of filepaths
	 * @return validation string
	 */
    public static String getValidationString(String[] paths){
    	String validationString = "";
    	for (String path : paths){
    		if (!validatePath(path)){
    			validationString += "File Path Does not exist: "+path+" \n";
    		}
    	}
    	return validationString;
    }
    
	/**
	 * Get Colour at specific intensity
	 * @param  power  double power of intensity
	 * @return colour of intensity
	 */
    public static Color getColor(double power)
    {
    	power = power/100;
    	
        double H = power * 0.4; // Hue (0.4 = Green)
        double S = 0.9; // Saturation
        double B = 0.9; // Brightness

        return Color.getHSBColor((float)H, (float)S, (float)B);
    }
}