package zamani.metafy.core.xml;
import java.io.File;
import java.util.ArrayList;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/** 
 * XMLReader class
 * Reads in an Aluka xml file and parses it into the relevant Metadata data structures
 * 
 * @author Michael Ferguson
 * 
 */
public class XMLReader {

	public static ArrayList<ArrayList<String>> metadataFieldsList;
	public static ArrayList<ArrayList<String>> metadataList;
		
	static ArrayList<String> metadataFields;
	static ArrayList<String> metadata;
	
	public static String identifierPrefix;
	public static String sourceOfData;
	public static int currentPIDcount;
	static String[] spatialTypes;
	
	public static ArrayList<String[]> idAndFileTypeList;

    /**
     * XMLReader Constructor
     * Reads Aluka xml file and parses it into the Metadata data structures
     * Makes use of getTagValuesDepthTwo and getTagValuesDepthThree to access all the relevant xml values
     * @param xmlFilePath String
    */
	 public XMLReader(String xmlFilePath) {
	
		 idAndFileTypeList = new ArrayList<String[]>();
		 metadataFieldsList = new ArrayList<ArrayList<String>>();
		 metadataList = new ArrayList<ArrayList<String>>();
		 	
		 metadataFields = new ArrayList<String>();
		 metadata = new ArrayList<String>();
		 
		 identifierPrefix = "";
		 sourceOfData = "";
		 currentPIDcount = 0;
		 
	  try {
	  File file = new File(xmlFilePath);
	  DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	  DocumentBuilder db = dbf.newDocumentBuilder();
	  Document doc = db.parse(file);
	  doc.getDocumentElement().normalize();
	  
	  
	  spatialTypes = new String[]{"Coverage Spatial Country", "Coverage Spatial Name", "Coverage Spatial Structure", "Coverage Spatial Position", "Coverage Spatial Part", "Coverage Spatial Locality"};
	
	  NodeList nodeLst = doc.getElementsByTagName("record");
	  int index=0;
	  for (int s = 0; s < nodeLst.getLength(); s++) {
	    Node fstNode = nodeLst.item(s);
	    
	    if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
	    	
	    	getTagValuesDepthTwo(fstNode, "description", "Description");
	    	String PID = getTagValuesDepthTwo(fstNode, "identifier", "Identifier");
	    	
	    	String[] tempArray = new String[5];
	    	tempArray[0] = PID;
	    	tempArray[1] = "";
	    	tempArray[2] = index+"";
	    	tempArray[3] = "Red";
	    	
	    	getTagValuesDepthTwo(fstNode, "title", "Title");
	    	getTagValuesDepthTwo(fstNode, "creator", "Creator");
	    	getTagValuesDepthTwo(fstNode, "date", "Date");
	    	getTagValuesDepthTwo(fstNode, "format", "Format");
	    	getTagValuesDepthTwo(fstNode, "source", "Source");
	    	getTagValuesDepthTwo(fstNode, "type", "Type");
	    	getTagValuesDepthTwo(fstNode, "coverage.spatial", "Coverage Spatial");	
	    	
	    	getTagValuesDepthThree(fstNode, "position", "Position Latitude", "latitude");    	
	    	getTagValuesDepthThree(fstNode, "position", "Position Langitude", "longitude");
	    	getTagValuesDepthThree(fstNode, "position", "Position Ellipse", "ellipse");
	    	
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Date", "date");
	    	String camera = getTagValuesDepthThree(fstNode, "calibration", "Calibration Camera", "camera");
	    	tempArray[4] = camera;
	    	
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Decentering P1", "decentering.p1");
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Decentering P2", "decentering.p2");
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Distortion Affine B1", "distortion.affine.b1");
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Distortion Affine B2", "distortion.affine.b2");
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Distortion Radial K1", "distortion.radial.k1");
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Distortion Radial K2", "distortion.radial.k2");
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Distortion Radial K3", "distortion.radial.k3");
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Length Focal", "length.focal");
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Pixel Size X", "pixel.size.x");    	
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Pixel Size Y", "pixel.size.y");
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Principal Point X", "principal.point.x");
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Principal Point Y", "principal.point.y");    	
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Size X", "size.x");
	    	getTagValuesDepthThree(fstNode, "calibration", "Calibration Size Y", "size.y");
	
	    	
	    	metadataFieldsList.add(metadataFields);
	    	metadataList.add(metadata);
	    	idAndFileTypeList.add(tempArray); 
	    	
	    	// Reset arrays
	    	metadataFields = new ArrayList<String>();
	    	metadata = new ArrayList<String>();
	    	index++;
	    }
	    
	    if (s==nodeLst.getLength()-1){
	    	String PID = getTagValuesDepthTwo(fstNode, "identifier", "Identifier");
	    	sourceOfData = getTagValuesDepthTwo(fstNode, "source", "Source");
	    	
	    	String[] splitPID = PID.split(":");
	    	    	
	    	identifierPrefix = splitPID[0];
	    	currentPIDcount = Integer.parseInt(splitPID[1].replaceFirst("^0+(?!$)", ""));
	    }
	    
	  }
	  
	  } catch (Exception e) {
	    e.printStackTrace();
	  }
	 }
	 
    /**
     * Get Tag Names and Values of Depth Two and add them to the metadata and metadata fields lists
     * @param fstNode Node
     * @param tag String
     * @param tagFormattedName String
    */
	 public static String getTagValuesDepthTwo(Node fstNode, String tag, String tagFormattedName){
	     Element fstElmnt = (Element) fstNode;
	     NodeList fstNmElmntLst = fstElmnt.getElementsByTagName(tag);
	     String value = "";
	     int spatialTypeIndex = 0;
	     int dataIndex = 0;
	     for (int i=0; i<fstNmElmntLst.getLength(); i++){    
	    	 value = ((Node) fstNmElmntLst.item(i).getChildNodes().item(0)).getNodeValue();
	    	 
	    	 if (tag.equals("identifier")){
	    		 String PID = value.trim().replaceAll("(?<=\\p{L})(?=\\d)", ":");
	         	 metadata.add(PID);
	        	 metadataFields.add(tagFormattedName); 
	        	 return PID;
	    	 }
	    	 else if(tag.equals("description")){
	        	 metadata.add(value.trim());
	        	 metadataFields.add(tagFormattedName); 
	        	 return "";
	    	 }
	    	 else if (tagFormattedName.equals("Coverage Spatial")){
	    		 String spatialName = spatialTypes[spatialTypeIndex];
	        	 metadata.add(value.trim());
	        	 metadataFields.add(spatialName); 
	        	 spatialTypeIndex++;
	    	 }
	    	 else if (tagFormattedName.equals("Date")){
	    		 // Only process the first data (Second one is the Camera calibration date)
	    		if (dataIndex==0){
	           	 	metadata.add(value.trim());
	           	 	metadataFields.add(tagFormattedName); 
	    		}
	    		 dataIndex++;
	    	 }
	    	 else{
	        	 metadata.add(value.trim());
	        	 metadataFields.add(tagFormattedName); 
	    	 }
	     }
	     
	     return value.trim();
	 }
	 
	    /**
	     * Get Tag Names and Values of Depth Three and add them to the metadata and metadata fields lists
	     * @param fstNode Node
	     * @param tag String
	     * @param tagFormattedName String
	     * @param tagChild String
	    */
	 public static String getTagValuesDepthThree(Node fstNode, String tag, String tagFormattedName, String tagChild){
	     Element fstElmnt = (Element) fstNode;
	     NodeList fstNmElmntLst = fstElmnt.getElementsByTagName(tag);
	     String value = "";
	     
	     if (fstNmElmntLst.getLength()!=0){
		     NodeList list = ((Element)fstNmElmntLst.item(0)).getElementsByTagName(tagChild);
		     for (int i=0; i<list.getLength(); i++){
		    	 value = ((Node) list.item(i).getChildNodes().item(0)).getNodeValue();
		    	 metadata.add(value.trim());
		    	 metadataFields.add(tagFormattedName);
		     }
	     }
	     return value;
	 }

}