package zamani.metafy.core.xml;
import java.io.File;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
 
/** 
 * XMLWriter class
 * Outputs Metadata Item xml files linked to a site as well as a Country and Site Collection xml file
 * 
 * @author Michael Ferguson
 * 
 */
public class XMLWriter {

    /**
     * Write Metadata Item XML file
     * @param metadataFields ArrayList<String>
     * @param metadata ArrayList<String>
     * @param outputfilePath String
     * @param collectionPID String
     * @param filePathServer String
     * @param filePathClient String
     * @param fileSize String
    */
	public static void writeXMLFile(ArrayList<String> metadataFields, ArrayList<String> metadata, String outputfilePath, String collectionPID, String filePathServer, String filePathClient, String fileSize) {

	  try {
 
		DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
 
		// root elements
		Document doc = docBuilder.newDocument();
		Element rootElement = doc.createElement("foxml:digitalObject");
		doc.appendChild(rootElement);
		
		// set attributes of root element
		rootElement.setAttribute("VERSION", "1.1");
		rootElement.setAttribute("PID", metadata.get(metadataFields.indexOf("Identifier")));
		rootElement.setAttribute("xmlns:foxml", "info:fedora/fedora-system:def/foxml#");
		rootElement.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
		rootElement.setAttribute("xsi:schemaLocation", "info:fedora/fedora-system:def/foxml# http://www.fedora.info/definitions/1/0/foxml1-1.xsd");

		// Object Properties element
		Element objectProperties = doc.createElement("foxml:objectProperties");
		rootElement.appendChild(objectProperties);
 
		// property element
		Element property = doc.createElement("foxml:property");
		objectProperties.appendChild(property);
		// property attributes
		property.setAttribute("NAME", "info:fedora/fedora-system:def/model#state");
		property.setAttribute("VALUE", "Active");
		
		// property element
		Element property2 = doc.createElement("foxml:property");
		objectProperties.appendChild(property2);
		// property attributes
		property2.setAttribute("NAME", "info:fedora/fedora-system:def/model#label");
		// **Title**
		property2.setAttribute("VALUE", metadata.get(2));
 
		
		// datastream element
		Element datastream = doc.createElement("foxml:datastream");
		rootElement.appendChild(datastream);
		// set attributes
		datastream.setAttribute("ID", "DC");
		datastream.setAttribute("STATE", "A");
		datastream.setAttribute("CONTROL_GROUP", "X");
		
		// datastreamVersion element
		Element datastreamVersion = doc.createElement("foxml:datastreamVersion");
		datastream.appendChild(datastreamVersion);
		// property attributes
		datastreamVersion.setAttribute("FORMAT_URI", "http://www.openarchives.org/OAI/2.0/oai_dc/");
		datastreamVersion.setAttribute("ID", "DC.1");
		datastreamVersion.setAttribute("MIMETYPE", "text/xml");
		datastreamVersion.setAttribute("LABEL", "Dublin Core Record for this object");
		
		// xml Content element
		Element xmlContent = doc.createElement("foxml:xmlContent");
		datastreamVersion.appendChild(xmlContent);
		
		
		// dc element
		Element dc = doc.createElement("oai_dc:dc");
		xmlContent.appendChild(dc);
		// property attributes
		dc.setAttribute("xmlns:oai_dc", "http://www.openarchives.org/OAI/2.0/oai_dc/");
		dc.setAttribute("xmlns:dc", "http://purl.org/dc/elements/1.1/");
		
		
		//position and calibration
		boolean calibrationSettings = false;
		boolean calibrationSettingsClose = false;
		boolean positionSettings = false;
		boolean positionSettingsClose = false;
		int positionInitialIndex = -1;
		int calibrationInitialIndex = -1;
		
		Element calibrationField = null;
		Element positionField = null;
		
		
		int index=0;
		for (String fieldName: metadataFields){
			// nickname elements
			String fieldNameFormatted = "";
			
			if (fieldName.contains("Position") && !fieldName.contains("Coverage")){
				positionSettings = true;
				if(positionInitialIndex==-1){
					positionInitialIndex = index;
				}
			}
			else{
				if (positionSettings){
					positionSettings = false;
					positionSettingsClose = true;
				}
			}
			
			if (fieldName.contains("Calibration")){
				calibrationSettings = true;
				if(calibrationInitialIndex==-1){
					calibrationInitialIndex = index;
				}
			}
			else{
				if (calibrationSettings){
					calibrationSettings = false;
					calibrationSettingsClose = true;
				}
			}
			
			if (positionSettings && positionInitialIndex>-1){
				positionField = doc.createElement("position");
				positionInitialIndex = -2;
			}
			
			if (calibrationSettings && calibrationInitialIndex>-1){
				calibrationField = doc.createElement("calibration");
				calibrationInitialIndex = -2;
			}
			
			if (calibrationSettings){
				
				if (fieldName.contains(" ")){
					fieldNameFormatted = toCamelCase(fieldName.replace("Calibration ", "").toLowerCase());
				}
				else{
					fieldNameFormatted = fieldName.replace("Calibration ", "").toLowerCase();
				}
				
				Element field = doc.createElement(fieldNameFormatted);
				String metadataValue = metadata.get(index) == null ? "": metadata.get(index);
				
				field.appendChild(doc.createTextNode(metadataValue));
				calibrationField.appendChild(field);
				
				if (metadataFields.size()==index+2){
					calibrationSettingsClose = true;
				}	
			}
			
			if (positionSettings){
				if (fieldName.contains(" ")){
					fieldNameFormatted = fieldName.replace("Position ", "").toLowerCase();
				}
				//System.out.println("dc:"+fieldNameFormatted);
				Element field = doc.createElement(fieldNameFormatted);
				String metadataValue = metadata.get(index) == null ? "": metadata.get(index);
				
				field.appendChild(doc.createTextNode(metadataValue));
				positionField.appendChild(field);
				
				if (metadataFields.size()==index+2){
					positionSettingsClose = true;
				}
			}
			
			if (!positionSettings && !calibrationSettings){
				
				if (fieldName.contains(" ")){
					fieldNameFormatted = fieldName.split(" ")[0].toLowerCase();
				}
				else{
					fieldNameFormatted = fieldName.toLowerCase();
				}
				
				Element field = doc.createElement("dc:"+fieldNameFormatted);
				String metadataValue = metadata.get(index) == null ? "": metadata.get(index);
				
				field.appendChild(doc.createTextNode(metadataValue));
				dc.appendChild(field);
			}
			
			index++;
		}
		
		//------------------------------------------------------------------------
		// *RDF**
		// datastream element
		Element rdfDatastream = doc.createElement("foxml:datastream");
		rootElement.appendChild(rdfDatastream);
		// set attributes
		rdfDatastream.setAttribute("ID", "RELS-EXT");
		rdfDatastream.setAttribute("STATE", "A");
		rdfDatastream.setAttribute("CONTROL_GROUP", "X");
		//rdfDatastream.setAttribute("VERSIONABLE", "true");
		
		// datastreamVersion element
		Element rdfDatastreamVersion = doc.createElement("foxml:datastreamVersion");
		rdfDatastream.appendChild(rdfDatastreamVersion);
		// property attributes
		rdfDatastreamVersion.setAttribute("FORMAT_URI", "info:fedora/fedora-system:FedoraRELSExt-1.0");
		rdfDatastreamVersion.setAttribute("ID", "RELS-EXT.0");
		rdfDatastreamVersion.setAttribute("MIMETYPE", "application/rdf+xml");
		rdfDatastreamVersion.setAttribute("LABEL", "RDF Statements about this object");
					
		// xml Content element
		Element xmlContent2 = doc.createElement("foxml:xmlContent");
		rdfDatastreamVersion.appendChild(xmlContent2);
		
		// rdf element
		Element rdf = doc.createElement("rdf:RDF");
		xmlContent2.appendChild(rdf);
		// property attributes
		rdf.setAttribute("xmlns:fedora-model", "info:fedora/fedora-system:def/model#");
		rdf.setAttribute("xmlns:rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
		rdf.setAttribute("xmlns:rel", "info:fedora/fedora-system:def/relations-external#");
	
		// rdf Description element
		Element rdfDescription = doc.createElement("rdf:Description");
		rdf.appendChild(rdfDescription);
		// property attributes
		rdfDescription.setAttribute("rdf:about", "info:fedora/"+metadata.get(1));
		
		// rel element
		Element relCollection = doc.createElement("rel:isMemberOf");
		rdfDescription.appendChild(relCollection);

		relCollection.setAttribute("rdf:resource", "info:fedora/"+collectionPID);
		//------------------------------------------------------------------------
		
		//------------------------------------------------------------------------
		// *SITE* DATASTREAM
		// datastream element
		Element fileDatastream = doc.createElement("foxml:datastream");
		rootElement.appendChild(fileDatastream);
		
		// set attributes
		fileDatastream.setAttribute("ID", "FILE");
		fileDatastream.setAttribute("STATE", "A");
		fileDatastream.setAttribute("CONTROL_GROUP", "X");
		
		// datastreamVersion element
		Element siteDatastreamVersion = doc.createElement("foxml:datastreamVersion");
		fileDatastream.appendChild(siteDatastreamVersion);
		// property attributes
		siteDatastreamVersion.setAttribute("ID", "FILE1.0");
		siteDatastreamVersion.setAttribute("MIMETYPE", "text/xml");
		siteDatastreamVersion.setAttribute("LABEL", "File info about this object");
		
		// xml Content digest element
		Element xmlContent3 = doc.createElement("foxml:xmlContent");
		siteDatastreamVersion.appendChild(xmlContent3);
		
		// rdf Description element
		Element siteElement = doc.createElement("file");
		xmlContent3.appendChild(siteElement);
		
		Element sizeElement = doc.createElement("size");
		sizeElement.appendChild(doc.createTextNode(fileSize));
		
		
		siteElement.appendChild(sizeElement);
		
		// Get file path
		//String filePath = file.getPath();
		
		Element clientPathElement = doc.createElement("client_path");
		clientPathElement.appendChild(doc.createTextNode(filePathClient));
		siteElement.appendChild(clientPathElement);
		
		Element serverPathElement = doc.createElement("server_path");
		serverPathElement.appendChild(doc.createTextNode(filePathServer));
		siteElement.appendChild(serverPathElement);

		//------------------------------------------------------------------------
		// *POSITION* DATASTREAM
		// datastream element
		if (positionSettingsClose){
			Element positionDatastream = doc.createElement("foxml:datastream");
			rootElement.appendChild(positionDatastream);
			
			// set attributes
			positionDatastream.setAttribute("ID", "POSITION");
			positionDatastream.setAttribute("STATE", "A");
			positionDatastream.setAttribute("CONTROL_GROUP", "X");
			
			// datastreamVersion element
			Element positionDatastreamVersion = doc.createElement("foxml:datastreamVersion");
			positionDatastream.appendChild(positionDatastreamVersion);
			// property attributes
			positionDatastreamVersion.setAttribute("ID", "POSITION1.0");
			positionDatastreamVersion.setAttribute("MIMETYPE", "text/xml");
			positionDatastreamVersion.setAttribute("LABEL", "Position info about this object");
			
			// xml Content element
			Element xmlContent4 = doc.createElement("foxml:xmlContent");
			positionDatastreamVersion.appendChild(xmlContent4);
			
			// Add the position Field
			xmlContent4.appendChild(positionField);
		}

		//------------------------------------------------------------------------
		// *CALIBRATION* DATASTREAM
		// datastream element
		if (calibrationSettingsClose){
			Element calibrationDatastream = doc.createElement("foxml:datastream");
			rootElement.appendChild(calibrationDatastream);
			
			// set attributes
			calibrationDatastream.setAttribute("ID", "CALIBRATION");
			calibrationDatastream.setAttribute("STATE", "A");
			calibrationDatastream.setAttribute("CONTROL_GROUP", "X");
			
			// datastreamVersion element
			Element calibrationDatastreamVersion = doc.createElement("foxml:datastreamVersion");
			calibrationDatastream.appendChild(calibrationDatastreamVersion);
			// property attributes
			calibrationDatastreamVersion.setAttribute("ID", "CALIBRATION1.0");
			calibrationDatastreamVersion.setAttribute("MIMETYPE", "text/xml");
			calibrationDatastreamVersion.setAttribute("LABEL", "Calibration info about this object");
			
			// xml Content element
			Element xmlContent4 = doc.createElement("foxml:xmlContent");
			calibrationDatastreamVersion.appendChild(xmlContent4);
			
			// Add the calibration Field
			xmlContent4.appendChild(calibrationField);
		}
		//------------------------------------------------------------------------
		
		
		// write the content into xml file
		TransformerFactory transformerFactory = TransformerFactory.newInstance();
		Transformer transformer = transformerFactory.newTransformer();
		DOMSource source = new DOMSource(doc);
		StreamResult result = new StreamResult(new File(outputfilePath));
 
		transformer.transform(source, result);
		
	  } catch (ParserConfigurationException pce) {
		pce.printStackTrace();
	  } catch (TransformerException tfe) {
		tfe.printStackTrace();
	  }
		//dc.appendChild(positionField);
		//positionSettingsClose = false;
		//System.out.println("positionSettingsClose");
	
	}
	
    /**
     * Converts Country and Sites to camel case
     * @param sentence String
    */
	public static String toCamelCase(String sentence) {
	    String out = "";
	    String[] splitSentence = sentence.split(" ");
	    String temp;
	    
	    for (int i=0; i<splitSentence.length; i++){
	    	if (i==0){
		    	temp = splitSentence[i];
		    	out += temp;
	    	}
	    	else{
		    	temp = splitSentence[i];
		    	temp = temp.substring(0, 1).toUpperCase() + temp.substring(1);	
		    	out += temp;
	    	}
	    }
	    return out;
	}
	
    /**
     * Write Country Collection XML file
     * @param filePath String
     * @param collectionPID String
     * @param country String
    */
	public static void writeXMLCountryCollectionFile(String filePath, String collectionPID, String country) {
		
		  try {
	 
			DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
	 
			// root elements
			Document doc = docBuilder.newDocument();
			Element rootElement = doc.createElement("foxml:digitalObject");
			doc.appendChild(rootElement);
			
			// set attributes of root element
			rootElement.setAttribute("VERSION", "1.1");
			rootElement.setAttribute("PID", collectionPID);
			rootElement.setAttribute("xmlns:foxml", "info:fedora/fedora-system:def/foxml#");
			rootElement.setAttribute("xmlns:rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
			rootElement.setAttribute("xmlns:rel", "http://www.nsdl.org/ontologies/relationships#");
			rootElement.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
			rootElement.setAttribute("xsi:schemaLocation", "info:fedora/fedora-system:def/foxml# http://www.fedora.info/definitions/1/0/foxml1-1.xsd");
			//rootElement.setAttribute("xmlns:rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
			//rootElement.setAttribute("xmlns:rel", "http://www.nsdl.org/ontologies/relationships#");
	 
			// Object Properties element
			Element objectProperties = doc.createElement("foxml:objectProperties");
			rootElement.appendChild(objectProperties);
	 
			// property element
			Element property = doc.createElement("foxml:property");
			objectProperties.appendChild(property);
			// property attributes
			property.setAttribute("NAME", "info:fedora/fedora-system:def/model#state");
			property.setAttribute("VALUE", "Active");
			
			// property element
			Element property2 = doc.createElement("foxml:property");
			objectProperties.appendChild(property2);
			// property attributes
			property2.setAttribute("NAME", "info:fedora/fedora-system:def/model#label");
			// **Title**
			property2.setAttribute("VALUE", country+" Collection");
			
			//------------------------------------------------------------------------
			// *Dublin Core*
			
			// datastream element
			Element datastream = doc.createElement("foxml:datastream");
			rootElement.appendChild(datastream);
			// set attributes
			datastream.setAttribute("ID", "DC");
			datastream.setAttribute("STATE", "A");
			datastream.setAttribute("CONTROL_GROUP", "X");
			
			// datastreamVersion element
			Element datastreamVersion = doc.createElement("foxml:datastreamVersion");
			datastream.appendChild(datastreamVersion);
			// property attributes
			datastreamVersion.setAttribute("FORMAT_URI", "http://www.openarchives.org/OAI/2.0/oai_dc/");
			datastreamVersion.setAttribute("ID", "DC1.0");
			datastreamVersion.setAttribute("MIMETYPE", "text/xml");
			datastreamVersion.setAttribute("LABEL", "Dublin Core Record for this object");
			
			// xml Content element
			Element xmlContent = doc.createElement("foxml:xmlContent");
			datastreamVersion.appendChild(xmlContent);
			
			// dc element
			Element dc = doc.createElement("oai_dc:dc");
			xmlContent.appendChild(dc);
			// property attributes
			dc.setAttribute("xmlns:dc", "http://purl.org/dc/elements/1.1/");
			dc.setAttribute("xmlns:oai_dc", "http://www.openarchives.org/OAI/2.0/oai_dc/");
				
			//fieldName.replaceAll("\\s+","")
			Element field = doc.createElement("dc:title");
			field.appendChild(doc.createTextNode(country+" Collection"));
			dc.appendChild(field);
			
			Element field2 = doc.createElement("dc:identifier");
			field2.appendChild(doc.createTextNode(collectionPID));
			dc.appendChild(field2);
			
			Element field3 = doc.createElement("dc:spatial");
			field3.appendChild(doc.createTextNode(country));
			dc.appendChild(field3);

			//------------------------------------------------------------------------
			// *RDF**
			// datastream element
			Element rdfDatastream = doc.createElement("foxml:datastream");
			rootElement.appendChild(rdfDatastream);
			// set attributes
			rdfDatastream.setAttribute("ID", "RELS-EXT");
			rdfDatastream.setAttribute("STATE", "A");
			rdfDatastream.setAttribute("CONTROL_GROUP", "X");
			//rdfDatastream.setAttribute("VERSIONABLE", "true");
			
			// datastreamVersion element
			Element rdfDatastreamVersion = doc.createElement("foxml:datastreamVersion");
			rdfDatastream.appendChild(rdfDatastreamVersion);
			// property attributes
			rdfDatastreamVersion.setAttribute("FORMAT_URI", "info:fedora/fedora-system:FedoraRELSExt-1.0");
			rdfDatastreamVersion.setAttribute("ID", "RELS-EXT.0");
			rdfDatastreamVersion.setAttribute("MIMETYPE", "application/rdf+xml");
			rdfDatastreamVersion.setAttribute("LABEL", "RDF Statements about this object");
						
			// xml Content element
			/*Element xmlContent2 = doc.createElement("foxml:contentDigest");
			rdfDatastreamVersion.appendChild(xmlContent2);
			
			xmlContent2.setAttribute("DIGEST", "none");
			xmlContent2.setAttribute("TYPE", "DISABLED");
			*/
			// xml Content digest element
			Element xmlContent2 = doc.createElement("foxml:xmlContent");
			rdfDatastreamVersion.appendChild(xmlContent2);

			
			// rdf element
			Element rdf = doc.createElement("rdf:RDF");
			xmlContent2.appendChild(rdf);
			// property attributes

			rdf.setAttribute("xmlns:fedora-model", "info:fedora/fedora-system:def/model#");
			rdf.setAttribute("xmlns:rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
			rdf.setAttribute("xmlns:rel", "info:fedora/fedora-system:def/relations-external#");
			
			//rdf.setAttribute("xmlns:rdfs", "http://www.w3.org/2000/01/rdf-schema#");
			//rdf.setAttribute("xmlns:fedora", "info:fedora/fedora-system:def/relations-external#");
			//rdf.setAttribute("xmlns:myns", "http://www.nsdl.org/ontologies/relationships#");
			//rdf.setAttribute("xmlns:dc", "http://purl.org/dc/elements/1.1/");
			//rdf.setAttribute("xmlns:oai_dc", "http://www.openarchives.org/OAI/2.0/oai_dc/");
				
			// rdf Description element
			Element rdfDescription = doc.createElement("rdf:Description");
			rdf.appendChild(rdfDescription);
			// property attributes
			rdfDescription.setAttribute("rdf:about", "info:fedora/"+collectionPID);
			
			// rel element
			Element relmemberOf = doc.createElement("rel:isMemberOf");
			rdfDescription.appendChild(relmemberOf);
			relmemberOf.setAttribute("rdf:resource", "info:fedora/zamaniProjectCollection:1");
			
			Element relCollection = doc.createElement("rel:isCollection");
			relCollection.appendChild(doc.createTextNode("true"));
			rdfDescription.appendChild(relCollection);

			//------------------------------------------------------------------------
			
			
			// write the content into xml file
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source2 = new DOMSource(doc);
			StreamResult result = new StreamResult(new File(filePath));
	 
			transformer.transform(source2, result);
	 
		  } catch (ParserConfigurationException pce) {
			pce.printStackTrace();
		  } catch (TransformerException tfe) {
			tfe.printStackTrace();
		  }
		}
	
    /**
     * Write Site Collection XML file
     * @param filePath String
     * @param collectionPID String
     * @param countryCollectionPID String
     * @param siteName String
     * @param siteLatitude String
     * @param siteLongitude String
     * @param country String
    */
	public static void writeXMLSiteCollectionFile(String filePath, String collectionPID, String countryCollectionPID, String siteName, String siteLatitude, String siteLongitude, String country) {
		
		  try {
	 
			DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
	 
			// root elements
			Document doc = docBuilder.newDocument();
			Element rootElement = doc.createElement("foxml:digitalObject");
			doc.appendChild(rootElement);
			
			// set attributes of root element
			rootElement.setAttribute("VERSION", "1.1");
			rootElement.setAttribute("PID", collectionPID);
			rootElement.setAttribute("xmlns:foxml", "info:fedora/fedora-system:def/foxml#");
			rootElement.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
			rootElement.setAttribute("xsi:schemaLocation", "info:fedora/fedora-system:def/foxml# http://www.fedora.info/definitions/1/0/foxml1-1.xsd");
			rootElement.setAttribute("xmlns:rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
			rootElement.setAttribute("xmlns:rel", "http://www.nsdl.org/ontologies/relationships#");
	 
			// Object Properties element
			Element objectProperties = doc.createElement("foxml:objectProperties");
			rootElement.appendChild(objectProperties);
	 
			// property element
			Element property = doc.createElement("foxml:property");
			objectProperties.appendChild(property);
			// property attributes
			property.setAttribute("NAME", "info:fedora/fedora-system:def/model#state");
			property.setAttribute("VALUE", "Active");
			
			// property element
			Element property2 = doc.createElement("foxml:property");
			objectProperties.appendChild(property2);
			// property attributes
			property2.setAttribute("NAME", "info:fedora/fedora-system:def/model#label");
			// **Title**
			property2.setAttribute("VALUE", siteName+" Collection");
			
			//------------------------------------------------------------------------
			// *Dublin Core*
			
			// datastream element
			Element datastream = doc.createElement("foxml:datastream");
			rootElement.appendChild(datastream);
			// set attributes
			datastream.setAttribute("ID", "DC");
			datastream.setAttribute("STATE", "A");
			datastream.setAttribute("CONTROL_GROUP", "X");
			
			// datastreamVersion element
			Element datastreamVersion = doc.createElement("foxml:datastreamVersion");
			datastream.appendChild(datastreamVersion);
			// property attributes
			datastreamVersion.setAttribute("FORMAT_URI", "http://www.openarchives.org/OAI/2.0/oai_dc/");
			datastreamVersion.setAttribute("ID", "DC1.0");
			datastreamVersion.setAttribute("MIMETYPE", "text/xml");
			datastreamVersion.setAttribute("LABEL", "Dublin Core Record for this object");
			
			// xml Content element
			Element xmlContent = doc.createElement("foxml:xmlContent");
			datastreamVersion.appendChild(xmlContent);
			
			// dc element
			Element dc = doc.createElement("oai_dc:dc");
			xmlContent.appendChild(dc);
			// property attributes
			dc.setAttribute("xmlns:dc", "http://purl.org/dc/elements/1.1/");
			dc.setAttribute("xmlns:oai_dc", "http://www.openarchives.org/OAI/2.0/oai_dc/");
				
			//fieldName.replaceAll("\\s+","")
			Element field = doc.createElement("dc:title");
			field.appendChild(doc.createTextNode(siteName+" Collection"));
			dc.appendChild(field);
			
			Element field2 = doc.createElement("dc:identifier");
			field2.appendChild(doc.createTextNode(collectionPID));
			dc.appendChild(field2);
			
			Element field3 = doc.createElement("dc:spatial");
			field3.appendChild(doc.createTextNode(country));
			field3.appendChild(doc.createTextNode(siteName));
			dc.appendChild(field3);

			//------------------------------------------------------------------------
			// *RDF**
			// datastream element
			Element rdfDatastream = doc.createElement("foxml:datastream");
			rootElement.appendChild(rdfDatastream);
			// set attributes
			rdfDatastream.setAttribute("ID", "RELS-EXT");
			rdfDatastream.setAttribute("STATE", "A");
			rdfDatastream.setAttribute("CONTROL_GROUP", "X");
			//rdfDatastream.setAttribute("VERSIONABLE", "true");
			
			// datastreamVersion element
			Element rdfDatastreamVersion = doc.createElement("foxml:datastreamVersion");
			rdfDatastream.appendChild(rdfDatastreamVersion);
			// property attributes
			rdfDatastreamVersion.setAttribute("FORMAT_URI", "info:fedora/fedora-system:FedoraRELSExt-1.0");
			rdfDatastreamVersion.setAttribute("ID", "RELS-EXT.0");
			rdfDatastreamVersion.setAttribute("MIMETYPE", "application/rdf+xml");
			rdfDatastreamVersion.setAttribute("LABEL", "RDF Statements about this object");
						
			// xml Content digest element
			Element xmlContent2 = doc.createElement("foxml:xmlContent");
			rdfDatastreamVersion.appendChild(xmlContent2);

			
			// rdf element
			Element rdf = doc.createElement("rdf:RDF");
			xmlContent2.appendChild(rdf);
			// property attributes

			rdf.setAttribute("xmlns:fedora-model", "info:fedora/fedora-system:def/model#");
			rdf.setAttribute("xmlns:rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
			rdf.setAttribute("xmlns:rel", "info:fedora/fedora-system:def/relations-external#");
			
			// rdf Description element
			Element rdfDescription = doc.createElement("rdf:Description");
			rdf.appendChild(rdfDescription);
			// property attributes
			rdfDescription.setAttribute("rdf:about", "info:fedora/"+collectionPID);
			
			
			// rel element
			Element relmemberOf = doc.createElement("rel:isMemberOf");
			rdfDescription.appendChild(relmemberOf);
			relmemberOf.setAttribute("rdf:resource", "info:fedora/"+countryCollectionPID);
			
			Element relCollection = doc.createElement("rel:isCollection");
			relCollection.appendChild(doc.createTextNode("true"));
			rdfDescription.appendChild(relCollection);
			//------------------------------------------------------------------------
			
			//------------------------------------------------------------------------
			// *SITE* DATASTREAM
			// datastream element
			Element siteDatastream = doc.createElement("foxml:datastream");
			rootElement.appendChild(siteDatastream);
			
			// set attributes
			siteDatastream.setAttribute("ID", "SITE");
			siteDatastream.setAttribute("STATE", "A");
			siteDatastream.setAttribute("CONTROL_GROUP", "X");
			
			// datastreamVersion element
			Element siteDatastreamVersion = doc.createElement("foxml:datastreamVersion");
			siteDatastream.appendChild(siteDatastreamVersion);
			// property attributes
			siteDatastreamVersion.setAttribute("ID", "SITE1.0");
			siteDatastreamVersion.setAttribute("MIMETYPE", "text/xml");
			siteDatastreamVersion.setAttribute("LABEL", "Site info about this object");
			
			// xml Content digest element
			Element xmlContent3 = doc.createElement("foxml:xmlContent");
			siteDatastreamVersion.appendChild(xmlContent3);
			
			// rdf Description element
			Element siteElement = doc.createElement("site");
			xmlContent3.appendChild(siteElement);
			
			Element countryElement = doc.createElement("country");
			countryElement.appendChild(doc.createTextNode(country));
			siteElement.appendChild(countryElement);
			
			Element siteNameElement = doc.createElement("name");
			siteNameElement.appendChild(doc.createTextNode(siteName));
			siteElement.appendChild(siteNameElement);
			
			Element positionElement = doc.createElement("position");
			siteElement.appendChild(positionElement);
			
			Element siteLatitudeElement = doc.createElement("latitude");
			siteLatitudeElement.appendChild(doc.createTextNode(siteLatitude));
			positionElement.appendChild(siteLatitudeElement);
			
			Element siteLongitudeElement = doc.createElement("longitude");
			siteLongitudeElement.appendChild(doc.createTextNode(siteLongitude));
			positionElement.appendChild(siteLongitudeElement);
			
			//------------------------------------------------------------------------
			
			// write the content into xml file
			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source2 = new DOMSource(doc);
			StreamResult result = new StreamResult(new File(filePath));
	 
			transformer.transform(source2, result);
	 
		  } catch (ParserConfigurationException pce) {
			pce.printStackTrace();
		  } catch (TransformerException tfe) {
			tfe.printStackTrace();
		  }
		}
	
}